Package: explodemap
Title: Hierarchical Exploded-View Cartography
Version: 0.2.0
Authors@R: 
    person("George", "Arthur", , "prigasgenthian48@gmail.com", role = c("aut", "cre"))
Description: Tools for generating hierarchical exploded-view maps from
    dense administrative boundary data. The package applies rigid-body
    translations to polygon geometries using a centroid-driven vector
    field, preserving the internal geometry of each feature while separating
    units within and across regions. Parameters can be derived
    analytically from dataset geometry using closed-form models for
    regional separation and local expansion. The package also includes
    grouped layouts, optional bounded collision refinement, and an
    interactive focus-map widget for selected-area inspection in
    'htmlwidgets' and 'Shiny'. It implements the methodology described in
    George Arthur (2026) <https://github.com/PrigasG/explodemap> "A
    Hierarchical Vector-Based Framework for Multi-Scale Exploded-View
    Cartography".
License: MIT + file LICENSE
URL: https://prigasg.github.io/explodemap/,
        https://github.com/PrigasG/explodemap
BugReports: https://github.com/PrigasG/explodemap/issues
Depends: R (>= 4.1.0)
Imports: dplyr, ggplot2, htmlwidgets, purrr, rlang, sf, utils
Suggests: knitr, rmarkdown, shiny, testthat (>= 3.0.0), tigris
VignetteBuilder: knitr
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-05-20 20:17:52 UTC; priga
Author: George Arthur [aut, cre]
Maintainer: George Arthur <prigasgenthian48@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-27 20:10:29 UTC
Built: R 4.5.2; ; 2026-05-27 20:54:17 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,exploded_map)
S3method(plot,grouped_exploded_map)
S3method(print,exploded_map)
S3method(print,grouped_exploded_map)
S3method(summary,exploded_map)
S3method(summary,grouped_exploded_map)
export(cache_clear)
export(cache_list)
export(calibration_row)
export(compute_stats)
export(derive_params)
export(estimate_block_radii)
export(explode_grouped)
export(explode_section)
export(explode_sf)
export(explode_sf_with_lookup)
export(explode_state)
export(export_topojson)
export(focus_map)
export(focus_map_preset)
export(focusmapOutput)
export(layout_regions)
export(renderFocusmap)
importFrom(rlang,":=")
importFrom(rlang,.data)

# explodemap <img src="man/figures/logo.png" alt="explodemap logo" align="right" height="139"/>

<!-- badges: start -->

[![R-CMD-check](https://img.shields.io/badge/R--CMD--check-passing-brightgreen)](https://github.com/PrigasG/explodemap)
[![License:
MIT](https://img.shields.io/badge/License-MIT-blue.svg)](https://opensource.org/licenses/MIT)

<!-- badges: end -->

`explodemap` provides tools for hierarchical exploded-view cartography
of dense administrative boundary data.

The package generates exploded maps by applying rigid-body translations
to polygon geometries, separating units within and across regions while
preserving each feature's internal geometry exactly. It supports both
the two-level core workflow described in the paper and a three-level
grouped layout extension for larger multi-region or national displays.

The methodology implemented here is described in:

> Arthur, G. *A Hierarchical Vector-Based Framework for Multi-Scale
> Exploded-View Cartography: Centroid-Driven Spatial Displacement for
> Dense Administrative Maps.*

## Installation

``` r
# Install from a local package source directory
devtools::install_local("path/to/explodemap")

# Or install from a tarball
install.packages("explodemap_0.2.0.tar.gz", repos = NULL, type = "source")
```

## What the package does

`explodemap` supports four main workflows:

-   explode any projected sf polygon dataset using a grouping column
-   explode U.S. municipal or county subdivision data directly from
    TIGER/Line via `explode_state()`
-   generate three-level grouped layouts for national or multi-region
    displays using `explode_grouped()`
-   add interactive selected-area focus, labels, and information cards
    using `focus_map()` in htmlwidgets or Shiny

The package also includes analytical parameter derivation, cross-dataset
calibration helpers, optional bounded collision refinement for dense
municipal cores, and optional TopoJSON export for downstream tools such
as Power BI. Shiny workflows are supported with `focusmapOutput()`,
`renderFocusmap()`, stable selection events, information cards, and
`quiet = TRUE` options on geometry builders to keep server logs clean.

For a compact overview, see the
[workflow guide](https://prigasg.github.io/explodemap/articles/workflow-guide.html).

## Quick start

``` r
library(sf)
library(explodemap)

sq <- function(xmin, ymin, size = 1000) {
  st_polygon(list(matrix(
    c(
      xmin, ymin,
      xmin + size, ymin,
      xmin + size, ymin + size,
      xmin, ymin + size,
      xmin, ymin
    ),
    ncol = 2,
    byrow = TRUE
  )))
}

geom <- st_sfc(
  sq(0, 0), sq(3000, 0),      # Region A
  sq(12000, 0), sq(15000, 0), # Region B
  crs = 3857
)

x <- st_sf(
  id = c("a1", "a2", "b1", "b2"),
  region = c("A", "A", "B", "B"),
  geometry = geom
)

result <- explode_sf(x, region_col = "region", plot = FALSE)

print(result)
plot(result, "both")
```

Inside Shiny, keep geometry computation quiet and render the returned
object explicitly:

``` r
result <- explode_sf(x, region_col = "region", plot = FALSE, quiet = TRUE)
focus_map(result, label_col = "id", group_col = "region")
```

## Public dashboards

For public Shiny dashboards, pre-cache or cache boundary downloads where
possible, and keep map computation out of automatic plotting paths:

``` r
result <- explode_sf(x, region_col = "region", plot = FALSE, quiet = TRUE)
```

When a dashboard downloads live TIGER/Line or `tigris` data, wrap the load
and map-generation steps in Shiny validation so users see a plain message
instead of a stack trace:

``` r
safe_map <- reactive({
  tryCatch(
    explode_sf(x(), region_col = "region", plot = FALSE, quiet = TRUE),
    error = function(e) {
      validate(need(FALSE, conditionMessage(e)))
    }
  )
})
```

Installed example apps in `inst/examples/` include this pattern. TopoJSON
export is optional and requires the external `mapshaper` command-line tool;
apps deployed to managed services should check for it before offering
exports.

## Core entry points

### Explode any projected `sf` object

``` r
result <- explode_sf(my_sf, region_col = "district")
```

### Explode a US state from TIGER/Line

``` r
nj <- explode_state(
  state_fips = "34", crs = 32118,
  region_map = list(
    North   = c("Bergen","Essex","Hudson","Morris","Passaic","Sussex","Union","Warren"),
    Central = c("Hunterdon","Mercer","Middlesex","Monmouth","Somerset"),
    South   = c("Atlantic","Burlington","Camden","Cape May","Cumberland",
                "Gloucester","Ocean","Salem")
  ),
  label = "New Jersey"
)
```

### Explode using an external lookup table

``` r
groups <- read.csv("region_assignments.csv")

result <- explode_sf_with_lookup(
  my_sf, join_col = "GEOID", lookup = groups,
  lookup_key = "geoid", region_col = "region"
)
```

### Filter a visible section before exploding

In Shiny dashboards and drill-down maps, it is often clearer to filter the
section the user selected before calling `explode_sf()`. The selected
subset can then use a more local grouping column while the all-region view
continues to use the broader region groups.

``` r
selected_region <- input$region_focus
if (is.null(selected_region) || !nzchar(selected_region)) {
  selected_region <- "all"
}

visible_sf <- if (identical(selected_region, "all")) {
  nj_counties
} else {
  nj_counties[nj_counties$region == selected_region, ]
}

visible_sf$explode_group <- if (identical(selected_region, "all")) {
  visible_sf$region
} else {
  visible_sf$county_name
}

exploded <- explode_sf(
  visible_sf,
  region_col = "explode_group",
  alpha_r = if (identical(selected_region, "all")) 26000 else 5250,
  alpha_l = if (identical(selected_region, "all")) 12500 else 0,
  refine = TRUE,
  refine_within = "all",
  plot = FALSE,
  quiet = TRUE
)

focus_map(
  exploded,
  label_col = "county_name",
  group_col = "region",
  info_cols = c("population_label", "section_rank", "region")
)
```

This pattern keeps a compact, focused map for section filters while still
using the same `explode_sf()` and `focus_map()` workflow as the full map.
Future package helpers may wrap this filter-first workflow directly.

## Grouped layouts

For larger layouts where region blocks need to be separated at an
additional level, use `explode_grouped()`:

``` r
result <- explode_grouped(
  states_sf, region_col = "hhs_region",
  mode = "auto_collision",
  label = "US by HHS Region"
)
```

Anchor modes:

-   `"auto"`: radial anchor placement
-   `"auto_collision"`: radial placement with iterative collision-aware
    refinement
-   `"manual"`: user-supplied positions

## Working with results

Two-level outputs are returned as exploded_map objects. Grouped layouts
are returned as grouped_exploded_map objects.

Common methods and helpers include:

``` r
print(result)           # Geometry stats and parameters
summary(result)         # Full diagnostic with implied gammas
plot(result)            # Exploded map
plot(result, "both")    # Original + exploded
calibration_row(result) # One-row data.frame for calibration tables
```

Grouped layouts also support:

``` r
plot(result, "all")       # original + local + grouped
```

## Interactive focus maps

`focus_map()` accepts raw `sf`, `exploded_map`, and
`grouped_exploded_map` objects. For exploded objects it automatically
uses the WGS84 displaced geometry, so the same result can be plotted,
exported, or rendered as an interactive focus map.

``` r
focus_map(
  result,
  label_col = "NAME",
  id_col = "GEOID",
  group_col = "region",
  group_palette = c(North = "#4C78A8", Central = "#F58518", South = "#54A24B"),
  info_cols = c("population", "median_income"),
  info_card_scale = 0.95
)
```

In Shiny, use the exported widget helpers. The widget emits a selection
value at `input$<outputId>_selected` containing the selected feature ID,
label, group, and properties.

``` r
ui <- fluidPage(
  focusmapOutput("map", height = "650px"),
  verbatimTextOutput("selected")
)

server <- function(input, output, session) {
  output$map <- renderFocusmap({
    focus_map(result, label_col = "NAME", id_col = "GEOID")
  })

  output$selected <- renderPrint(input$map_selected)
}
```

Use `group_palette` when a dashboard legend or data category already has
meaningful colours. Unmatched groups fall back to the built-in palette, so
apps can provide partial palettes while still rendering all groups.

For drill-down dashboards, use `explode_section()` to explode only the
selected section while keeping the rest of the geography as muted context:

``` r
focused <- explode_section(
  municipalities,
  section_col = "nj_region",
  section = "South",
  region_col = "county_name",
  alpha_r = 900,
  alpha_l = 600,
  plot = FALSE,
  quiet = TRUE
)

focus_map(
  focused,
  label_col = "NAME",
  id_col = "GEOID",
  context_col = ".explodemap_role",
  context_mode = "fade",
  context_opacity = 0.16
)
```

This pattern supports the "click a region, explode that region, keep the
state as context" workflow without forcing app code to manually stitch
focus and background geometries back together.

For dense municipal maps, tiny polygons can opt into a closer focus without
changing the default county-scale behavior:

``` r
focus_map(
  focused_municipalities,
  label_col = "NAME",
  id_col = "GEOID",
  min_focus_width = 115,
  min_focus_height = 88,
  tiny_feature_threshold = 52,
  tiny_feature_boost = 1.45,
  origin_context = "inset",
  origin_context_position = "bottom-left",
  focus_context_opacity = 0.14,
  show_drag_zoom = TRUE
)
```

## Mathematical guarantees

For the two-level core workflow, the package implements the analytical
parameter formulas described in the paper:

-   alpha_r = gamma_r \* w_bar / (2 \* sin(pi / n_regions))
-   alpha_l = gamma_l \* 2 \* R_local / sqrt(n_bar)

Default coefficients are:

-   gamma_r = 3.0
-   gamma_l = 1.136
-   p = 1.25

All geometric quantities other than the gamma coefficients are computed
from the dataset itself. The gamma coefficients are dimensionless
legibility constants calibrated on New Jersey and validated across
multiple U.S. states and a Canada example.

For the two-level core, the paper states three key properties:

| Property | Guarantee | Scope |
|--------------------------|----------------------------|------------------|
| **Proposition 1** | Internal geometry preserved exactly (rigid translation) | Per feature |
| **Proposition 2** | Radial ordering within regions preserved | Per region |
| **Proposition 3** | Max displacement bounded by α_r + α_l | Global |

The grouped three-level extension preserves structural grouping and
directional correspondence at higher levels rather than topological
coverage.

## Parameters

Two-level parameters derived automatically via Analytical Results 1–2:

-   `alpha_r = gamma_r * w_bar / (2 * sin(pi / n_regions))` — regional
    separation
-   `alpha_l = gamma_l * 2 * R_local / sqrt(n_bar)` — local expansion

Defaults: `gamma_r = 3.0`, `gamma_l = 1.136`, `p = 1.25`

All quantities except `gamma_r` and `gamma_l` are computed from the
dataset geometry. The gamma coefficients are dimensionless legibility
constants calibrated from the paper examples and intended as practical
defaults. You can override `alpha_r` and `alpha_l` independently when a
particular map needs more or less visual separation.

For very dense municipal cores, you can add a bounded collision-refinement
pass after the analytical displacement:

``` r
refined <- explode_sf(
  my_sf,
  region_col = "district",
  refine = TRUE,
  refine_min_gap = 250,
  refine_max_shift = 150
)
```

This optional layer nudges close same-region neighbors apart while capping
the extra correction per feature. Use `refine_within = "all"` if the
remaining crowding crosses region boundaries.

For app code, all main geometry workflows accept `quiet = TRUE`:

``` r
explode_sf(my_sf, "district", plot = FALSE, quiet = TRUE)
explode_sf_with_lookup(my_sf, "GEOID", lookup, plot = FALSE, quiet = TRUE)
explode_state("34", crs = 32118, region_map = regions, plot = FALSE, quiet = TRUE)
explode_grouped(my_sf, "district", mode = "auto_collision", plot = FALSE, quiet = TRUE)
layout_regions(my_sf, "district", mode = "auto_collision", quiet = TRUE)
```

## Examples

Small examples that run without external downloads are installed with
the package:

``` r
source(system.file("examples/basic_explode_sf.R", package = "explodemap"))
source(system.file("examples/collision_refinement.R", package = "explodemap"))
source(system.file("examples/lookup_workflow.R", package = "explodemap"))
source(system.file("examples/manual_parameter_tuning.R", package = "explodemap"))
```

Paper-scale examples that download public boundary data are also
available:

``` r
source(system.file("examples/run_calibration.R", package = "explodemap"))
source(system.file("examples/run_canada.R", package = "explodemap"))
source(system.file("examples/run_hhs.R", package = "explodemap"))
```

Interactive focus-map examples are installed as app scripts:

``` r
if (interactive()) {
  shiny::runApp(system.file("examples/focusmap_munis_app.R", package = "explodemap"))
  shiny::runApp(system.file("examples/focusmap_counties_app.R", package = "explodemap"))
}
```

## Export

`explodemap` can optionally export `TopoJSON` through the external
`mapshaper` command-line tool:

``` r
export_topojson(result, "exploded.topojson")
```

This is useful for downstream tools such as Power BI. To use it, install
mapshaper separately:

``` bash
npm install -g mapshaper
```

## Vignettes

``` r
vignette("getting-started", package = "explodemap")
vignette("grouped-layouts", package = "explodemap")
```

## Citation

If you use this package in academic work, please cite:

> Arthur, G. (2026). A hierarchical vector-based framework for
> multi-scale exploded-view cartography. *Working paper*.

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)

## ----setup--------------------------------------------------------------------
library(sf)
library(explodemap)

## ----synthetic-data-----------------------------------------------------------
sq <- function(xmin, ymin, size = 1000) {
  st_polygon(list(matrix(
    c(xmin, ymin,
      xmin + size, ymin,
      xmin + size, ymin + size,
      xmin, ymin + size,
      xmin, ymin),
    ncol = 2,
    byrow = TRUE
  )))
}

geom <- st_sfc(
  sq(0, 0), sq(3000, 0),       # Region A
  sq(12000, 0), sq(15000, 0),  # Region B
  crs = 3857
)

x <- st_sf(
  id     = c("a1", "a2", "b1", "b2"),
  region = c("A", "A", "B", "B"),
  geometry = geom
)

x

## ----explode-basic------------------------------------------------------------
result <- explode_sf(x, region_col = "region", plot = FALSE)

## ----explode-quiet------------------------------------------------------------
quiet_result <- explode_sf(x, region_col = "region", plot = FALSE, quiet = TRUE)
class(quiet_result)

## ----class-check--------------------------------------------------------------
class(result)
names(result)

## ----print--------------------------------------------------------------------
print(result)

## ----summary------------------------------------------------------------------
summary(result)

## ----plot-exploded------------------------------------------------------------
plot(result)

## ----plot-both, fig.width = 10------------------------------------------------
plot(result, "both")

## ----calibration--------------------------------------------------------------
calibration_row(result)

## ----manual-------------------------------------------------------------------
manual <- explode_sf(
  x,
  region_col = "region",
  alpha_r = 100,
  alpha_l = 200,
  plot = FALSE
)

manual$params

## ----partial-manual-----------------------------------------------------------
more_region_gap <- explode_sf(
  x,
  region_col = "region",
  alpha_r = result$params$alpha_r * 1.5,
  plot = FALSE
)

more_region_gap$params

## ----refine-example, eval=FALSE-----------------------------------------------
# refined <- explode_sf(
#   x,
#   region_col = "region",
#   refine = TRUE,
#   refine_min_gap = 0.15,
#   refine_max_shift = 0.10,
#   plot = FALSE
# )
# 
# refined$refinement

## ----refine, echo=FALSE-------------------------------------------------------
has_refinement <- "refine" %in% names(formals(explode_sf))

if (has_refinement) {
  refined <- explode_sf(
    x,
    region_col = "region",
    refine = TRUE,
    refine_min_gap = 0.15,
    refine_max_shift = 0.10,
    plot = FALSE
  )

  refined$refinement
} else {
  message("This installed explodemap build does not include refinement yet.")
}

## ----centroid-mode------------------------------------------------------------
pos <- explode_sf(
  x,
  region_col = "region",
  centroid_fun = "point_on_surface",
  plot = FALSE
)

## ----tiger-example, eval = FALSE----------------------------------------------
# nj <- explode_state(
#   state_fips = "34", crs = 32118,
#   region_map = list(
#     North   = c("Bergen", "Essex", "Hudson", "Morris",
#                 "Passaic", "Sussex", "Union", "Warren"),
#     Central = c("Hunterdon", "Mercer", "Middlesex",
#                 "Monmouth", "Somerset"),
#     South   = c("Atlantic", "Burlington", "Camden", "Cape May",
#                 "Cumberland", "Gloucester", "Ocean", "Salem")
#   ),
#   label = "New Jersey"
# )

## ----lookup-example, eval = FALSE---------------------------------------------
# groups <- read.csv("region_assignments.csv")
# 
# result <- explode_sf_with_lookup(
#   my_sf,
#   join_col   = "GEOID",
#   lookup     = groups,
#   lookup_key = "geoid",
#   region_col = "region"
# )

## ----export-example, eval = FALSE---------------------------------------------
# result <- explode_sf(
#   my_sf,
#   region_col = "region",
#   export = "output.geojson"
# )

## ----focus-map, eval=FALSE----------------------------------------------------
# focus_map(
#   result,
#   label_col = "id",
#   id_col = "id",
#   group_col = "region",
#   group_palette = c(A = "#4C78A8", B = "#F58518", C = "#54A24B"),
#   info_cols = c("id", "region"),
#   info_card_scale = 0.95
# )

## ----shiny-focus-map, eval=FALSE----------------------------------------------
# ui <- shiny::fluidPage(
#   focusmapOutput("map", height = "650px"),
#   shiny::verbatimTextOutput("selected")
# )
# 
# server <- function(input, output, session) {
#   exploded <- explode_sf(x, "region", plot = FALSE, quiet = TRUE)
# 
#   output$map <- renderFocusmap({
#     focus_map(
#       exploded,
#       label_col = "id",
#       id_col = "id",
#       group_col = "region",
#       group_palette = c(A = "#4C78A8", B = "#F58518", C = "#54A24B")
#     )
#   })
# 
#   output$selected <- shiny::renderPrint(input$map_selected)
# }

## ----section-focus-map, eval=FALSE--------------------------------------------
# focused <- explode_section(
#   x,
#   section_col = "region",
#   section = "A",
#   region_col = "county",
#   alpha_r = 900,
#   alpha_l = 600,
#   plot = FALSE,
#   quiet = TRUE
# )
# 
# focus_map(
#   focused,
#   label_col = "id",
#   context_col = ".explodemap_role",
#   context_mode = "fade"
# )

## ----tiny-feature-focus, eval=FALSE-------------------------------------------
# focus_map(
#   focused,
#   label_col = "id",
#   context_col = ".explodemap_role",
#   context_mode = "fade",
#   min_focus_width = 115,
#   min_focus_height = 88,
#   tiny_feature_threshold = 52,
#   tiny_feature_boost = 1.45,
#   origin_context = "inset",
#   origin_context_position = "bottom-left",
#   focus_context_opacity = 0.14,
#   show_drag_zoom = TRUE
# )

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(explodemap)
library(sf)
library(dplyr)

## ----nj-calibration, eval = FALSE---------------------------------------------
# nj <- explode_state(
#   state_fips = "34", crs = 32118,
#   region_map = list(
#     North   = c("Bergen", "Essex", "Hudson", "Morris",
#                 "Passaic", "Sussex", "Union", "Warren"),
#     Central = c("Hunterdon", "Mercer", "Middlesex",
#                 "Monmouth", "Somerset"),
#     South   = c("Atlantic", "Burlington", "Camden", "Cape May",
#                 "Cumberland", "Gloucester", "Ocean", "Salem")
#   ),
#   label = "New Jersey"
# )
# 
# summary(nj)

## ----pa-region-map------------------------------------------------------------
pa_region_map <- list(
  Southeast    = c("Philadelphia", "Delaware", "Chester",
                   "Montgomery", "Bucks"),
  Northeast    = c("Pike", "Monroe", "Carbon", "Northampton", "Lehigh",
                   "Luzerne", "Lackawanna", "Wayne", "Susquehanna",
                   "Wyoming", "Sullivan", "Columbia", "Montour",
                   "Schuylkill", "Berks", "Bradford"),
  Central      = c("Centre", "Clinton", "Lycoming", "Tioga", "Potter",
                   "Cameron", "Elk", "Clearfield", "Jefferson", "Indiana",
                   "Blair", "Huntingdon", "Mifflin", "Snyder", "Union",
                   "Northumberland", "Juniata", "Perry", "Dauphin",
                   "Lebanon"),
  SouthCentral = c("York", "Adams", "Lancaster", "Cumberland", "Franklin",
                   "Fulton", "Bedford", "Somerset", "Cambria"),
  Southwest    = c("Allegheny", "Westmoreland", "Fayette", "Greene",
                   "Washington", "Beaver", "Butler", "Armstrong",
                   "Lawrence"),
  Northwest    = c("Erie", "Crawford", "Mercer", "Venango", "Clarion",
                   "Forest", "Warren", "McKean")
)

## ----pa-transfer, eval = FALSE------------------------------------------------
# pa <- explode_state(
#   state_fips = "42", crs = 26918,
#   region_map = pa_region_map,
#   label = "Pennsylvania"
# )
# 
# summary(pa)

## ----pa-sensitivity, eval = FALSE---------------------------------------------
# alpha_l_canonical <- 12447
# alpha_r_canonical <- 20174
# 
# factors <- c(0.85, 0.90, 0.95, 1.00, 1.05, 1.10, 1.15)
# labels  <- c("-15%", "-10%", "-5%", "canonical", "+5%", "+10%", "+15%")
# 
# rows <- list()
# for (i in seq_along(factors)) {
#   run <- explode_state(
#     state_fips = "42", crs = 26918,
#     region_map = pa_region_map,
#     alpha_r = alpha_r_canonical,
#     alpha_l = round(alpha_l_canonical * factors[i]),
#     plot = FALSE, export = FALSE,
#     label = paste0("PA ", labels[i])
#   )
#   rows[[i]] <- calibration_row(run)
#   rows[[i]]$label <- labels[i]
#   rows[[i]]$factor <- factors[i]
# }
# 
# sensitivity_df <- bind_rows(rows)
# print(sensitivity_df)

## ----calibration, eval = FALSE------------------------------------------------
# source(system.file("registries/state_registry.R", package = "explodemap"))
# 
# calib_rows <- list()
# for (key in names(state_registry)) {
#   reg <- state_registry[[key]]
# 
#   result <- tryCatch(
#     explode_state(
#       state_fips = reg$fips, crs = reg$crs,
#       region_map = reg$region_map,
#       allow_other = TRUE, plot = FALSE,
#       label = reg$name
#     ),
#     error = function(e) {
#       message("ERROR: ", e$message)
#       NULL
#     }
#   )
# 
#   if (is.null(result)) next
#   calib_rows[[key]] <- calibration_row(result)
# }
# 
# calib_df <- bind_rows(calib_rows)
# print(calib_df)

## ----ohio, eval = FALSE-------------------------------------------------------
# oh <- explode_state(
#   state_fips = "39", crs = 32617,
#   region_map = list(
#     Northeast = c("Cuyahoga", "Summit", "Lorain", "Lake", "Medina",
#                   "Portage", "Geauga", "Ashtabula", "Trumbull", "Mahoning",
#                   "Columbiana", "Carroll", "Stark", "Wayne", "Holmes",
#                   "Harrison", "Jefferson"),
#     Northwest = c("Lucas", "Wood", "Fulton", "Williams", "Defiance",
#                   "Paulding", "Henry", "Putnam", "Hancock", "Sandusky",
#                   "Erie", "Ottawa", "Seneca", "Wyandot", "Crawford",
#                   "Huron", "Ashland", "Richland", "Morrow", "Knox",
#                   "Marion", "Hardin", "Logan", "Union", "Delaware",
#                   "Allen", "Van Wert", "Auglaize", "Shelby", "Mercer"),
#     Central   = c("Franklin", "Licking", "Fairfield", "Pickaway",
#                   "Madison", "Fayette", "Ross", "Clark", "Greene",
#                   "Montgomery", "Preble", "Darke", "Miami", "Champaign"),
#     Southwest = c("Hamilton", "Butler", "Warren", "Clermont", "Clinton",
#                   "Highland", "Brown", "Adams", "Scioto", "Lawrence",
#                   "Gallia", "Jackson", "Pike"),
#     Southeast = c("Belmont", "Monroe", "Washington", "Meigs", "Morgan",
#                   "Noble", "Guernsey", "Muskingum", "Perry", "Hocking",
#                   "Athens", "Tuscarawas", "Coshocton", "Vinton")
#   ),
#   label = "Ohio"
# )
# 
# summary(oh)
# plot(oh, "both")

## ----canada-regions-----------------------------------------------------------
province_regions <- data.frame(
  PRUID  = c("10", "11", "12", "13", "24", "35",
             "46", "47", "48", "59", "60", "61", "62"),
  region = c(rep("Atlantic", 4), "Quebec", "Ontario",
             rep("Prairies", 3), "Pacific",
             rep("Territories", 3)),
  stringsAsFactors = FALSE
)

## ----canada-download, eval = FALSE--------------------------------------------
# cache_file <- file.path(path.expand("~"), "explode_map_cache",
#                         "canada_csds_2021.rds")
# 
# if (file.exists(cache_file)) {
#   sf_raw <- readRDS(cache_file)
# } else {
#   url <- paste0(
#     "https://www12.statcan.gc.ca/census-recensement/2021/geo/sip-pis/",
#     "boundary-limites/files-fichiers/lcsd000b21a_e.zip"
#   )
#   tmp <- tempfile(fileext = ".zip")
#   download.file(url, tmp, mode = "wb")
#   dir <- file.path(tempdir(), "canada_csds")
#   dir.create(dir, showWarnings = FALSE)
#   unzip(tmp, exdir = dir)
#   shp <- list.files(dir, "\\.shp$", recursive = TRUE, full.names = TRUE)
#   sf_raw <- st_read(shp[1], quiet = TRUE)
#   dir.create(dirname(cache_file), showWarnings = FALSE, recursive = TRUE)
#   saveRDS(sf_raw, cache_file)
# }

## ----canada-explode, eval = FALSE---------------------------------------------
# sf_proj <- sf_raw |>
#   st_transform(3347) |>
#   left_join(province_regions, by = "PRUID")
# 
# sf_proj$region[is.na(sf_proj$region)] <- "Other"
# 
# sf_prov <- sf_proj |>
#   filter(region != "Territories")
# 
# canada <- explode_sf(
#   sf_prov,
#   region_col = "region",
#   allow_other = TRUE,
#   label = "Canada (provinces)"
# )
# 
# summary(canada)
# plot(canada, "both")

## ----hhs-lookup---------------------------------------------------------------
hhs_lookup <- data.frame(
  STUSPS = c(
    "CT", "ME", "MA", "NH", "RI", "VT",
    "NJ", "NY", "PR", "VI",
    "DE", "DC", "MD", "PA", "VA", "WV",
    "AL", "FL", "GA", "KY", "MS", "NC", "SC", "TN",
    "IL", "IN", "MI", "MN", "OH", "WI",
    "AR", "LA", "NM", "OK", "TX",
    "IA", "KS", "MO", "NE",
    "CO", "MT", "ND", "SD", "UT", "WY",
    "AZ", "CA", "HI", "NV", "GU", "AS", "MP",
    "AK", "ID", "OR", "WA"
  ),
  hhs_region = c(
    rep("1", 6), rep("2", 4), rep("3", 6), rep("4", 8),
    rep("5", 6), rep("6", 5), rep("7", 4), rep("8", 6),
    rep("9", 7), rep("10", 4)
  ),
  stringsAsFactors = FALSE
)

## ----hhs-download, eval = FALSE-----------------------------------------------
# cache_file <- file.path(path.expand("~"), "explode_map_cache",
#                         "us_states.rds")
# 
# if (file.exists(cache_file)) {
#   states_sf <- readRDS(cache_file)
# } else {
#   url <- "https://www2.census.gov/geo/tiger/TIGER2024/STATE/tl_2024_us_state.zip"
#   tmp <- tempfile(fileext = ".zip")
#   download.file(url, tmp, mode = "wb", quiet = TRUE)
#   dir <- file.path(tempdir(), "us_states")
#   dir.create(dir, showWarnings = FALSE)
#   unzip(tmp, exdir = dir)
#   shp <- list.files(dir, "\\.shp$", recursive = TRUE, full.names = TRUE)
#   states_sf <- st_read(shp[1], quiet = TRUE)
#   dir.create(dirname(cache_file), showWarnings = FALSE, recursive = TRUE)
#   saveRDS(states_sf, cache_file)
# }

## ----hhs-explode, eval = FALSE------------------------------------------------
# states_proj <- states_sf |>
#   st_transform(5070) |>
#   left_join(hhs_lookup, by = "STUSPS")
# 
# states_proj$hhs_region[is.na(states_proj$hhs_region)] <- "Other"
# 
# hhs <- explode_grouped(
#   states_proj,
#   region_col   = "hhs_region",
#   mode         = "auto_collision",
#   alpha_l      = 120000,
#   p            = 1.25,
#   kappa        = 1.8,
#   padding      = 80000,
#   delta        = 20000,
#   lambda       = 0.18,
#   eta          = 0.18,
#   padding_sep  = 30000,
#   max_iter     = 60,
#   label        = "US by HHS Region"
# )
# 
# print(hhs)
# plot(hhs, "all")

## ----session-info-------------------------------------------------------------
sessionInfo()

# =============================================================================
# Cross-State Calibration Example
# =============================================================================
# Runs explode_state() across multiple US states using the state registry,
# collects calibration rows, and reports gamma stability.
#
# Usage:
#   source(system.file("examples/run_calibration.R", package = "explodemap"))
# =============================================================================

library(explodemap)

# Load state registry
source(system.file("registries/state_registry.R", package = "explodemap"))

run_calibration <- function(states = names(state_registry),
                            gamma_r = 3.0, gamma_l = 1.136) {
  rows <- list()

  for (key in states) {
    reg <- state_registry[[key]]
    message("\n=== Processing: ", reg$name, " ===")

    result <- tryCatch(
      explode_state(
        state_fips = reg$fips, crs = reg$crs, region_map = reg$region_map,
        gamma_r = gamma_r, gamma_l = gamma_l,
        allow_other = TRUE, plot = FALSE, label = reg$name
      ),
      error = function(e) { message("  ERROR: ", e$message); NULL }
    )
    if (is.null(result)) next

    rows[[key]] <- calibration_row(result)
  }

  calib_df <- dplyr::bind_rows(rows)

  message("\n=== CALIBRATION TABLE ===")
  print(calib_df, row.names = FALSE)

  # Stability report
  known_r <- calib_df$gamma_r_implied[!is.na(calib_df$gamma_r_implied)]
  if (length(known_r) >= 2) {
    message("\ngamma_r across ", length(known_r), " states:")
    message("  values: ", paste(round(known_r, 3), collapse = ", "))
    message("  mean:   ", round(mean(known_r), 4))
    message("  CV:     ", round(sd(known_r) / mean(known_r), 4))
  }

  message("\nR_local/w_bar across all states:")
  message("  values: ", paste(round(calib_df$ratio, 2), collapse = ", "))
  message("  mean:   ", round(mean(calib_df$ratio, na.rm = TRUE), 2))

  invisible(calib_df)
}

# Run all configured states
calib <- run_calibration()

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)

## ----eval=FALSE---------------------------------------------------------------
# result <- explode_sf(
#   my_sf,
#   region_col = "region",
#   plot = FALSE,
#   quiet = TRUE
# )

## ----eval=FALSE---------------------------------------------------------------
# result <- explode_sf_with_lookup(
#   my_sf,
#   join_col = "GEOID",
#   lookup = region_lookup,
#   lookup_key = "geoid",
#   region_col = "region",
#   plot = FALSE,
#   quiet = TRUE
# )

## ----eval=FALSE---------------------------------------------------------------
# result <- explode_state(
#   state_fips = "34",
#   crs = 32118,
#   region_map = nj_regions,
#   plot = FALSE,
#   quiet = TRUE
# )

## ----eval=FALSE---------------------------------------------------------------
# result <- explode_sf(
#   my_sf,
#   region_col = "region",
#   refine = TRUE,
#   refine_min_gap = 250,
#   refine_max_shift = 150,
#   plot = FALSE
# )

## ----eval=FALSE---------------------------------------------------------------
# grouped <- explode_grouped(
#   my_sf,
#   region_col = "hhs_region",
#   mode = "auto_collision",
#   plot = FALSE,
#   quiet = TRUE
# )

## ----eval=FALSE---------------------------------------------------------------
# focus_map(
#   result,
#   group_col = "region",
#   group_palette = c(North = "#4C78A8", Central = "#F58518", South = "#54A24B"),
#   info_cols = c("NAME", "GEOID"),
#   focus_size = 0.76,
#   focus_padding = 40,
#   lift_scale = 1.16,
#   info_card_scale = 1.2
# )

## ----eval=FALSE---------------------------------------------------------------
# ui <- fluidPage(
#   focusmapOutput("map", height = "700px")
# )
# 
# server <- function(input, output, session) {
#   result <- explode_sf(munis, "region", plot = FALSE, quiet = TRUE)
# 
#   output$map <- renderFocusmap({
#     focus_map(
#       result,
#       label_col = "NAME",
#       id_col = "GEOID",
#       group_col = "region",
#       group_palette = c(North = "#4C78A8", Central = "#F58518", South = "#54A24B"),
#       info_cols = c("GEOID", "population")
#     )
#   })
# }

## ----eval=FALSE---------------------------------------------------------------
# focused <- explode_section(
#   munis,
#   section_col = "nj_region",
#   section = input$region,
#   region_col = "county_name",
#   alpha_r = 900,
#   alpha_l = 600,
#   plot = FALSE,
#   quiet = TRUE
# )
# 
# focus_map(
#   focused,
#   label_col = "NAME",
#   id_col = "GEOID",
#   context_col = ".explodemap_role",
#   context_mode = "fade",
#   context_opacity = 0.16,
#   performance_mode = TRUE
# )

## ----eval=FALSE---------------------------------------------------------------
# source(system.file("examples/basic_explode_sf.R", package = "explodemap"))
# source(system.file("examples/collision_refinement.R", package = "explodemap"))
# source(system.file("examples/lookup_workflow.R", package = "explodemap"))
# source(system.file("examples/manual_parameter_tuning.R", package = "explodemap"))

## ----eval=FALSE---------------------------------------------------------------
# if (interactive()) {
#   shiny::runApp(system.file("examples/focusmap_munis_app.R", package = "explodemap"))
#   shiny::runApp(system.file("examples/focusmap_counties_app.R", package = "explodemap"))
# }

library(shiny)
library(sf)
library(dplyr)
library(tigris)
library(explodemap)

options(tigris_use_cache = TRUE)

states <- data.frame(
  name = c("New Jersey", "Texas", "Georgia", "California", "Pennsylvania"),
  abbr = c("NJ", "TX", "GA", "CA", "PA"),
  stringsAsFactors = FALSE
)

app_error <- function(action, error) {
  paste0(
    "We could not ", action, ". ",
    conditionMessage(error),
    " Check your internet connection, try a smaller boundary file, or choose a different state."
  )
}

assign_quadrant_regions <- function(sf_obj) {
  centroids <- suppressWarnings(sf::st_centroid(sf_obj))
  coords <- sf::st_coordinates(centroids)
  cx <- stats::median(coords[, 1])
  cy <- stats::median(coords[, 2])
  dx <- coords[, 1] - cx
  dy <- coords[, 2] - cy

  sf_obj$region <- dplyr::case_when(
    dy >= 0 & abs(dy) >= abs(dx) ~ "North",
    dy <  0 & abs(dy) >= abs(dx) ~ "South",
    dx >= 0 & abs(dx) >  abs(dy) ~ "East",
    TRUE ~ "West"
  )
  sf_obj
}

load_munis <- function(state_abbr, cb = TRUE, year = 2024) {
  munis <- tigris::county_subdivisions(
    state = state_abbr,
    year = year,
    cb = cb,
    class = "sf"
  ) |>
    sf::st_make_valid() |>
    dplyr::filter(!sf::st_is_empty(.data$geometry))

  counties <- tigris::counties(
    state = state_abbr,
    year = year,
    cb = TRUE,
    class = "sf"
  ) |>
    sf::st_drop_geometry() |>
    dplyr::transmute(
      STATEFP = as.character(.data$STATEFP),
      COUNTYFP = as.character(.data$COUNTYFP),
      county_name = dplyr::coalesce(.data$NAMELSAD, .data$NAME)
    )

  out <- munis |>
    dplyr::mutate(
      STATEFP = as.character(.data$STATEFP),
      COUNTYFP = as.character(.data$COUNTYFP),
      COUSUBFP = as.character(.data$COUSUBFP)
    ) |>
    dplyr::left_join(counties, by = c("STATEFP", "COUNTYFP")) |>
    dplyr::mutate(
      muni_label = dplyr::coalesce(.data$NAMELSAD, .data$NAME),
      muni_id = paste0(.data$STATEFP, .data$COUNTYFP, .data$COUSUBFP),
      muni_geoid = .data$GEOID,
      county_name = dplyr::coalesce(.data$county_name, paste0("County ", .data$COUNTYFP))
    )

  if (!nrow(out)) {
    stop(
      "No county subdivisions were returned for ", state_abbr, ". ",
      "Try another state or turn off cartographic boundary files.",
      call. = FALSE
    )
  }

  out
}

ui <- fluidPage(
  tags$head(
    tags$style(HTML("
      .container-fluid { padding: 0; }
      .focus-shell { display: grid; grid-template-columns: 300px minmax(0, 1fr); height: 100vh; }
      .focus-controls { padding: 18px; border-right: 1px solid #dbe6f1; overflow-y: auto; background: #fff; }
      .focus-controls h2 { margin-top: 0; font-size: 20px; }
      .focus-map { min-width: 0; min-height: 0; padding: 12px; background: #f5f8fc; }
      .focus-map .html-widget, .focus-map .shiny-html-output { width: 100% !important; height: 100% !important; }
      @media (max-width: 900px) {
        .focus-shell { grid-template-columns: 1fr; grid-template-rows: auto minmax(560px, 1fr); height: auto; min-height: 100vh; }
        .focus-controls { border-right: 0; border-bottom: 1px solid #dbe6f1; }
        .focus-map { height: 75vh; }
      }
      .shiny-output-error-validation {
        color: #713f12;
        background: #fff7ed;
        border: 1px solid #fed7aa;
        border-radius: 6px;
        padding: 14px 16px;
        margin: 12px;
        font-size: 15px;
        line-height: 1.4;
      }
    "))
  ),
  div(
    class = "focus-shell",
    div(
      class = "focus-controls",
      h2("focus_map: county subdivisions"),
      selectInput("state", "State", choices = stats::setNames(states$abbr, states$name), selected = "TX"),
      checkboxInput("cb", "Use cartographic boundary file", value = TRUE),
      radioButtons("mode", "Mode", choices = c("Raw" = "raw", "Exploded by region" = "exploded"), inline = TRUE),
      sliderInput("font_size", "Focused label size", min = 4, max = 16, value = 8, step = 1),
      sliderInput("lift_scale", "Lift scale", min = 1.00, max = 1.35, value = 1.10, step = 0.01),
      sliderInput("focus_size", "Zoomed area size", min = 0.35, max = 1.30, value = 1.17, step = 0.01),
      sliderInput("focus_padding", "Focus padding", min = 12, max = 120, value = 56, step = 4),
      sliderInput("info_card_scale", "Info card size", min = 0.75, max = 1.60, value = 1.60, step = 0.05),
      checkboxInput("show_labels", "Show focused labels", value = TRUE),
      helpText("Zoomed area size controls how much of the viewport the selected area may occupy. Focus padding keeps the boundary away from the edge; lower it if a high zoomed-area size stops getting bigger. Exploded mode uses four spatial regions; county-level detail stays in the card.")
    ),
    div(
      class = "focus-map",
      focusmapOutput("map", height = "100%")
    )
  )
)

server <- function(input, output, session) {
  munis <- reactive({
    tryCatch(
      load_munis(input$state, cb = isTRUE(input$cb)),
      error = function(e) {
        validate(need(FALSE, app_error("load county subdivision boundaries", e)))
      }
    )
  }) |>
    bindCache(input$state, input$cb)

  map_data <- reactive({
    x <- munis()

    if (identical(input$mode, "exploded")) {
      validate(need(
        nrow(x) >= 3,
        "Exploded mode needs at least three mapped areas. Choose a state with more subdivisions or switch to Raw mode."
      ))

      result <- tryCatch(
        {
          projected <- sf::st_transform(x, 5070) |>
            assign_quadrant_regions()
          explode_sf(
            projected,
            region_col = "region",
            plot = FALSE,
            quiet = TRUE,
            label = paste(input$state, "county subdivisions")
          )
        },
        error = function(e) {
          validate(need(FALSE, app_error("create the exploded map", e)))
        }
      )

      list(x = result, label_col = NULL, group_col = "region")
    } else {
      list(x = x, label_col = "muni_label", group_col = NULL)
    }
  })

  output$map <- renderFocusmap({
    md <- map_data()
    tryCatch(
      focus_map(
        md$x,
        label_col = md$label_col,
        id_col = "muni_geoid",
        group_col = md$group_col,
        info_cols = c("muni_label", "county_name", "muni_geoid"),
        info_labels = c(muni_label = "Subdivision", county_name = "County", muni_geoid = "GEOID"),
        info_title = "muni_label",
        font_size = input$font_size,
        lift_scale = input$lift_scale,
        focus_size = input$focus_size,
        focus_padding = input$focus_padding,
        info_card_scale = input$info_card_scale,
        show_labels = isTRUE(input$show_labels),
        performance_mode = TRUE,
        width = "100%",
        height = "100%"
      ),
      error = function(e) {
        validate(need(FALSE, app_error("render the interactive map", e)))
      }
    )
  })
}

shinyApp(ui, server)

# =============================================================================
# Canada Validation Example
# =============================================================================
# Non-US validation of the exploded-view framework using Statistics Canada
# 2021 Census Subdivision (CSD) boundaries.
#
# Tests whether gamma_l = 1.136 and gamma_r = 3.0 produce coherent layouts
# outside the US administrative system.
#
# Usage:
#   source(system.file("examples/run_canada.R", package = "explodemap"))
# =============================================================================

library(explodemap)
library(sf)
library(dplyr)

# Province -> Region mapping (Statistics Canada standard)
province_regions <- data.frame(
  PRUID = c("10","11","12","13", "24", "35", "46","47","48", "59", "60","61","62"),
  region = c(rep("Atlantic", 4), "Quebec", "Ontario",
             rep("Prairies", 3), "Pacific", rep("Territories", 3)),
  stringsAsFactors = FALSE
)

# Download and preprocess
download_canada_csds <- function() {
  cache_file <- file.path(path.expand("~"), "explode_map_cache", "canada_csds_2021.rds")
  if (file.exists(cache_file)) return(readRDS(cache_file))

  url <- paste0("https://www12.statcan.gc.ca/census-recensement/2021/geo/sip-pis/",
                "boundary-limites/files-fichiers/lcsd000b21a_e.zip")
  tmp <- tempfile(fileext = ".zip")
  download.file(url, tmp, mode = "wb", quiet = FALSE)
  dir <- file.path(tempdir(), "canada_csds")
  dir.create(dir, showWarnings = FALSE)
  unzip(tmp, exdir = dir)
  shp <- list.files(dir, pattern = "\\.shp$", recursive = TRUE, full.names = TRUE)
  obj <- st_read(shp[1], quiet = TRUE)
  dir.create(dirname(cache_file), showWarnings = FALSE, recursive = TRUE)
  saveRDS(obj, cache_file)
  obj
}

run_canada <- function(exclude_territories = FALSE) {
  message("\n=== CANADA VALIDATION ===\n")

  sf_raw <- download_canada_csds()
  sf_proj <- sf_raw |> st_transform(3347)  # Statistics Canada Lambert

  # Attach regions via PRUID
  sf_reg <- sf_proj |>
    left_join(province_regions, by = "PRUID")
  sf_reg$region[is.na(sf_reg$region)] <- "Other"

  if (exclude_territories) {
    sf_reg <- sf_reg |> filter(region != "Territories")
    message("Territories excluded. Remaining: ", nrow(sf_reg), " units.")
  }

  # Use the core package API
  result <- explode_sf(
    sf_reg, region_col = "region",
    gamma_r = 3.0, gamma_l = 1.136,
    allow_other = TRUE,
    label = if (exclude_territories) "Canada (provinces)" else "Canada (all)"
  )

  message("\n--- Validation ---")
  message("R_local/w_bar = ", round(result$stats$ratio, 2),
          "  (US dense-municipal ref: ~15.8)")

  result
}

# Run
canada <- run_canada(exclude_territories = TRUE)

# State registry for cross-state calibration
# Used by inst/examples/run_calibration.R
# Each entry contains FIPS, CRS, region map, and known-good parameters (if any)

state_registry <- list(

  NJ = list(
    name = "New Jersey", fips = "34", crs = 32118,
    region_map = list(
      North   = c("Bergen","Essex","Hudson","Morris","Passaic","Sussex","Union","Warren"),
      Central = c("Hunterdon","Mercer","Middlesex","Monmouth","Somerset"),
      South   = c("Atlantic","Burlington","Camden","Cape May","Cumberland",
                   "Gloucester","Ocean","Salem")
    ),
    known_alpha_r = 6000, known_alpha_l = 10000
  ),

  PA = list(
    name = "Pennsylvania", fips = "42", crs = 26918,
    region_map = list(
      Southeast    = c("Philadelphia","Delaware","Chester","Montgomery","Bucks"),
      Northeast    = c("Pike","Monroe","Carbon","Northampton","Lehigh","Luzerne",
                       "Lackawanna","Wayne","Susquehanna","Wyoming","Sullivan",
                       "Columbia","Montour","Schuylkill","Berks","Bradford"),
      Central      = c("Centre","Clinton","Lycoming","Tioga","Potter","Cameron",
                       "Elk","Clearfield","Jefferson","Indiana","Blair",
                       "Huntingdon","Mifflin","Snyder","Union","Northumberland",
                       "Juniata","Perry","Dauphin","Lebanon"),
      SouthCentral = c("York","Adams","Lancaster","Cumberland","Franklin",
                       "Fulton","Bedford","Somerset","Cambria"),
      Southwest    = c("Allegheny","Westmoreland","Fayette","Greene",
                       "Washington","Beaver","Butler","Armstrong","Lawrence"),
      Northwest    = c("Erie","Crawford","Mercer","Venango","Clarion",
                       "Forest","Warren","McKean")
    ),
    known_alpha_r = 25000, known_alpha_l = NA
  ),

  OH = list(
    name = "Ohio", fips = "39", crs = 32617,
    region_map = list(
      Northeast = c("Cuyahoga","Summit","Lorain","Lake","Medina","Portage","Geauga",
                    "Ashtabula","Trumbull","Mahoning","Columbiana","Carroll","Stark",
                    "Wayne","Holmes","Harrison","Jefferson"),
      Northwest = c("Lucas","Wood","Fulton","Williams","Defiance","Paulding","Henry",
                    "Putnam","Hancock","Sandusky","Erie","Ottawa","Seneca","Wyandot",
                    "Crawford","Huron","Ashland","Richland","Morrow","Knox","Marion",
                    "Hardin","Logan","Union","Delaware","Allen","Van Wert","Auglaize",
                    "Shelby","Mercer"),
      Central   = c("Franklin","Licking","Fairfield","Pickaway","Madison","Fayette",
                    "Ross","Clark","Greene","Montgomery","Preble","Darke","Miami","Champaign"),
      Southwest = c("Hamilton","Butler","Warren","Clermont","Clinton","Highland",
                    "Brown","Adams","Scioto","Lawrence","Gallia","Jackson","Pike"),
      Southeast = c("Belmont","Monroe","Washington","Meigs","Morgan","Noble","Guernsey",
                    "Muskingum","Perry","Hocking","Athens","Tuscarawas","Coshocton","Vinton")
    ),
    known_alpha_r = NA, known_alpha_l = NA
  ),

  NY = list(
    name = "New York", fips = "36", crs = 32618,
    region_map = list(
      NYC          = c("New York","Kings","Queens","Bronx","Richmond"),
      LongIsland   = c("Nassau","Suffolk"),
      HudsonValley = c("Westchester","Rockland","Orange","Putnam","Dutchess",
                       "Ulster","Sullivan","Columbia","Greene"),
      Upstate      = c("Albany","Rensselaer","Schenectady","Saratoga","Washington",
                       "Warren","Hamilton","Fulton","Montgomery","Schoharie","Otsego",
                       "Delaware","Chenango","Broome","Tioga","Tompkins","Cortland",
                       "Onondaga","Oswego","Madison","Oneida","Herkimer","Lewis",
                       "Jefferson","St. Lawrence","Franklin","Clinton","Essex"),
      Western      = c("Monroe","Ontario","Wayne","Seneca","Cayuga","Yates","Schuyler",
                       "Steuben","Chemung","Livingston","Allegany","Cattaraugus",
                       "Chautauqua","Erie","Niagara","Orleans","Genesee","Wyoming")
    ),
    known_alpha_r = NA, known_alpha_l = NA
  )
)

# Basic explodemap workflow ----------------------------------------------------
#
# This example uses a tiny synthetic sf object, so it runs without downloads.
# It is the safest first example for users learning the package API.
#
# Usage:
#   source(system.file("examples/basic_explode_sf.R", package = "explodemap"))

library(explodemap)
library(sf)

square <- function(xmin, ymin, size = 1000) {
  st_polygon(list(matrix(
    c(
      xmin, ymin,
      xmin + size, ymin,
      xmin + size, ymin + size,
      xmin, ymin + size,
      xmin, ymin
    ),
    ncol = 2,
    byrow = TRUE
  )))
}

geom <- st_sfc(
  square(0, 0),
  square(3000, 0),
  square(12000, 0),
  square(15000, 0),
  crs = 3857
)

units <- st_sf(
  id = c("a1", "a2", "b1", "b2"),
  region = c("A", "A", "B", "B"),
  geometry = geom
)

result <- explode_sf(units, region_col = "region", plot = FALSE)

print(result)
summary(result)
plot(result, "both")

# Manual parameter tuning ------------------------------------------------------
#
# The default workflow derives alpha_r and alpha_l from geometry. Users can
# still override either one independently when a map needs a visual adjustment.
#
# Usage:
#   source(system.file("examples/manual_parameter_tuning.R", package = "explodemap"))

library(explodemap)
library(sf)

square <- function(xmin, ymin, size = 1000) {
  st_polygon(list(matrix(
    c(
      xmin, ymin,
      xmin + size, ymin,
      xmin + size, ymin + size,
      xmin, ymin + size,
      xmin, ymin
    ),
    ncol = 2,
    byrow = TRUE
  )))
}

geom <- st_sfc(
  square(0, 0),
  square(2000, 0),
  square(10000, 0),
  square(12000, 0),
  crs = 3857
)

units <- st_sf(
  id = c("a1", "a2", "b1", "b2"),
  region = c("A", "A", "B", "B"),
  geometry = geom
)

derived <- explode_sf(units, region_col = "region", plot = FALSE)

# Increase only regional separation; keep derived local expansion.
more_region_gap <- explode_sf(
  units,
  region_col = "region",
  alpha_r = derived$params$alpha_r * 1.5,
  plot = FALSE,
  label = "More regional gap"
)

# Increase only local expansion; keep derived regional separation.
more_local_gap <- explode_sf(
  units,
  region_col = "region",
  alpha_l = derived$params$alpha_l * 1.5,
  plot = FALSE,
  label = "More local gap"
)

print(derived)
print(more_region_gap)
print(more_local_gap)

# Optional collision refinement for dense municipal cores.
#
# Keep refine = FALSE for the pure analytical explodemap. Turn it on when
# dense same-region polygons still touch or visually crowd after displacement.
# This toy example sets alpha_r = alpha_l = 0 to isolate the refinement pass.

library(sf)
library(explodemap)

sq <- function(xmin, ymin, size = 1) {
  st_polygon(list(matrix(
    c(
      xmin, ymin,
      xmin + size, ymin,
      xmin + size, ymin + size,
      xmin, ymin + size,
      xmin, ymin
    ),
    ncol = 2,
    byrow = TRUE
  )))
}

dense_sf <- st_sf(
  id = c("a1", "a2", "a3", "b1", "b2", "b3"),
  region = c("A", "A", "A", "B", "B", "B"),
  geometry = st_sfc(
    sq(0, 0), sq(1, 0), sq(0, 1),
    sq(8, 0), sq(9, 0), sq(8, 1),
    crs = 3857
  )
)

result <- explode_sf(
  dense_sf,
  region_col = "region",
  alpha_r = 0,
  alpha_l = 0,
  refine = TRUE,
  refine_min_gap = 1.25,
  refine_max_shift = 0.5,
  refine_within = "region",
  plot = FALSE,
  label = "Dense example with refinement"
)

summary(result)
plot(result, "both")

# =============================================================================
# HHS Grouped Layout Example
# =============================================================================
# Three-level national grouped layout using HHS regions.
# Demonstrates auto and auto_collision anchor modes.
#
# Usage:
#   source(system.file("examples/run_hhs.R", package = "explodemap"))
# =============================================================================

library(explodemap)
library(sf)
library(dplyr)

# HHS Region assignments (50 states + DC + territories)
hhs_lookup <- data.frame(
  STUSPS = c(
    "CT","ME","MA","NH","RI","VT",             # Region 1
    "NJ","NY","PR","VI",                       # Region 2
    "DE","DC","MD","PA","VA","WV",             # Region 3
    "AL","FL","GA","KY","MS","NC","SC","TN",   # Region 4
    "IL","IN","MI","MN","OH","WI",             # Region 5
    "AR","LA","NM","OK","TX",                  # Region 6
    "IA","KS","MO","NE",                       # Region 7
    "CO","MT","ND","SD","UT","WY",             # Region 8
    "AZ","CA","HI","NV","GU","AS","MP",        # Region 9
    "AK","ID","OR","WA"                        # Region 10
  ),
  hhs_region = c(
    rep("1", 6), rep("2", 4), rep("3", 6), rep("4", 8),
    rep("5", 6), rep("6", 5), rep("7", 4), rep("8", 6),
    rep("9", 7), rep("10", 4)
  ),
  stringsAsFactors = FALSE
)

run_hhs <- function(mode = "auto_collision") {
  message("\n=== HHS GROUPED LAYOUT ===\n")

  # Download state boundaries (use TIGER/Line states)
  cache_file <- file.path(path.expand("~"), "explode_map_cache", "us_states.rds")
  if (file.exists(cache_file)) {
    states_sf <- readRDS(cache_file)
  } else {
    url <- "https://www2.census.gov/geo/tiger/TIGER2024/STATE/tl_2024_us_state.zip"
    tmp <- tempfile(fileext = ".zip")
    download.file(url, tmp, mode = "wb", quiet = TRUE)
    dir <- file.path(tempdir(), "us_states")
    dir.create(dir, showWarnings = FALSE)
    unzip(tmp, exdir = dir)
    shp <- list.files(dir, pattern = "\\.shp$", recursive = TRUE, full.names = TRUE)
    states_sf <- st_read(shp[1], quiet = TRUE)
    dir.create(dirname(cache_file), showWarnings = FALSE, recursive = TRUE)
    saveRDS(states_sf, cache_file)
  }

  # Project to Albers Equal Area and attach HHS regions
  states_proj <- states_sf |>
    st_transform(5070) |>
    left_join(hhs_lookup, by = "STUSPS")

  states_proj$hhs_region[is.na(states_proj$hhs_region)] <- "Other"

  # Run grouped explosion
  result <- explode_grouped(
    states_proj, region_col = "hhs_region",
    mode = mode,
    alpha_l = 120000, p = 1.25,
    kappa = 1.8, padding = 80000, delta = 20000,
    lambda = 0.18, eta = 0.18,
    padding_sep = 30000, max_iter = 60,
    label = paste("US by HHS Region (", mode, ")")
  )

  result
}

# Run with collision-aware anchors
hhs <- run_hhs("auto_collision")

# Lookup-table grouping workflow ----------------------------------------------
#
# Use this pattern when geometry and grouping information live in separate
# files. Unmatched rows are labelled "Other"; set allow_other = TRUE to keep
# them in the output.
#
# Usage:
#   source(system.file("examples/lookup_workflow.R", package = "explodemap"))

library(explodemap)
library(sf)

square <- function(xmin, ymin, size = 1000) {
  st_polygon(list(matrix(
    c(
      xmin, ymin,
      xmin + size, ymin,
      xmin + size, ymin + size,
      xmin, ymin + size,
      xmin, ymin
    ),
    ncol = 2,
    byrow = TRUE
  )))
}

geom <- st_sfc(
  square(0, 0),
  square(3000, 0),
  square(12000, 0),
  square(15000, 0),
  crs = 3857
)

units <- st_sf(
  geoid = c("001", "002", "003", "004"),
  name = c("Alpha", "Beta", "Gamma", "Delta"),
  geometry = geom
)

lookup <- data.frame(
  geoid = c("001", "002", "003"),
  planning_region = c("North", "North", "South"),
  stringsAsFactors = FALSE
)

result <- explode_sf_with_lookup(
  units,
  join_col = "geoid",
  lookup = lookup,
  lookup_key = "geoid",
  region_col = "planning_region",
  allow_other = TRUE,
  plot = FALSE,
  label = "Lookup workflow"
)

print(result)
plot(result, "both")

# =============================================================================
# R/section.R - Section-focus exploded maps
# =============================================================================

#' Explode one selected section and keep the rest as context
#'
#' `explode_section()` is a dashboard-oriented helper for exploratory maps.
#' It applies an exploded layout only to the requested section, then recombines
#' the untouched remainder of the layer as geographic context. The result can
#' be passed directly to [focus_map()] with `context_col` to fade or hide the
#' non-selected features.
#'
#' @param sf_obj Projected `sf` object.
#' @param section_col Column containing the high-level sections users choose
#'   from, such as `"North"`, `"Central"`, and `"South"`.
#' @param section Selected section value. Values in `all_values` explode the
#'   full layer.
#' @param region_col Column used for the explosion inside the selected section.
#'   Defaults to `section_col`. For municipality drill-downs, this is often a
#'   county column.
#' @param layout `"explode"` for [explode_sf()] or `"grouped"` for
#'   [explode_grouped()].
#' @param context `"fade"` or `"hide"` keeps non-selected features in the
#'   returned object as context; `"none"` drops them.
#' @param role_col Name of the role column added to the output. Focus features
#'   are marked `"focus"` and context features are marked `"context"`.
#' @param all_values Values that mean "all sections". Default `"all"`.
#' @param ... Passed to [explode_sf()] or [explode_grouped()].
#'
#' @return An `exploded_map` or `grouped_exploded_map` object with recombined
#'   focus/context geometry and extra diagnostics.
#' @export
#'
#' @examples
#' \donttest{
#' poly <- function(xmin, ymin, xmax, ymax) {
#'   sf::st_polygon(list(rbind(
#'     c(xmin, ymin), c(xmax, ymin), c(xmax, ymax),
#'     c(xmin, ymax), c(xmin, ymin)
#'   )))
#' }
#'
#' municipalities <- sf::st_sf(
#'   NAME = c("A", "B", "C", "D"),
#'   nj_region = c("South", "South", "North", "North"),
#'   county_name = c("Atlantic", "Cape May", "Bergen", "Hudson"),
#'   geometry = sf::st_sfc(
#'     poly(0, 0, 1000, 1000),
#'     poly(2000, 0, 3000, 1000),
#'     poly(0, 2000, 1000, 3000),
#'     poly(2000, 2000, 3000, 3000),
#'     crs = 3857
#'   )
#' )
#'
#' focused <- explode_section(
#'   municipalities,
#'   section_col = "nj_region",
#'   section = "South",
#'   region_col = "county_name",
#'   alpha_r = 1800,
#'   alpha_l = 1200,
#'   plot = FALSE,
#'   quiet = TRUE
#' )
#'
#' focus_map(
#'   focused,
#'   label_col = "NAME",
#'   context_col = ".explodemap_role",
#'   context_mode = "fade"
#' )
#' }
explode_section <- function(sf_obj,
                            section_col,
                            section,
                            region_col = section_col,
                            layout = c("explode", "grouped"),
                            context = c("fade", "hide", "none"),
                            role_col = ".explodemap_role",
                            all_values = "all",
                            ...) {
  layout <- match.arg(layout)
  context <- match.arg(context)

  if (!inherits(sf_obj, "sf")) {
    stop("`sf_obj` must be an sf object.", call. = FALSE)
  }
  if (!section_col %in% names(sf_obj)) {
    stop("section_col '", section_col, "' not found in sf_obj.", call. = FALSE)
  }
  if (!region_col %in% names(sf_obj)) {
    stop("region_col '", region_col, "' not found in sf_obj.", call. = FALSE)
  }

  section <- as.character(section %||% all_values[[1]])
  all_values <- as.character(all_values)
  is_all <- length(section) == 0 || section[[1]] %in% all_values

  if (is_all) {
    sf_obj[[role_col]] <- "focus"
    result <- .explode_section_dispatch(sf_obj, region_col, layout, ...)
    result$params$section_col <- section_col
    result$params$section <- all_values[[1]]
    result$params$context <- "none"
    result$params$role_col <- role_col
    result$diagnostics$section_col <- section_col
    result$diagnostics$section <- all_values[[1]]
    result$diagnostics$n_focus <- nrow(sf_obj)
    result$diagnostics$n_context <- 0L
    return(result)
  }

  section_values <- as.character(sf_obj[[section_col]])
  focus_idx <- !is.na(section_values) & section_values %in% section
  if (!any(focus_idx)) {
    stop(
      "No features matched section value(s): ",
      paste(section, collapse = ", "),
      call. = FALSE
    )
  }

  sf_focus <- sf_obj[focus_idx, , drop = FALSE]
  sf_context <- sf_obj[!focus_idx, , drop = FALSE]
  sf_focus[[role_col]] <- "focus"
  sf_context[[role_col]] <- "context"

  result <- .explode_section_dispatch(sf_focus, region_col, layout, ...)

  if (!identical(context, "none") && nrow(sf_context) > 0) {
    if (identical(layout, "grouped")) {
      result$sf_grouped <- .combine_focus_context_sf(
        result$sf_grouped,
        sf_context,
        role_col
      )
      result$sf_grouped_wgs <- sf::st_transform(result$sf_grouped, 4326)
    } else {
      result$sf_exp <- .combine_focus_context_sf(
        result$sf_exp,
        sf_context,
        role_col
      )
      result$sf_exp_wgs <- sf::st_transform(result$sf_exp, 4326)
    }
  }

  result$params$section_col <- section_col
  result$params$section <- section
  result$params$context <- context
  result$params$role_col <- role_col
  result$diagnostics$section_col <- section_col
  result$diagnostics$section <- section
  result$diagnostics$n_focus <- nrow(sf_focus)
  result$diagnostics$n_context <- if (identical(context, "none")) 0L else nrow(sf_context)
  result
}

#' @keywords internal
.explode_section_dispatch <- function(sf_obj, region_col, layout, ...) {
  if (identical(layout, "grouped")) {
    explode_grouped(sf_obj, region_col = region_col, ...)
  } else {
    explode_sf(sf_obj, region_col = region_col, ...)
  }
}

#' @keywords internal
.combine_focus_context_sf <- function(focus_sf, context_sf, role_col) {
  context_sf <- sf::st_transform(context_sf, sf::st_crs(focus_sf))

  focus_names <- names(focus_sf)
  context_missing <- setdiff(focus_names, names(context_sf))
  for (nm in context_missing) {
    context_sf[[nm]] <- NA
  }

  context_sf <- context_sf[, focus_names, drop = FALSE]
  focus_sf[[role_col]] <- as.character(focus_sf[[role_col]])
  context_sf[[role_col]] <- as.character(context_sf[[role_col]])

  dplyr::bind_rows(focus_sf, context_sf)
}

test_that("explode_section explodes selected section and keeps context", {
  x <- make_test_sf()
  x$subregion <- x$id

  out <- explode_section(
    x,
    section_col = "region",
    section = "A",
    region_col = "subregion",
    alpha_r = 10,
    alpha_l = 5,
    plot = FALSE,
    quiet = TRUE
  )

  expect_s3_class(out, "exploded_map")
  expect_equal(nrow(out$sf_exp), nrow(x))
  expect_equal(sum(out$sf_exp$.explodemap_role == "focus"), 2)
  expect_equal(sum(out$sf_exp$.explodemap_role == "context"), 2)
  expect_equal(out$diagnostics$n_focus, 2)
  expect_equal(out$diagnostics$n_context, 2)
})

test_that("explode_section can drop context", {
  x <- make_test_sf()
  x$subregion <- x$id

  out <- explode_section(
    x,
    section_col = "region",
    section = "A",
    region_col = "subregion",
    context = "none",
    alpha_r = 10,
    alpha_l = 5,
    plot = FALSE,
    quiet = TRUE
  )

  expect_equal(nrow(out$sf_exp), 2)
  expect_true(all(out$sf_exp$.explodemap_role == "focus"))
})

test_that("focus_map carries context options into widget payload", {
  x <- make_test_sf()
  x$.explodemap_role <- c("focus", "focus", "context", "context")

  widget <- focus_map(
    x,
    label_col = "id",
    context_col = ".explodemap_role",
    context_mode = "fade",
    context_opacity = 0.2,
    simplify = FALSE
  )

  expect_equal(widget$x$options$contextMode, "fade")
  expect_equal(widget$x$options$contextValues, list("context"))
  expect_equal(widget$x$options$contextOpacity, 0.2)
  expect_match(widget$x$geojson_str, '"context"')
})

make_test_sf <- function() {
  sq <- function(xmin, ymin, size = 1) {
    sf::st_polygon(list(matrix(
      c(
        xmin, ymin,
        xmin + size, ymin,
        xmin + size, ymin + size,
        xmin, ymin + size,
        xmin, ymin
      ),
      ncol = 2,
      byrow = TRUE
    )))
  }

  geom <- sf::st_sfc(
    sq(0, 0),   # A1
    sq(2, 0),   # A2
    sq(10, 0),  # B1
    sq(12, 0),  # B2
    crs = 3857
  )

  sf::st_sf(
    id = c("a1", "a2", "b1", "b2"),
    region = c("A", "A", "B", "B"),
    geometry = geom
  )
}

make_test_lookup <- function() {
  data.frame(
    id = c("a1", "a2", "b1"),
    region2 = c("North", "North", "South"),
    stringsAsFactors = FALSE
  )
}

make_grouped_sf <- function() {
  sq <- function(xmin, ymin, size = 1) {
    sf::st_polygon(list(matrix(
      c(
        xmin, ymin,
        xmin + size, ymin,
        xmin + size, ymin + size,
        xmin, ymin + size,
        xmin, ymin
      ),
      ncol = 2,
      byrow = TRUE
    )))
  }

  geom <- sf::st_sfc(
    sq(0, 0),
    sq(2, 0),
    sq(6, 0),
    sq(8, 0),
    sq(20, 0),
    sq(22, 0),
    crs = 3857
  )

  sf::st_sf(
    id = paste0("u", 1:6),
    region = c("R1", "R1", "R2", "R2", "R3", "R3"),
    geometry = geom
  )
}

test_that("print, summary, plot, and calibration_row work for exploded_map", {
  x <- make_test_sf()
  out <- explode_sf(x, region_col = "region", plot = FALSE)

  expect_invisible(print(out))
  expect_invisible(summary(out))
  expect_invisible(plot(out, "exploded"))
  expect_invisible(plot(out, "original"))
  expect_invisible(plot(out, "both"))

  row <- calibration_row(out)
  expect_s3_class(row, "data.frame")
  expect_equal(nrow(row), 1)
  expect_true(all(c(
    "label", "n_units", "n_regions", "w_bar_km", "R_local_km",
    "ratio", "alpha_r", "alpha_l", "gamma_r_implied", "gamma_l_implied"
  ) %in% names(row)))
})

test_that("calibration_row errors for non-exploded_map input", {
  expect_error(
    calibration_row(list()),
    "exploded_map"
  )
})

test_that("explode_grouped returns grouped_exploded_map structure", {
  x <- make_grouped_sf()

  out <- explode_grouped(
    x,
    region_col = "region",
    mode = "auto",
    plot = FALSE
  )

  expect_s3_class(out, "grouped_exploded_map")
  expect_true(all(c(
    "sf_orig", "sf_local", "sf_grouped", "sf_grouped_wgs",
    "stats", "params", "anchors", "plots", "diagnostics"
  ) %in% names(out)))

  expect_s3_class(out$sf_grouped, "sf")
  expect_equal(nrow(out$sf_grouped), nrow(x))
  expect_equal(as.integer(sf::st_crs(out$sf_grouped_wgs)$epsg), 4326)
})

test_that("explode_grouped manual mode uses supplied anchors", {
  x <- make_grouped_sf()

  anchors <- data.frame(
    region = c("R1", "R2", "R3"),
    anchor_x = c(0, 100, 200),
    anchor_y = c(0, 50, 100)
  )

  out <- explode_grouped(
    x,
    region_col = "region",
    mode = "manual",
    anchors = anchors,
    plot = FALSE
  )

  expect_s3_class(out, "grouped_exploded_map")
  expect_equal(nrow(out$anchors), 3)
})

test_that("grouped methods run invisibly", {
  x <- make_grouped_sf()

  out <- explode_grouped(
    x,
    region_col = "region",
    mode = "auto",
    plot = FALSE
  )

  expect_invisible(print(out))
  expect_invisible(plot(out, "grouped"))
  expect_invisible(plot(out, "original"))
  expect_invisible(plot(out, "local"))
  expect_invisible(plot(out, "all"))
})

test_that("explode_grouped accepts visual alias arguments", {
  x <- make_grouped_sf()

  out <- explode_grouped(
    x,
    region_col = "region",
    mode = "auto_collision",
    anchor_expand = 1.2,
    anchor_buffer = 100,
    density_scale = 50,
    block_sep = 75,
    max_iter = 2,
    plot = FALSE,
    quiet = TRUE
  )

  expect_s3_class(out, "grouped_exploded_map")
  expect_equal(out$params$kappa, 1.2)
  expect_equal(out$params$padding, 100)
  expect_equal(out$params$delta, 50)
  expect_equal(out$params$padding_sep, 75)
})

test_that("explode_sf_with_lookup joins lookup and labels unmatched as Other", {
  x <- make_test_sf()
  lookup <- make_test_lookup()

  out <- explode_sf_with_lookup(
    x,
    join_col = "id",
    lookup = lookup,
    lookup_key = "id",
    region_col = "region2",
    allow_other = TRUE,
    plot = FALSE
  )

  expect_s3_class(out, "exploded_map")
  expect_true("region2" %in% names(out$sf_orig))
  expect_true("Other" %in% out$sf_orig$region2)
})

test_that("explode_sf_with_lookup errors on bad join column", {
  x <- make_test_sf()
  lookup <- make_test_lookup()

  expect_error(
    explode_sf_with_lookup(
      x,
      join_col = "bad_col",
      lookup = lookup,
      lookup_key = "id",
      region_col = "region2",
      plot = FALSE
    ),
    "join_col"
  )
})

test_that("explode_sf_with_lookup errors on bad lookup key", {
  x <- make_test_sf()
  lookup <- make_test_lookup()

  expect_error(
    explode_sf_with_lookup(
      x,
      join_col = "id",
      lookup = lookup,
      lookup_key = "bad_key",
      region_col = "region2",
      plot = FALSE
    ),
    "lookup_key"
  )
})

test_that("focus_map carries named group palettes into widget options", {
  x <- make_test_sf()
  pal <- c(A = "#111111", B = "#222222")

  widget <- focus_map(
    x,
    label_col = "id",
    group_col = "region",
    group_palette = pal,
    simplify = FALSE
  )

  expect_s3_class(widget, "htmlwidget")
  expect_equal(widget$x$options$groupPalette, as.list(pal))
  expect_true(widget$x$options$hasGroups)
})

test_that("focus_map ignores unnamed group palettes with a warning", {
  x <- make_test_sf()

  expect_warning(
    widget <- focus_map(
      x,
      label_col = "id",
      group_col = "region",
      group_palette = c("#111111", "#222222"),
      simplify = FALSE
    ),
    "group_palette must be a named character vector"
  )

  expect_null(widget$x$options$groupPalette)
})

test_that("focus_map accepts partial group palettes", {
  x <- make_test_sf()

  widget <- focus_map(
    x,
    label_col = "id",
    group_col = "region",
    group_palette = c(A = "#111111"),
    simplify = FALSE
  )

  expect_equal(widget$x$options$groupPalette, list(A = "#111111"))
})

test_that("focus_map carries adaptive tiny-feature focus options", {
  x <- make_test_sf()

  widget <- focus_map(
    x,
    label_col = "id",
    simplify = FALSE,
    min_focus_width = 120,
    min_focus_height = 90,
    tiny_feature_threshold = 45,
    tiny_feature_boost = 1.6
  )

  expect_equal(widget$x$options$minFocusWidth, 120)
  expect_equal(widget$x$options$minFocusHeight, 90)
  expect_equal(widget$x$options$tinyFeatureThreshold, 45)
  expect_equal(widget$x$options$tinyFeatureBoost, 1.6)
})

test_that("focus_map validates adaptive tiny-feature focus options", {
  x <- make_test_sf()

  expect_error(
    focus_map(x, label_col = "id", min_focus_width = -1, simplify = FALSE),
    "min_focus_width"
  )
  expect_error(
    focus_map(x, label_col = "id", min_focus_height = -1, simplify = FALSE),
    "min_focus_height"
  )
  expect_error(
    focus_map(x, label_col = "id", tiny_feature_threshold = -1, simplify = FALSE),
    "tiny_feature_threshold"
  )
  expect_error(
    focus_map(x, label_col = "id", tiny_feature_boost = 0.8, simplify = FALSE),
    "tiny_feature_boost"
  )
})

test_that("focus_map carries origin context options", {
  x <- make_test_sf()

  default_widget <- focus_map(x, label_col = "id", simplify = FALSE)
  expect_equal(default_widget$x$options$originContext, "none")

  widget <- focus_map(
    x,
    label_col = "id",
    simplify = FALSE,
    origin_context = "inset",
    origin_context_position = "top-left",
    focus_context_opacity = 0.15
  )

  expect_equal(widget$x$options$originContext, "inset")
  expect_equal(widget$x$options$originContextPosition, "top-left")
  expect_equal(widget$x$options$focusContextOpacity, 0.15)
})

test_that("focus_map carries drag zoom visibility option", {
  x <- make_test_sf()

  default_widget <- focus_map(x, label_col = "id", simplify = FALSE)
  expect_false(default_widget$x$options$showDragZoom)

  widget <- focus_map(
    x,
    label_col = "id",
    simplify = FALSE,
    show_drag_zoom = TRUE
  )

  expect_true(widget$x$options$showDragZoom)
})

test_that("focus_map_preset returns municipal and drilldown options", {
  municipal <- focus_map_preset("municipal")
  expect_equal(municipal$min_focus_width, 260)
  expect_equal(municipal$origin_context, "both")
  expect_true(municipal$show_drag_zoom)

  drilldown <- focus_map_preset("drilldown")
  expect_equal(drilldown$context_mode, "fade")
  expect_false(drilldown$context_clickable)

  combined <- focus_map_preset("municipal_drilldown")
  expect_equal(combined$context_mode, "fade")
  expect_equal(combined$origin_context, "both")
})

test_that("focus_map applies presets without overriding explicit arguments", {
  x <- make_test_sf()

  widget <- focus_map(
    x,
    label_col = "id",
    simplify = FALSE,
    focus_preset = "municipal",
    min_focus_width = 180,
    show_drag_zoom = FALSE
  )

  expect_equal(widget$x$options$focusPreset, "municipal")
  expect_equal(widget$x$options$minFocusWidth, 180)
  expect_false(widget$x$options$showDragZoom)
  expect_equal(widget$x$options$originContext, "both")
  expect_equal(widget$x$options$tinyFeatureBoost, 1.35)
})

# =============================================================================
# R/export.R — Optional TopoJSON export via external mapshaper
# =============================================================================
#
# Provides a convenience helper for exporting exploded maps as TopoJSON,
# a common format for downstream tools such as Power BI, D3.js,
# and Observable.
#
# Requires the external `mapshaper` CLI tool (not an R dependency):
#   npm install -g mapshaper
#
# Design:
#   - Accepts sf, exploded_map, or grouped_exploded_map objects
#   - Writes a temporary GeoJSON via sf::st_write
#   - Shells out to mapshaper for GeoJSON → TopoJSON conversion
#   - Optionally simplifies geometry during conversion
#   - Keeps explodemap free of heavy JS/topology dependencies
# =============================================================================


# ─────────────────────────────────────────────────────────────────────────────
# INTERNAL HELPERS
# ─────────────────────────────────────────────────────────────────────────────

#' Normalize input to an export-ready sf object in WGS84
#' @param x An sf, exploded_map, or grouped_exploded_map object
#' @return sf object
#' @keywords internal
.as_export_sf <- function(x) {
  if (inherits(x, "grouped_exploded_map")) {
    if (is.null(x$sf_grouped_wgs)) {
      stop(
        "`grouped_exploded_map` is missing its WGS84 grouped geometry. ",
        "Recreate it with explode_grouped() before exporting.",
        call. = FALSE
      )
    }
    return(x$sf_grouped_wgs)
  }

  if (inherits(x, "exploded_map")) {
    if (is.null(x$sf_exp_wgs)) {
      stop(
        "`exploded_map` is missing its WGS84 exploded geometry. ",
        "Recreate it with explode_sf() before exporting.",
        call. = FALSE
      )
    }
    return(x$sf_exp_wgs)
  }

  if (inherits(x, "sf")) {
    crs <- sf::st_crs(x)

    if (is.na(crs)) {
      warning(
        "Input sf object has no CRS, so export_topojson() cannot transform it to WGS84. ",
        "Exporting coordinates as-is.",
        call. = FALSE
      )
      return(x)
    }

    if (!isTRUE(crs$epsg == 4326L)) {
      return(sf::st_transform(x, 4326))
    }

    return(x)
  }

  stop(
    "`x` must be an sf object, exploded_map, or grouped_exploded_map.",
    call. = FALSE
  )
}


#' Locate the mapshaper executable on the system path
#' @return Path to mapshaper executable
#' @keywords internal
.find_mapshaper <- function() {
  exe <- Sys.which("mapshaper")

  if (!nzchar(exe)) {
    stop(
      "The `mapshaper` command-line tool was not found on your system path.\n",
      "Install it with: npm install -g mapshaper\n",
      "Then restart your R session and try again.",
      call. = FALSE
    )
  }

  exe
}


#' Build mapshaper CLI arguments
#' @param input Path to input GeoJSON file
#' @param output Path to output TopoJSON file
#' @param simplify Optional simplification proportion in `(0, 1)`.
#' @return Character vector of arguments for system2()
#' @keywords internal
.build_mapshaper_args <- function(input, output, simplify = NULL) {
  args <- c(input)

  if (!is.null(simplify)) {
    if (!is.numeric(simplify) || length(simplify) != 1 ||
        simplify <= 0 || simplify >= 1) {
      stop(
        "`simplify` must be a single number between 0 and 1 (exclusive).",
        call. = FALSE
      )
    }

    args <- c(args, "-simplify", as.character(simplify))
  }

  c(args, "-o", "format=topojson", output)
}


# ─────────────────────────────────────────────────────────────────────────────
# PUBLIC API
# ─────────────────────────────────────────────────────────────────────────────

#' Export an exploded map as TopoJSON
#'
#' Converts an `sf`, `exploded_map`, or `grouped_exploded_map` object to
#' TopoJSON using the external `mapshaper` command-line tool. The input is
#' first written as a temporary GeoJSON file, then converted by mapshaper.
#'
#' This is intended as a convenience helper for downstream tools such as
#' Power BI, D3.js, and Observable that prefer or require TopoJSON input.
#'
#' @param x An `sf`, `exploded_map`, or `grouped_exploded_map` object.
#'   For `exploded_map` objects, the WGS84 exploded geometry (`sf_exp_wgs`)
#'   is exported. For `grouped_exploded_map` objects, the WGS84 grouped
#'   geometry (`sf_grouped_wgs`) is exported.
#' @param file Output file path. Should end in `.topojson` or `.json`.
#' @param simplify Optional simplification proportion passed to
#'   `mapshaper -simplify`. Must be a single number between 0 and 1
#'   (exclusive). For example, `simplify = 0.5` retains 50% of vertices.
#'   Default is `NULL` (no simplification). Note that simplification
#'   modifies polygon geometry and therefore breaks the exact geometry
#'   preservation guarantee of Proposition 1.
#' @param overwrite Logical; if `TRUE`, overwrite `file` if it already
#'   exists. Default is `FALSE`.
#'
#' @details
#' This function requires the external `mapshaper` command-line tool to be
#' installed and available on the system path. It can be installed with:
#'
#' \preformatted{npm install -g mapshaper}
#'
#' The `mapshaper` tool is not an R package dependency; it is invoked via
#' `system2()`. If mapshaper is not found, the function errors with an
#' informative message.
#'
#' @return Invisibly returns the output file path.
#'
#' @examples
#' \dontrun{
#' # Two-level export
#' result <- explode_sf(my_sf, region_col = "region", plot = FALSE)
#' export_topojson(result, "exploded.topojson")
#'
#' # Three-level grouped export
#' grp <- explode_grouped(my_sf, region_col = "region", plot = FALSE)
#' export_topojson(grp, "grouped.topojson")
#'
#' # With simplification (breaks Proposition 1 guarantee)
#' export_topojson(result, "simplified.topojson", simplify = 0.5)
#'
#' # Raw sf object
#' export_topojson(my_sf, "raw.topojson", overwrite = TRUE)
#' }
#'
#' @export
export_topojson <- function(x, file, simplify = NULL, overwrite = FALSE) {
  if (!is.character(file) || length(file) != 1 || !nzchar(file)) {
    stop("`file` must be a non-empty file path.", call. = FALSE)
  }

  ext <- tolower(tools::file_ext(file))
  if (!ext %in% c("topojson", "json")) {
    warning(
      "Output file does not end in .topojson or .json. ",
      "The file will still be written as TopoJSON.",
      call. = FALSE
    )
  }

  if (file.exists(file) && !isTRUE(overwrite)) {
    stop(
      "Output file '", file, "' already exists. ",
      "Use `overwrite = TRUE` to replace it.",
      call. = FALSE
    )
  }

  if (.shiny_is_running()) {
    warning(
      "export_topojson() calls system2() which blocks the R process synchronously. ",
      "In Shiny this freezes the session (and all other sessions on a ",
      "single-process server) until mapshaper finishes. ",
      "Consider running exports inside a downloadHandler() or outside of ",
      "reactive contexts. On shinyapps.io, mapshaper is not installed and ",
      "this function will error immediately.",
      call. = FALSE
    )
  }

  sf_obj <- .as_export_sf(x)
  exe <- .find_mapshaper()

  out_path <- normalizePath(file, winslash = "/", mustWork = FALSE)
  out_dir <- dirname(out_path)
  dir.create(out_dir, recursive = TRUE, showWarnings = FALSE)

  tmp_geojson <- tempfile(fileext = ".geojson")
  on.exit(unlink(tmp_geojson), add = TRUE)

  sf::st_write(
    sf_obj,
    tmp_geojson,
    driver = "GeoJSON",
    quiet = TRUE,
    delete_dsn = TRUE
  )

  args <- .build_mapshaper_args(
    input = tmp_geojson,
    output = out_path,
    simplify = simplify
  )

  cmd_out <- tryCatch(
    suppressWarnings(system2(exe, args = args, stdout = TRUE, stderr = TRUE)),
    error = function(e) {
      stop(
        "TopoJSON export failed while running mapshaper. ",
        "Check that mapshaper works from your system terminal. ",
        "Details: ", conditionMessage(e),
        call. = FALSE
      )
    }
  )

  if (!file.exists(out_path)) {
    detail <- if (length(cmd_out)) paste(cmd_out, collapse = "\n") else ""
    stop(
      "TopoJSON export failed; mapshaper did not create the output file.",
      if (nzchar(detail)) paste0("\nMapshaper output:\n", detail) else "",
      call. = FALSE
    )
  }

  size_kb <- round(file.info(out_path)$size / 1024, 1)
  message("Exported: ", out_path, " (", size_kb, " KB)")

  invisible(out_path)
}

test_that("export_topojson errors clearly when mapshaper is missing", {
  skip_if(nzchar(Sys.which("mapshaper")), "mapshaper is installed")

  x <- make_test_sf()

  expect_error(
    export_topojson(x, tempfile(fileext = ".topojson")),
    "mapshaper"
  )
})


test_that("export_topojson validates simplify", {
  x <- make_test_sf()

  expect_error(
    .build_mapshaper_args("in.geojson", "out.topojson", simplify = 0),
    "between 0 and 1"
  )

  expect_error(
    .build_mapshaper_args("in.geojson", "out.topojson", simplify = 1),
    "between 0 and 1"
  )
})

# =============================================================================
# R/presets.R - Interactive workflow presets
# =============================================================================

#' Focus-map option presets
#'
#' Returns a named list of [focus_map()] arguments for common interactive
#' mapping workflows. Presets are intentionally plain lists so they can be
#' inspected, modified, or passed through `do.call()`.
#'
#' @param name Preset name. `"municipal"` tunes small-area focus maps with
#'   adaptive sizing, source cues, drag zoom, and dense-layer performance.
#'   `"drilldown"` tunes selected-section maps with faded context and source
#'   cues. `"municipal_drilldown"` combines both.
#' @return A named list of [focus_map()] arguments.
#' @export
#' @examples
#' focus_map_preset("municipal")
focus_map_preset <- function(name = c("municipal", "drilldown", "municipal_drilldown", "none")) {
  name <- match.arg(name)

  municipal <- list(
    focus_size = 0.78,
    focus_padding = 44,
    min_focus_width = 260,
    min_focus_height = 200,
    tiny_feature_threshold = 64,
    tiny_feature_boost = 1.35,
    max_zoom = 40,
    origin_context = "both",
    origin_context_position = "top-left",
    focus_context_opacity = 0.14,
    show_drag_zoom = TRUE,
    performance_mode = TRUE
  )

  drilldown <- list(
    context_mode = "fade",
    context_opacity = 0.16,
    context_clickable = FALSE,
    origin_context = "socket",
    focus_context_opacity = 0.22
  )

  switch(
    name,
    none = list(),
    municipal = municipal,
    drilldown = drilldown,
    municipal_drilldown = utils::modifyList(drilldown, municipal)
  )
}

# =============================================================================
# R/methods.R — S3 methods for exploded_map and grouped_exploded_map objects
# =============================================================================

#' Print an exploded_map object
#' @param x An `exploded_map` object
#' @param ... Ignored
#' @return Invisibly returns `x`, the original `exploded_map` object. The
#'   method is called for its side effect of printing a compact diagnostic
#'   overview of units, regions, derived parameters, and optional refinement
#'   information.
#' @export
print.exploded_map <- function(x, ...) {
  stats  <- x$stats
  params <- x$params
  diag   <- x$diagnostics

  cat("\n-- ", diag$label, " ", paste(rep("-", 40), collapse = ""), "\n", sep = "")
  n_units_display <- if (!is.null(stats$n_units_input)) stats$n_units_input else stats$n_units
  cat("  n units   : ", format(n_units_display, big.mark = ","), "\n")
  if (!is.null(stats$n_units_excluded) && stats$n_units_excluded > 0) {
    cat("  stats from: ", format(stats$n_units_stats, big.mark = ","),
        " units (excluded ", stats$n_units_excluded, " 'Other')\n", sep = "")
  }
  cat("  n regions : ", stats$n_regions, "\n")
  cat("  w_bar     : ", fmt_dist(stats$w_bar), "\n")
  cat("  R_local   : ", fmt_dist(stats$R_local), "\n")
  cat("  n_bar     : ", round(stats$n_bar), "\n")
  cat("  R_local/w : ", round(stats$ratio, 2), "\n")
  cat("  alpha_r   : ", fmt_dist(params$alpha_r), "\n")
  cat("  alpha_l   : ", fmt_dist(params$alpha_l), "\n")
  cat("  p         : ", params$p, "\n")
  cat("  max ||t|| : ", fmt_dist(params$alpha_r + params$alpha_l),
      "  (Proposition 3 bound)\n")
  if (!is.null(x$refinement) && isTRUE(x$refinement$enabled)) {
    cat("  refine    : ", x$refinement$within,
        " pairs, gap ", fmt_dist(x$refinement$min_gap),
        ", max shift ", fmt_dist(x$refinement$max_shift_observed),
        " / ", fmt_dist(x$refinement$max_shift), "\n", sep = "")
  }
  invisible(x)
}

#' Plot an exploded_map object
#' @param x An `exploded_map` object
#' @param which "exploded" (default), "original", or "both"
#' @param ... Ignored
#' @return Invisibly returns `x`, the original `exploded_map` object. The
#'   method is called for its side effect of drawing the stored `ggplot2`
#'   original and/or exploded map objects.
#' @export
plot.exploded_map <- function(x, which = c("exploded", "original", "both"), ...) {
  which <- match.arg(which)
  if (which == "original" || which == "both") print(x$plots$orig)
  if (which == "exploded" || which == "both") print(x$plots$exp)
  invisible(x)
}

#' Summary of an exploded_map object
#' @param object An `exploded_map` object
#' @param ... Ignored
#' @return Invisibly returns `object`, the original `exploded_map` object. The
#'   method is called for its side effect of printing a human-readable summary
#'   of dataset size, geometry statistics, displacement parameters, refinement
#'   diagnostics, and implied calibration coefficients.
#' @export
summary.exploded_map <- function(object, ...) {
  stats  <- object$stats
  params <- object$params

  cat("\nExploded Map Summary\n")
  cat("====================\n")
  cat("Dataset:     ", object$diagnostics$label, "\n")
  n_units_display <- if (!is.null(stats$n_units_input)) stats$n_units_input else stats$n_units
  cat("Units:       ", format(n_units_display, big.mark = ","), "\n")
  if (!is.null(stats$n_units_excluded) && stats$n_units_excluded > 0) {
    cat("Stats from:  ", format(stats$n_units_stats, big.mark = ","),
        " units (excluded ", stats$n_units_excluded, " 'Other')\n", sep = "")
  }
  cat("Regions:     ", stats$n_regions, "\n")
  cat("Grouped by:  ", object$diagnostics$region_col, "\n\n")
  cat("Geometry Statistics\n")
  cat("  Characteristic diameter (w_bar): ", fmt_dist(stats$w_bar), "\n")
  cat("  Regional radius (R_local):       ", fmt_dist(stats$R_local), "\n")
  cat("  Median units/region (n_bar):     ", round(stats$n_bar), "\n")
  cat("  Tightness ratio (R_local/w_bar): ", round(stats$ratio, 2), "\n\n")
  cat("Parameters\n")
  cat("  alpha_r: ", fmt_dist(params$alpha_r), "  (regional separation)\n")
  cat("  alpha_l: ", fmt_dist(params$alpha_l), "  (local expansion)\n")
  cat("  p:       ", params$p, "\n\n")
  if (!is.null(object$refinement) && isTRUE(object$refinement$enabled)) {
    cat("Collision Refinement\n")
    cat("  within:          ", object$refinement$within, "\n")
    cat("  min_gap:         ", fmt_dist(object$refinement$min_gap), "\n")
    cat("  max_shift cap:   ", fmt_dist(object$refinement$max_shift), "\n")
    cat("  max_shift used:  ", fmt_dist(object$refinement$max_shift_observed), "\n")
    cat("  iterations:      ", object$refinement$iterations, "\n")
    cat("  corrected pairs: ", object$refinement$corrected_pairs, "\n\n")
  }
  cat("Implied Gamma Coefficients\n")
  cat("  gamma_r: ", round(object$gamma_r_implied, 4), "\n")
  cat("  gamma_l: ", round(object$gamma_l_implied, 4), "\n")
  invisible(object)
}

#' Extract calibration row from an exploded_map
#'
#' Returns a one-row data.frame suitable for binding across datasets
#' to build a calibration table.
#'
#' @param x An `exploded_map` object
#' @return A one-row data.frame
#' @export
calibration_row <- function(x) {
  if (!inherits(x, "exploded_map"))
    stop("Input must be an exploded_map object.", call. = FALSE)
  stats  <- x$stats
  params <- x$params
  data.frame(
    label           = x$diagnostics$label,
    n_units         = stats$n_units,
    n_regions       = stats$n_regions,
    w_bar_km        = round(stats$w_bar / 1000, 2),
    R_local_km      = round(stats$R_local / 1000, 1),
    ratio           = round(stats$ratio, 2),
    alpha_r         = round(params$alpha_r),
    alpha_l         = round(params$alpha_l),
    gamma_r_implied = round(x$gamma_r_implied, 4),
    gamma_l_implied = round(x$gamma_l_implied, 4),
    stringsAsFactors = FALSE
  )
}

# =============================================================================
# GROUPED EXPLODED MAP S3 METHODS
# =============================================================================

#' Print a grouped_exploded_map object
#' @param x A `grouped_exploded_map` object
#' @param ... Ignored
#' @return Invisibly returns `x`, the original `grouped_exploded_map` object.
#'   The method is called for its side effect of printing a compact diagnostic
#'   overview of units, regions, grouping mode, and grouped-layout parameters.
#' @export
print.grouped_exploded_map <- function(x, ...) {
  stats  <- x$stats
  params <- x$params
  diag   <- x$diagnostics

  cat("\n-- ", diag$label, " (grouped) ", paste(rep("-", 30), collapse = ""), "\n", sep = "")
  cat("  n units   : ", format(stats$n_units, big.mark = ","), "\n")
  cat("  n regions : ", stats$n_regions, "\n")
  cat("  mode      : ", params$mode, "\n")
  cat("  alpha_l   : ", fmt_dist(params$alpha_l), "\n")
  cat("  p         : ", params$p, "\n")
  if (params$mode != "manual") {
    cat("  kappa     : ", params$kappa, "\n")
    cat("  padding   : ", fmt_dist(params$padding), "\n")
  }
  if (params$mode == "auto_collision") {
    cat("  lambda    : ", params$lambda, "\n")
    cat("  eta       : ", params$eta, "\n")
  }
  invisible(x)
}

#' Plot a grouped_exploded_map object
#' @param x A `grouped_exploded_map` object
#' @param which "grouped" (default), "original", "local", or "all"
#' @param ... Ignored
#' @return Invisibly returns `x`, the original `grouped_exploded_map` object.
#'   The method is called for its side effect of drawing the stored `ggplot2`
#'   grouped, original, and/or local map objects.
#' @export
plot.grouped_exploded_map <- function(x, which = c("grouped", "original", "local", "all"), ...) {
  which <- match.arg(which)
  if (which == "original" || which == "all") print(x$plots$orig)
  if (which == "local"    || which == "all") {
    if (!is.null(x$plots$local)) print(x$plots$local)
  }
  if (which == "grouped"  || which == "all") print(x$plots$grouped)
  invisible(x)
}

#' Summary of a grouped_exploded_map object
#' @param object A `grouped_exploded_map` object
#' @param ... Ignored
#' @return Invisibly returns `object`, the original `grouped_exploded_map`
#'   object. The method is called for its side effect of printing dataset size,
#'   grouping diagnostics, anchor parameters, and anchor radii.
#' @export
summary.grouped_exploded_map <- function(object, ...) {
  stats  <- object$stats
  params <- object$params

  cat("\nGrouped Exploded Map Summary\n")
  cat("============================\n")
  cat("Dataset:     ", object$diagnostics$label, "\n")
  cat("Units:       ", format(stats$n_units, big.mark = ","), "\n")
  cat("Regions:     ", stats$n_regions, "\n")
  cat("Grouped by:  ", object$diagnostics$region_col, "\n")
  cat("Anchor mode: ", params$mode, "\n\n")
  cat("Level 1 Parameters\n")
  cat("  alpha_l: ", fmt_dist(params$alpha_l), "  (local expansion)\n")
  cat("  p:       ", params$p, "\n\n")
  if (params$mode != "manual") {
    cat("Anchor Parameters\n")
    cat("  kappa:       ", params$kappa, "\n")
    cat("  padding:     ", fmt_dist(params$padding), "\n")
    cat("  delta:       ", fmt_dist(params$delta), "\n")
    if (params$mode == "auto_collision") {
      cat("  lambda:      ", params$lambda, "\n")
      cat("  eta:         ", params$eta, "\n")
    }
  }
  if (!is.null(object$anchors)) {
    cat("\nAnchor Radii\n")
    print(object$anchors[, c(object$diagnostics$region_col,
                             "block_radius", "n_units")],
          row.names = FALSE)
  }
  invisible(object)
}

# =============================================================================
# R/api.R — Public API (three entry points) and S3 class
# =============================================================================

#' Explode a US state from TIGER/Line data
#'
#' Downloads municipal boundaries automatically, groups by county-to-region
#' mapping, derives parameters via Analytical Results 1-2, and returns an
#' `exploded_map` S3 object.
#'
#' @param state_fips 2-digit FIPS code (e.g. "34" for NJ)
#' @param crs Projected CRS EPSG code (metric units)
#' @param region_map Named list: region_name -> character vector of county names
#' @param gamma_r Regional clearance coefficient (default 3.0)
#' @param gamma_l Local clearance coefficient (default 1.136)
#' @param p Distance scaling exponent (default 1.25)
#' @param alpha_r Optional manual override for regional separation (metres).
#'   May be supplied independently of `alpha_l`.
#' @param alpha_l Optional manual override for local expansion (metres).
#'   May be supplied independently of `alpha_r`.
#' @param refine If TRUE, apply a bounded collision-refinement pass after the
#'   analytical displacement. Default is FALSE.
#' @param refine_min_gap Optional minimum boundary-to-boundary gap in map units.
#'   If NULL and `refine = TRUE`, uses 2% of the characteristic diameter.
#' @param refine_max_shift Optional maximum correction per feature in map units.
#'   If NULL and `refine = TRUE`, uses 10% of `alpha_r + alpha_l`.
#' @param refine_max_iter Maximum refinement iterations.
#' @param refine_step Fraction of each gap deficit corrected per iteration.
#' @param refine_within Refine pairs within each `"region"` (default) or across
#'   `"all"` features.
#' @param allow_other If TRUE, permits units mapped to "Other"
#' @param fix_invalid If TRUE, auto-repairs invalid geometries
#' @param centroid_fun "centroid" (default) or "point_on_surface"
#' @param plot Print plots on return (default TRUE). Automatically suppressed
#'   when called inside a live Shiny session; use [plot.exploded_map()] inside
#'   `renderPlot()` instead.
#' @param export NULL (no export), TRUE (auto-named GeoJSON), or a file path
#' @param label Title for plots and print output
#' @param quiet If `TRUE`, suppress all `message()` output (useful inside
#'   Shiny `reactive()` and `observe()` where messages are invisible to users).
#'   Default `FALSE`.
#' @return An `exploded_map` S3 object
#' @export
explode_state <- function(state_fips,
                          crs,
                          region_map,
                          gamma_r      = 3.0,
                          gamma_l      = 1.136,
                          p            = 1.25,
                          alpha_r      = NULL,
                          alpha_l      = NULL,
                          refine       = FALSE,
                          refine_min_gap = NULL,
                          refine_max_shift = NULL,
                          refine_max_iter = 20,
                          refine_step  = 0.5,
                          refine_within = c("region", "all"),
                          allow_other  = FALSE,
                          fix_invalid  = TRUE,
                          centroid_fun = c("centroid", "point_on_surface"),
                          plot         = TRUE,
                          export       = NULL,
                          label        = paste0("FIPS ", state_fips),
                          quiet        = FALSE) {

  if (!quiet) message("Downloading TIGER/Line data (FIPS ", state_fips, ")...")
  sf_raw <- .download_cousub(state_fips, crs)

  if (!quiet) message("Assigning regions...")
  sf_reg <- .attach_regions_tiger(sf_raw, state_fips, region_map, quiet = quiet)

  n_other   <- sum(sf_reg$region == "Other", na.rm = TRUE)
  n_regions <- dplyr::n_distinct(sf_reg$region[sf_reg$region != "Other"])

  if (n_regions < 2)
    stop("Fewer than 2 regions matched. Check county names in region_map.",
         call. = FALSE)
  if (n_other > 0 && !allow_other)
    stop(n_other, " units mapped to 'Other'. Fix region_map or pass allow_other = TRUE.",
         call. = FALSE)

  sf_reg <- validate_input(sf_reg, "region", allow_other, fix_invalid)

  sf_for_stats <- if (allow_other && n_other > 0)
    sf_reg[sf_reg$region != "Other", ] else sf_reg

  .run_explode(sf_reg, sf_for_stats,
               gamma_r, gamma_l, p, alpha_r, alpha_l,
               refine, refine_min_gap, refine_max_shift,
               refine_max_iter, refine_step, match.arg(refine_within),
               plot, export, label, "region", match.arg(centroid_fun),
               quiet = quiet)
}


#' Explode any sf object with an existing grouping column
#'
#' @param sf_obj Projected sf object (metric CRS)
#' @param region_col Name of the column defining groups
#' @param gamma_r Regional clearance coefficient (default 3.0)
#' @param gamma_l Local clearance coefficient (default 1.136)
#' @param p Distance scaling exponent (default 1.25)
#' @param alpha_r Optional manual override for regional separation (metres).
#'   May be supplied independently of `alpha_l`.
#' @param alpha_l Optional manual override for local expansion (metres).
#'   May be supplied independently of `alpha_r`.
#' @param refine If TRUE, apply a bounded collision-refinement pass after the
#'   analytical displacement. Default is FALSE.
#' @param refine_min_gap Optional minimum boundary-to-boundary gap in map units.
#'   If NULL and `refine = TRUE`, uses 2% of the characteristic diameter.
#' @param refine_max_shift Optional maximum correction per feature in map units.
#'   If NULL and `refine = TRUE`, uses 10% of `alpha_r + alpha_l`.
#' @param refine_max_iter Maximum refinement iterations.
#' @param refine_step Fraction of each gap deficit corrected per iteration.
#' @param refine_within Refine pairs within each `"region"` (default) or across
#'   `"all"` features.
#' @param allow_other If TRUE, permits "Other" units
#' @param fix_invalid If TRUE, auto-repairs invalid geometries
#' @param centroid_fun "centroid" (default) or "point_on_surface"
#' @param plot Print plots on return. Automatically suppressed when called
#'   inside a live Shiny session; use [plot.exploded_map()] inside
#'   `renderPlot()` instead.
#' @param export NULL, TRUE, or file path
#' @param label Title for plots
#' @param quiet If `TRUE`, suppress all `message()` output. Default `FALSE`.
#' @return An `exploded_map` S3 object
#' @export
explode_sf <- function(sf_obj,
                       region_col   = "region",
                       gamma_r      = 3.0,
                       gamma_l      = 1.136,
                       p            = 1.25,
                       alpha_r      = NULL,
                       alpha_l      = NULL,
                       refine       = FALSE,
                       refine_min_gap = NULL,
                       refine_max_shift = NULL,
                       refine_max_iter = 20,
                       refine_step  = 0.5,
                       refine_within = c("region", "all"),
                       allow_other  = FALSE,
                       fix_invalid  = TRUE,
                       centroid_fun = c("centroid", "point_on_surface"),
                       plot         = TRUE,
                       export       = NULL,
                       label        = "Custom Dataset",
                       quiet        = FALSE) {

  sf_obj <- validate_input(sf_obj, region_col, allow_other, fix_invalid)

  sf_for_stats <- if (allow_other && any(sf_obj[[region_col]] == "Other"))
    sf_obj[sf_obj[[region_col]] != "Other", ] else sf_obj

  .run_explode(sf_obj, sf_for_stats,
               gamma_r, gamma_l, p, alpha_r, alpha_l,
               refine, refine_min_gap, refine_max_shift,
               refine_max_iter, refine_step, match.arg(refine_within),
               plot, export, label, region_col, match.arg(centroid_fun),
               quiet = quiet)
}


#' Explode any sf object using an external lookup table
#'
#' Joins the lookup to `sf_obj` before exploding. Unmatched units are
#' labelled "Other".
#'
#' @param sf_obj Projected sf object
#' @param join_col Column in sf_obj to join on
#' @param lookup data.frame with join key and region column
#' @param lookup_key Column name in lookup matching join_col
#' @param region_col Column name in lookup containing region labels
#' @param quiet If `TRUE`, suppress `message()` output. Default `FALSE`.
#'   Passed through to [explode_sf()].
#' @param ... Passed to [explode_sf()]
#' @return An `exploded_map` S3 object
#' @export
explode_sf_with_lookup <- function(sf_obj,
                                   join_col,
                                   lookup,
                                   lookup_key = join_col,
                                   region_col = "region",
                                   quiet      = FALSE,
                                   ...) {
  if (!join_col %in% names(sf_obj))
    stop("join_col '", join_col, "' not found in sf_obj.", call. = FALSE)
  if (!lookup_key %in% names(lookup))
    stop("lookup_key '", lookup_key, "' not found in lookup.", call. = FALSE)
  if (!region_col %in% names(lookup))
    stop("region_col '", region_col, "' not found in lookup.", call. = FALSE)

  sf_joined <- sf_obj |>
    dplyr::left_join(
      lookup |> dplyr::select(dplyr::all_of(c(lookup_key, region_col))),
      by = stats::setNames(lookup_key, join_col)
    )

  n_matched <- sum(!is.na(sf_joined[[region_col]]))
  if (!quiet)
    message("Lookup join: ", n_matched, " / ", nrow(sf_joined), " units matched.")
  sf_joined[[region_col]][is.na(sf_joined[[region_col]])] <- "Other"

  explode_sf(sf_joined, region_col = region_col, quiet = quiet, ...)
}


# =============================================================================
# INTERNAL RUNNER (shared by all three entry points)
# =============================================================================

.run_explode <- function(sf_obj, sf_for_stats,
                         gamma_r, gamma_l, p,
                         alpha_r_override, alpha_l_override,
                         refine, refine_min_gap, refine_max_shift,
                         refine_max_iter, refine_step, refine_within,
                         plot, export, label,
                         region_col, centroid_fun = "centroid",
                         quiet = FALSE) {

  stats <- compute_stats(sf_for_stats, region_col, centroid_fun = centroid_fun)
  stats$n_units_input <- nrow(sf_obj)
  stats$n_units_stats <- nrow(sf_for_stats)
  stats$n_units_excluded <- nrow(sf_obj) - nrow(sf_for_stats)

  params <- derive_params(stats, gamma_r, gamma_l, p)

  # Manual overrides take priority, and may be supplied independently.
  if (!is.null(alpha_r_override)) {
    if (!is.numeric(alpha_r_override) || length(alpha_r_override) != 1 ||
        is.na(alpha_r_override) || alpha_r_override < 0) {
      stop("`alpha_r` must be a single non-negative number.", call. = FALSE)
    }
    params$alpha_r <- alpha_r_override
    params$gamma_r <- NA_real_
    if (!quiet) message("Using manual alpha_r = ", alpha_r_override, " m")
  }

  if (!is.null(alpha_l_override)) {
    if (!is.numeric(alpha_l_override) || length(alpha_l_override) != 1 ||
        is.na(alpha_l_override) || alpha_l_override < 0) {
      stop("`alpha_l` must be a single non-negative number.", call. = FALSE)
    }
    params$alpha_l <- alpha_l_override
    params$gamma_l <- NA_real_
    if (!quiet) message("Using manual alpha_l = ", alpha_l_override, " m")
  }

  sf_exp     <- explode_sf_core(sf_obj, region_col,
                                params$alpha_r, params$alpha_l, params$p,
                                centroid_fun)

  if (!is.logical(refine) || length(refine) != 1 || is.na(refine)) {
    stop("`refine` must be TRUE or FALSE.", call. = FALSE)
  }

  refinement <- list(enabled = FALSE)
  if (isTRUE(refine)) {
    if (is.null(refine_min_gap)) {
      refine_min_gap <- stats$w_bar * 0.02
    }
    if (is.null(refine_max_shift)) {
      refine_max_shift <- (params$alpha_r + params$alpha_l) * 0.10
    }

    refined <- refine_collisions(
      sf_exp,
      region_col,
      min_gap = refine_min_gap,
      max_shift = refine_max_shift,
      max_iter = refine_max_iter,
      step = refine_step,
      within = refine_within,
      centroid_fun = centroid_fun
    )
    sf_exp <- refined$sf
    refinement <- refined$diagnostics
    if (!quiet)
      message(
        "Collision refinement: corrected ", refinement$corrected_pairs,
        " pair-visits; max shift = ",
        fmt_dist(refinement$max_shift_observed), "."
      )
  }

  params$refine <- isTRUE(refine)
  params$refine_min_gap <- if (isTRUE(refine)) refine_min_gap else NULL
  params$refine_max_shift <- if (isTRUE(refine)) refine_max_shift else NULL
  params$refine_max_iter <- if (isTRUE(refine)) as.integer(refine_max_iter) else NULL
  params$refine_step <- if (isTRUE(refine)) refine_step else NULL
  params$refine_within <- if (isTRUE(refine)) refine_within else NULL

  sf_exp_wgs <- sf::st_transform(sf_exp, 4326)

  plots <- .make_plots(sf_obj, sf_exp, region_col, label, params)
  if (isTRUE(plot)) {
    if (.shiny_is_running()) {
      # Printing ggplot objects inside a Shiny reactive has no visible effect
      # and may open a rogue graphics device on some servers. Use
      # plot(result) inside renderPlot() to display the stored plots.
      message(
        "explodemap: auto-plot suppressed inside Shiny. ",
        "Use plot(result) inside renderPlot() to display the maps."
      )
    } else {
      print(plots$orig)
      print(plots$exp)
    }
  }

  # Handle export: NULL = none, TRUE = auto-name, character = explicit path
  .handle_export(export, sf_exp_wgs, label)

  # Implied gamma values (for calibration)
  gamma_r_implied <- params$alpha_r /
    (stats$w_bar / (2 * sin(pi / stats$n_regions)))
  gamma_l_implied <- params$alpha_l /
    (2 * stats$R_local / sqrt(stats$n_bar))

  result <- list(
    sf_orig         = sf_obj,
    sf_exp          = sf_exp,
    sf_exp_wgs      = sf_exp_wgs,
    stats           = stats,
    params          = params,
    gamma_r_implied = gamma_r_implied,
    gamma_l_implied = gamma_l_implied,
    plots           = plots,
    refinement      = refinement,
    diagnostics     = list(
      label        = label,
      region_col   = region_col,
      centroid_fun = centroid_fun
    )
  )
  structure(result, class = c("exploded_map", "list"))
}


#' Handle export logic
#' @keywords internal
.handle_export <- function(export, sf_exp_wgs, label) {
  if (is.null(export)) return(invisible(NULL))
  if (isTRUE(export)) {
    fname <- paste0("exploded_",
                    gsub("[^a-zA-Z0-9]", "_", tolower(label)), ".geojson")
  } else if (is.character(export)) {
    if (length(export) != 1 || !nzchar(export)) {
      stop(
        "`export` must be NULL, TRUE, or a single non-empty output file path.",
        call. = FALSE
      )
    }
    fname <- export
  } else {
    stop(
      "`export` must be NULL, TRUE, or a single non-empty output file path.",
      call. = FALSE
    )
  }
  sf::st_write(sf_exp_wgs, fname, driver = "GeoJSON",
               delete_dsn = TRUE, quiet = TRUE)
  message("Exported: ", fname)
}

# =============================================================================
# R/focusmap.R — Interactive focus-map viewer (htmlwidget)
# =============================================================================

#' Interactive focus-map viewer
#'
#' Renders spatial features as a smooth, interactive SVG map. Click any
#' polygon to zoom in and lift it from the map with a "toast" effect;
#' right-click or press Escape to reset. Camera transitions use D3's
#' optimal zoom interpolation for fluid 60 fps motion with zero server
#' round-trips.
#'
#' Accepts raw \code{sf} objects, \code{exploded_map} results from
#' \code{\link{explode_sf}}, or \code{grouped_exploded_map} results from
#' \code{\link{explode_grouped}}. For exploded objects, the displaced
#' (WGS 84) geometry is used automatically.
#'
#' @param x An \code{sf}, \code{exploded_map}, or
#'   \code{grouped_exploded_map} object.
#' @param label_col Character. Column name for polygon labels.
#'   Auto-detected if \code{NULL}.
#' @param id_col Optional stable feature ID column for Shiny selection events.
#'   Defaults to row order if \code{NULL}.
#' @param group_col Character. Optional column for region/group
#'   colouring. Polygons sharing a group value share a hue.
#' @param group_palette Optional named character vector of colours for
#'   `group_col` values. Names should match group values; unmatched groups
#'   fall back to the widget palette.
#' @param context_col Optional column identifying features that should remain
#'   as geographic context rather than active focus features.
#' @param context_values Character vector of values in `context_col` that mark
#'   context features. Default `"context"`.
#' @param context_mode How context features are drawn: `"fade"` keeps them
#'   visible but muted, `"hide"` makes them invisible, and `"show"` draws them
#'   normally.
#' @param context_fill Fill colour for context features when
#'   `context_mode = "fade"`.
#' @param context_opacity Fill opacity for faded context features.
#' @param context_clickable Should context features remain clickable? Default
#'   `FALSE`.
#' @param focus_preset Optional named preset for common interactive workflows.
#'   `"municipal"` tunes small-area focus, source cues, drag zoom, and dense
#'   layer performance. `"drilldown"` tunes context fading and source cues for
#'   selected-section maps. `"municipal_drilldown"` combines both. Explicit
#'   arguments supplied by the user override preset defaults.
#' @param simplify Controls geometry simplification for rendering
#'   performance. \code{TRUE} (default) applies a sensible tolerance
#'   (\code{dTolerance = 0.001} in WGS 84 degrees, \eqn{\approx}
#'   100 m). A positive number sets a custom tolerance. \code{FALSE}
#'   disables simplification. Only affects the widget copy — the
#'   original data is never modified.
#' @param fill Fill colour (used when \code{group_col} is \code{NULL}).
#'   Default \code{"#2d6ea3"}.
#' @param fill_opacity Fill opacity. Default \code{0.58}.
#' @param stroke Stroke colour. Default \code{"#ffffff"}.
#' @param lift_scale Initial toast lift scale. Default \code{1.16}.
#'   Increase this to make the lifted feature larger.
#' @param focus_padding Extra screen-space padding in pixels around the lifted
#'   feature during focus. Increase this if large lifted features feel too
#'   close to the map edge.
#' @param focus_size Target fraction of the map viewport the lifted feature
#'   may occupy. Increase this to make selected areas appear larger while
#'   preserving \code{focus_padding}.
#' @param min_focus_width,min_focus_height Minimum focused feature width and
#'   height in screen pixels. When a selected feature is very small, the widget
#'   may zoom past the usual density-aware default until the lifted feature
#'   reaches these dimensions. Set to \code{0} to disable either constraint.
#' @param tiny_feature_threshold Screen-pixel size below which a selected
#'   feature receives an adaptive lift-scale boost. Set to \code{0} to disable.
#' @param tiny_feature_boost Maximum multiplier applied to \code{lift_scale}
#'   for the smallest features. Values below \code{1} are not allowed.
#' @param max_zoom Optional maximum camera zoom. If \code{NULL}, a density-aware
#'   default is used.
#' @param origin_context How the selected feature's source location should be
#'   shown while focused. The default \code{"none"} keeps focus maps visually
#'   unchanged unless this feature is explicitly enabled. \code{"socket"} keeps
#'   the source outline in the main
#'   map, \code{"inset"} shows a small overview map, \code{"both"} uses both,
#'   and \code{"none"} disables the cue.
#' @param origin_context_position Position for the overview inset:
#'   \code{"bottom-left"}, \code{"bottom-right"}, \code{"top-left"}, or
#'   \code{"top-right"}.
#' @param focus_context_opacity Fill opacity for non-selected features while a
#'   feature is focused. Lower values make tiny selected areas easier to read.
#' @param show_drag_zoom Show a widget-level drag-zoom toggle. When enabled,
#'   users can draw a marquee rectangle to zoom into dense clusters while
#'   ordinary feature clicks continue to focus the map. Shift-drag works as a
#'   shortcut even when the button is hidden.
#' @param font_size Label font size in px. Default \code{14}.
#' @param show_labels Show labels on lifted shapes? Default \code{TRUE}.
#' @param show_sidebar Deprecated and has no effect. Will be removed in a
#'   future version.
#' @param performance_mode Logical or \code{NULL}. If \code{NULL}, dense
#'   layers automatically use shorter camera transitions and lighter
#'   in-flight rendering. Set \code{TRUE} to force it or \code{FALSE} to
#'   disable it.
#' @param info_cols Optional character vector of columns to show in a
#'   non-blocking focus card when a feature is selected.
#' @param info_labels Optional named character vector or list for display
#'   labels in the focus card. Names should match \code{info_cols}.
#' @param info_title Optional column to use as the focus card title. Defaults
#'   to \code{label_col}.
#' @param info_position Position for the focus card: \code{"top-right"},
#'   \code{"top-left"}, \code{"bottom-right"}, or \code{"bottom-left"}.
#' @param info_card_scale Relative size for the focus card. Values above
#'   \code{1} make the card larger; values below \code{1} make it more compact.
#' @param area_min Min screen area (px^2) for label visibility.
#' @param width_min Min screen width (px) for label visibility.
#' @param height_min Min screen height (px) for label visibility.
#' @param width Widget width. Default \code{"100\%"}.
#' @param height Widget height. Default \code{"600px"}.
#' @param elementId Optional element ID.
#'
#' @return An \code{htmlwidgets} object.
#'
#' @examples
#' \donttest{
#' poly <- function(xmin, ymin, xmax, ymax) {
#'   sf::st_polygon(list(rbind(
#'     c(xmin, ymin), c(xmax, ymin), c(xmax, ymax),
#'     c(xmin, ymax), c(xmin, ymin)
#'   )))
#' }
#'
#' counties <- sf::st_sf(
#'   NAME = c("A", "B"),
#'   region = c("North", "South"),
#'   geometry = sf::st_sfc(
#'     poly(-74.2, 40.0, -74.0, 40.2),
#'     poly(-73.9, 40.0, -73.7, 40.2),
#'     crs = 4326
#'   )
#' )
#'
#' focus_map(counties, label_col = "NAME", group_col = "region")
#' }
#'
#' @export
focus_map <- function(x,
                      label_col    = NULL,
                      id_col       = NULL,
                      group_col    = NULL,
                      group_palette = NULL,
                      context_col  = NULL,
                      context_values = "context",
                      context_mode = c("fade", "hide", "show"),
                      context_fill = "#cfd9df",
                      context_opacity = 0.18,
                      context_clickable = FALSE,
                      focus_preset = c("none", "municipal", "drilldown", "municipal_drilldown"),
                      simplify     = TRUE,
                      fill         = "#2d6ea3",
                      fill_opacity = 0.58,
                      stroke       = "#ffffff",
                      lift_scale   = 1.16,
                      focus_padding = 40,
                      focus_size   = 0.76,
                      min_focus_width = 0,
                      min_focus_height = 0,
                      tiny_feature_threshold = 48,
                      tiny_feature_boost = 1,
                      max_zoom     = NULL,
                      origin_context = c("none", "socket", "inset", "both"),
                      origin_context_position = c("bottom-left", "bottom-right", "top-left", "top-right"),
                      focus_context_opacity = 0.3,
                      show_drag_zoom = FALSE,
                      font_size    = 14,
                      show_labels  = TRUE,
                      show_sidebar = TRUE,
                      performance_mode = NULL,
                      info_cols    = NULL,
                      info_labels  = NULL,
                      info_title   = NULL,
                      info_position = c("top-right", "top-left", "bottom-right", "bottom-left"),
                      info_card_scale = 1,
                      area_min     = 5000,
                      width_min    = 95,
                      height_min   = 28,
                      width        = "100%",
                      height       = "600px",
                      elementId    = NULL) {

  if (!missing(show_sidebar)) {
    warning(
      "`show_sidebar` in focus_map() is not implemented and has no effect. ",
      "It will be removed in a future version of explodemap.",
      call. = FALSE
    )
  }

  focus_preset <- match.arg(focus_preset)
  preset <- focus_map_preset(focus_preset)
  if (!identical(focus_preset, "none")) {
    if (missing(context_mode) && !is.null(preset$context_mode)) context_mode <- preset$context_mode
    if (missing(context_opacity) && !is.null(preset$context_opacity)) context_opacity <- preset$context_opacity
    if (missing(focus_padding) && !is.null(preset$focus_padding)) focus_padding <- preset$focus_padding
    if (missing(focus_size) && !is.null(preset$focus_size)) focus_size <- preset$focus_size
    if (missing(min_focus_width) && !is.null(preset$min_focus_width)) min_focus_width <- preset$min_focus_width
    if (missing(min_focus_height) && !is.null(preset$min_focus_height)) min_focus_height <- preset$min_focus_height
    if (missing(tiny_feature_threshold) && !is.null(preset$tiny_feature_threshold)) tiny_feature_threshold <- preset$tiny_feature_threshold
    if (missing(tiny_feature_boost) && !is.null(preset$tiny_feature_boost)) tiny_feature_boost <- preset$tiny_feature_boost
    if (missing(max_zoom) && !is.null(preset$max_zoom)) max_zoom <- preset$max_zoom
    if (missing(origin_context) && !is.null(preset$origin_context)) origin_context <- preset$origin_context
    if (missing(origin_context_position) && !is.null(preset$origin_context_position)) origin_context_position <- preset$origin_context_position
    if (missing(focus_context_opacity) && !is.null(preset$focus_context_opacity)) focus_context_opacity <- preset$focus_context_opacity
    if (missing(show_drag_zoom) && !is.null(preset$show_drag_zoom)) show_drag_zoom <- preset$show_drag_zoom
    if (missing(performance_mode) && !is.null(preset$performance_mode)) performance_mode <- preset$performance_mode
  }

  sf_obj <- .as_viewer_sf(x)
  info_position <- match.arg(info_position)
  context_mode <- match.arg(context_mode)
  origin_context <- match.arg(origin_context)
  origin_context_position <- match.arg(origin_context_position)

  # Auto-detect label column
  if (is.null(label_col)) {
    candidates <- c("NAME", "name", "Name", "LABEL", "label",
                    "NAMELSAD", "county", "County")
    label_col <- intersect(candidates, names(sf_obj))[1]
    if (is.na(label_col) || is.null(label_col)) {
      label_col <- names(sf_obj)[1]
    }
  }

  stopifnot_col <- function(col) {
    if (!col %in% names(sf_obj)) {
      stop("Column '", col, "' not found in the data.", call. = FALSE)
    }
  }
  stopifnot_col(label_col)

  if (!is.null(id_col)) {
    if (!id_col %in% names(sf_obj)) {
      warning(
        "id_col '", id_col, "' not found in the data; falling back to row order. ",
        "In Shiny, input$<outputId>_selected$id will contain sequential integers ",
        "instead of the expected geographic identifiers.",
        call. = FALSE
      )
      id_col <- NULL
    }
  }

  if (!is.null(group_col)) {
    if (!group_col %in% names(sf_obj)) {
      warning("group_col '", group_col, "' not found; ignoring.", call. = FALSE)
      group_col <- NULL
    }
  }
  if (!is.null(context_col)) {
    if (!context_col %in% names(sf_obj)) {
      warning("context_col '", context_col, "' not found; ignoring.", call. = FALSE)
      context_col <- NULL
    }
  }
  context_values <- as.character(context_values %||% "context")
  context_values <- context_values[nzchar(context_values)]
  if (!length(context_values)) context_values <- "context"
  if (!is.numeric(context_opacity) || length(context_opacity) != 1 ||
      is.na(context_opacity)) {
    stop("`context_opacity` must be a single number.", call. = FALSE)
  }
  context_opacity <- max(0, min(1, context_opacity))
  .check_scalar_number <- function(value, name, lower = -Inf, allow_null = FALSE) {
    if (is.null(value) && isTRUE(allow_null)) {
      return(NULL)
    }
    if (!is.numeric(value) || length(value) != 1 || is.na(value)) {
      stop("`", name, "` must be a single number.", call. = FALSE)
    }
    if (value < lower) {
      stop("`", name, "` must be at least ", lower, ".", call. = FALSE)
    }
    value
  }
  min_focus_width <- .check_scalar_number(min_focus_width, "min_focus_width", lower = 0)
  min_focus_height <- .check_scalar_number(min_focus_height, "min_focus_height", lower = 0)
  tiny_feature_threshold <- .check_scalar_number(
    tiny_feature_threshold,
    "tiny_feature_threshold",
    lower = 0
  )
  tiny_feature_boost <- .check_scalar_number(tiny_feature_boost, "tiny_feature_boost", lower = 1)
  focus_context_opacity <- .check_scalar_number(
    focus_context_opacity,
    "focus_context_opacity",
    lower = 0
  )
  focus_context_opacity <- min(1, focus_context_opacity)
  if (!is.null(group_palette)) {
    if (is.null(names(group_palette)) || any(!nzchar(names(group_palette)))) {
      warning("group_palette must be a named character vector; ignoring.", call. = FALSE)
      group_palette <- NULL
    } else {
      palette_names <- names(group_palette)
      group_palette <- as.list(unname(as.character(group_palette)))
      names(group_palette) <- palette_names
    }
  }

  if (is.null(info_title)) {
    info_title <- label_col
  }
  if (!is.null(info_title) && !info_title %in% names(sf_obj)) {
    warning("info_title '", info_title, "' not found; using label_col.", call. = FALSE)
    info_title <- label_col
  }
  if (!is.null(info_cols)) {
    missing_info_cols <- setdiff(info_cols, names(sf_obj))
    if (length(missing_info_cols) > 0) {
      warning(
        "Ignoring info_cols not found in data: ",
        paste(missing_info_cols, collapse = ", "),
        call. = FALSE
      )
      info_cols <- intersect(info_cols, names(sf_obj))
    }
  }
  if (!is.null(info_labels)) {
    info_labels <- unlist(info_labels, use.names = TRUE)
    if (is.null(names(info_labels)) || any(!nzchar(names(info_labels)))) {
      warning("info_labels must be a named character vector/list; ignoring.", call. = FALSE)
      info_labels <- NULL
    } else {
      info_labels <- as.character(info_labels)
    }
  }

  # Ensure WGS 84
  if (is.na(sf::st_crs(sf_obj))) {
    sf_obj <- sf::st_set_crs(sf_obj, 4326)
  } else if (!identical(sf::st_crs(sf_obj)$epsg, 4326L)) {
    sf_obj <- sf::st_transform(sf_obj, 4326)
  }

  sf_obj <- sf::st_make_valid(sf_obj)
  sf_obj <- sf_obj[!sf::st_is_empty(sf_obj), ]

  if (nrow(sf_obj) == 0) {
    stop("No non-empty geometries remain after validation.", call. = FALSE)
  }

  # Simplify geometry for rendering performance.
  # Heavier states need heavier simplification in the widget copy.
  #   < 50 features  → 0.002° (~200 m) — light touch
  #   50–150         → 0.005° (~500 m) — moderate
  #   150–220        → 0.01°  (~1 km)  — Georgia-tier
  #   > 220          → 0.02°  (~2 km)  — Texas-tier (254 counties)
  # This does NOT affect the original data — only the widget copy.
  # Pass simplify = FALSE to disable, or a custom numeric tolerance.
  if (isTRUE(simplify)) {
    n <- nrow(sf_obj)

    tol <- if (n > 3000) {
      0.05
    } else if (n > 1500) {
      0.04
    } else if (n > 500) {
      0.03
    } else if (n > 220) {
      0.02
    } else if (n > 150) {
      0.01
    } else if (n > 50) {
      0.005
    } else {
      0.002
    }

    sf_obj <- sf::st_simplify(
      sf_obj,
      preserveTopology = TRUE,
      dTolerance = tol
    )
    sf_obj <- sf_obj[!sf::st_is_empty(sf_obj), ]
  } else if (is.numeric(simplify) && length(simplify) == 1 && simplify > 0) {
    sf_obj <- sf::st_simplify(
      sf_obj,
      preserveTopology = TRUE,
      dTolerance = simplify
    )
    sf_obj <- sf_obj[!sf::st_is_empty(sf_obj), ]
  }

  geojson_str <- .sf_to_geojson_fast(
    sf_obj = sf_obj,
    label_col = label_col,
    id_col = id_col,
    group_col = group_col,
    context_col = context_col,
    info_cols = info_cols,
    info_title = info_title
  )

  payload <- list(
    geojson_str = geojson_str,
    options = list(
      fill         = fill,
      groupPalette = group_palette,
      contextMode = context_mode,
      contextValues = as.list(context_values),
      contextFill = context_fill,
      contextOpacity = context_opacity,
      contextClickable = isTRUE(context_clickable),
      focusPreset = focus_preset,
      fillOpacity  = fill_opacity,
      stroke       = stroke,
      liftScale    = lift_scale,
      focusPadding = focus_padding,
      focusSize    = focus_size,
      minFocusWidth = min_focus_width,
      minFocusHeight = min_focus_height,
      tinyFeatureThreshold = tiny_feature_threshold,
      tinyFeatureBoost = tiny_feature_boost,
      maxZoom      = max_zoom,
      originContext = origin_context,
      originContextPosition = origin_context_position,
      focusContextOpacity = focus_context_opacity,
      showDragZoom = isTRUE(show_drag_zoom),
      fontSize     = font_size,
      showLabels   = show_labels,
      performanceMode = performance_mode,
      showInfoCard = !is.null(info_cols) && length(info_cols) > 0,
      infoPosition = info_position,
      infoCols = info_cols,
      infoKeys = make.names(info_cols %||% character()),
      infoLabels = info_labels,
      infoTitle = info_title,
      infoCardScale = info_card_scale,
      areaMin      = area_min,
      widthMin     = width_min,
      heightMin    = height_min,
      hasGroups    = !is.null(group_col)
    )
  )

  htmlwidgets::createWidget(
    name      = "focusmap",
    x         = payload,
    width     = width,
    height    = height,
    package   = "explodemap",
    elementId = elementId
  )
}


#' @rdname focus_map
#' @param outputId Shiny output ID.
#' @export
focusmapOutput <- function(outputId, width = "100%", height = "600px") {
  htmlwidgets::shinyWidgetOutput(outputId, "focusmap",
                                 width, height, package = "explodemap")
}

#' @rdname focus_map
#' @param expr Expression that returns a \code{focus_map()} widget.
#' @param env Environment in which to evaluate \code{expr}.
#' @param quoted Logical. Is \code{expr} already quoted?
#' @export
renderFocusmap <- function(expr, env = parent.frame(), quoted = FALSE) {
  if (!quoted) expr <- substitute(expr)
  htmlwidgets::shinyRenderWidget(expr, focusmapOutput, env, quoted = TRUE)
}


# ── internal helpers ─────────────────────────────────────────────────────────

#' @keywords internal
.as_viewer_sf <- function(x) {
  if (inherits(x, "grouped_exploded_map")) {
    if (!is.null(x$sf_grouped_wgs)) return(x$sf_grouped_wgs)
    stop("grouped_exploded_map has no sf_grouped_wgs component.", call. = FALSE)
  }
  if (inherits(x, "exploded_map")) {
    if (!is.null(x$sf_exp_wgs)) return(x$sf_exp_wgs)
    stop("exploded_map has no sf_exp_wgs component.", call. = FALSE)
  }
  if (inherits(x, "sf")) return(x)
  stop("x must be an sf, exploded_map, or grouped_exploded_map.", call. = FALSE)
}


#' Fast bulk GeoJSON via a single sf::st_write call
#'
#' Writes the whole sf to a temp GeoJSON file and passes the raw JSON
#' string to the widget. The JS side does \code{JSON.parse()}, which
#' avoids the \code{fromJSON → R list → toJSON} round-trip that can
#' mangle deeply nested coordinate arrays.
#' @keywords internal
.sf_to_geojson_fast <- function(sf_obj, label_col, id_col = NULL, group_col = NULL,
                                context_col = NULL, info_cols = NULL,
                                info_title = label_col) {
  # Build a slim sf with only the columns the widget needs
  slim <- data.frame(
    feature_id = as.character(seq_len(nrow(sf_obj))),
    id         = if (is.null(id_col)) as.character(seq_len(nrow(sf_obj))) else as.character(sf_obj[[id_col]]),
    NAME       = as.character(sf_obj[[label_col]]),
    stringsAsFactors = FALSE
  )

  if (!is.null(group_col)) {
    slim$group <- as.character(sf_obj[[group_col]])
  }

  if (!is.null(context_col)) {
    slim$context <- as.character(sf_obj[[context_col]])
  }

  if (!is.null(info_title)) {
    slim$info_title <- as.character(sf_obj[[info_title]])
  }

  for (col in info_cols %||% character()) {
    safe_col <- make.names(col)
    slim[[paste0("info_", safe_col)]] <- as.character(sf_obj[[col]])
  }

  slim <- sf::st_sf(slim, geometry = sf::st_geometry(sf_obj))

  tmp <- tempfile(fileext = ".geojson")
  on.exit(unlink(tmp), add = TRUE)

  # RFC7946=YES forces counter-clockwise exterior rings, which is what
  # D3's spherical geometry engine expects. Without this, GDAL may write
  # clockwise rings (OGC convention), causing D3 to interpret each polygon
  # as its spherical complement — "everything on Earth except this county."
  # That is the blue-square bug.
  sf::st_write(slim, tmp, driver = "GeoJSON", quiet = TRUE,
               delete_dsn = TRUE,
               layer_options = c("RFC7946=YES", "WRITE_BBOX=NO"))

  # Return raw JSON string — JS will parse it, not R.
  # encoding = "UTF-8" is required: sf::st_write produces UTF-8 GeoJSON but
  # readLines() defaults to the system locale on Windows, which silently
  # mojibakes non-ASCII feature names (accented characters, CJK, etc.).
  paste(readLines(tmp, encoding = "UTF-8", warn = FALSE), collapse = "\n")
}

#' @keywords internal
`%||%` <- function(a, b) if (is.null(a)) b else a

# =============================================================================
# R/tiger.R — TIGER/Line download and region-attachment helpers
# =============================================================================

# Cache setup
#
# Directory resolution order:
#   1. getOption("explodemap.cache_dir")  — set this in Shiny server() to a
#      writable path on the server, e.g. options(explodemap.cache_dir = "/tmp/em")
#   2. tools::R_user_dir("explodemap", "cache")  — XDG-compliant user cache
#      (~/.cache/explodemap on Linux, ~/Library/Caches/explodemap on macOS,
#       %LOCALAPPDATA%/explodemap on Windows).
#
# Both dir creation and file writes are wrapped in tryCatch so a read-only
# or ephemeral filesystem (shinyapps.io, Posit Cloud) degrades gracefully:
# the download still works, just without caching.
.cache_dir <- function() {
  d <- getOption(
    "explodemap.cache_dir",
    tools::R_user_dir("explodemap", which = "cache")
  )
  tryCatch(
    dir.create(d, showWarnings = FALSE, recursive = TRUE),
    error = function(e) NULL
  )
  d
}
.cache_path   <- function(key) file.path(.cache_dir(), paste0(key, ".rds"))
.cache_exists <- function(key) file.exists(.cache_path(key))
.cache_load   <- function(key) readRDS(.cache_path(key))
.cache_save   <- function(key, obj) {
  # withCallingHandlers muffles the "cannot open compressed file" warning that
  # gzfile emits on Windows when the target path is unwritable. The subsequent
  # write error is still caught by the outer tryCatch so the caller always
  # gets obj back without a crash.
  tryCatch(
    withCallingHandlers(
      { saveRDS(obj, .cache_path(key)); obj },
      warning = function(w) invokeRestart("muffleWarning")
    ),
    error = function(e) {
      message(
        "explodemap: cache write failed (", conditionMessage(e),
        "); proceeding without caching."
      )
      obj
    }
  )
}

.download_file_or_stop <- function(url, dest, label) {
  status <- tryCatch(
    utils::download.file(url, dest, quiet = TRUE, mode = "wb"),
    error = function(e) {
      stop(
        "Could not download ", label, " from the Census TIGER/Line server. ",
        "Check your internet connection and try again. ",
        "Details: ", conditionMessage(e),
        call. = FALSE
      )
    }
  )

  if (!identical(status, 0L)) {
    stop(
      "Could not download ", label, " from the Census TIGER/Line server. ",
      "Check your internet connection and try again.",
      call. = FALSE
    )
  }

  if (!file.exists(dest) || isTRUE(file.info(dest)$size == 0)) {
    stop(
      "Downloaded ", label, " but the file was empty. ",
      "The Census TIGER/Line server may be unavailable; please try again later.",
      call. = FALSE
    )
  }

  invisible(dest)
}


#' Download TIGER/Line county subdivision boundaries
#' @param fips 2-digit state FIPS code
#' @param crs Target projected CRS
#' @return sf object
#' @keywords internal
.download_cousub <- function(fips, crs) {
  if (!is.character(fips) || length(fips) != 1 || !grepl("^[0-9]{2}$", fips)) {
    stop("`state_fips` must be a 2-digit character string, such as \"34\" for New Jersey.",
         call. = FALSE)
  }

  key <- paste0("cousub_", fips)
  if (.cache_exists(key)) {
    obj <- .cache_load(key)
    if (!identical(sf::st_crs(obj)$epsg, as.integer(crs)))
      obj <- sf::st_transform(obj, crs)
    return(obj)
  }
  url <- paste0("https://www2.census.gov/geo/tiger/TIGER2024/COUSUB/",
                "tl_2024_", fips, "_cousub.zip")
  tmp <- tempfile(fileext = ".zip")
  .download_file_or_stop(url, tmp, paste0("county subdivisions for FIPS ", fips))
  dir <- file.path(tempdir(), paste0("cousub_", fips, "_", Sys.getpid()))
  dir.create(dir, showWarnings = FALSE)
  tryCatch(
    utils::unzip(tmp, exdir = dir),
    error = function(e) {
      stop(
        "Could not unzip county subdivision data for FIPS ", fips, ". ",
        "The downloaded file may be incomplete. Please try again. ",
        "Details: ", conditionMessage(e),
        call. = FALSE
      )
    }
  )
  shp <- list.files(dir, pattern = "\\.shp$", full.names = TRUE, recursive = TRUE)
  if (!length(shp)) {
    stop(
      "County subdivision data for FIPS ", fips, " did not contain a shapefile. ",
      "The Census download may have changed or failed.",
      call. = FALSE
    )
  }
  obj <- sf::st_read(shp[1], quiet = TRUE) |>
    dplyr::filter(.data$COUSUBFP != "00000") |>
    sf::st_transform(crs)
  .cache_save(key, obj)
}


#' Download national county name-to-FIPS lookup
#' @keywords internal
.get_national_counties <- function() {
  key <- "counties_national"
  if (.cache_exists(key)) return(.cache_load(key))
  url <- "https://www2.census.gov/geo/tiger/TIGER2024/COUNTY/tl_2024_us_county.zip"
  tmp <- tempfile(fileext = ".zip")
  .download_file_or_stop(url, tmp, "national county lookup")
  dir <- file.path(tempdir(), paste0("county_nat_", Sys.getpid()))
  dir.create(dir, showWarnings = FALSE)
  tryCatch(
    utils::unzip(tmp, exdir = dir),
    error = function(e) {
      stop(
        "Could not unzip the national county lookup. ",
        "The downloaded file may be incomplete. Please try again. ",
        "Details: ", conditionMessage(e),
        call. = FALSE
      )
    }
  )
  shp <- list.files(dir, pattern = "\\.shp$", full.names = TRUE, recursive = TRUE)
  if (!length(shp)) {
    stop(
      "The national county lookup download did not contain a shapefile. ",
      "The Census download may have changed or failed.",
      call. = FALSE
    )
  }
  obj <- sf::st_read(shp[1], quiet = TRUE) |>
    sf::st_drop_geometry() |>
    dplyr::select("STATEFP", "COUNTYFP", "NAME")
  .cache_save(key, obj)
}


#' Attach region labels to TIGER/Line data via county names
#' @keywords internal
.attach_regions_tiger <- function(sf_obj, fips, region_map, quiet = FALSE) {
  region_df <- dplyr::bind_rows(lapply(names(region_map), function(r)
    data.frame(county_name = region_map[[r]], region = r,
               stringsAsFactors = FALSE)))

  counties <- .get_national_counties() |>
    dplyr::filter(.data$STATEFP == fips) |>
    dplyr::select("COUNTYFP", "NAME")

  county_region <- counties |>
    dplyr::left_join(region_df, by = c("NAME" = "county_name"))

  sf_result <- sf_obj |>
    dplyr::left_join(county_region |> dplyr::select("COUNTYFP", "region"),
                     by = "COUNTYFP")

  n_matched <- sum(!is.na(sf_result$region))
  if (!quiet)
    message("Region assignment: ", n_matched, " / ", nrow(sf_result), " units matched.")

  unmatched <- counties$NAME[!counties$NAME %in% region_df$county_name]
  if (!quiet && length(unmatched) > 0)
    message("Unmatched counties: ",
            paste(utils::head(unmatched, 8), collapse = ", "),
            if (length(unmatched) > 8) paste0("... +", length(unmatched) - 8, " more"))

  sf_result$region[is.na(sf_result$region)] <- "Other"
  sf_result
}


#' Clear explodemap download cache
#'
#' @param key Specific cache key to clear, or NULL to clear all
#' @return Invisibly returns `NULL`. The function is called for its side effect
#'   of deleting cached `.rds` files and reporting what was removed.
#' @export
cache_clear <- function(key = NULL) {
  cache_dir <- .cache_dir()
  if (is.null(key)) {
    files <- list.files(cache_dir, pattern = "\\.rds$", full.names = TRUE)
    unlink(files)
    message("Cache cleared: ", length(files), " files removed from ", cache_dir)
  } else {
    p <- .cache_path(key)
    if (file.exists(p)) { unlink(p); message("Cleared: ", key) }
    else message("No cache found for: ", key)
  }
  invisible(NULL)
}

#' List cached datasets
#'
#' @return Invisibly returns a character vector of cached `.rds` file names.
#'   The visible output is a message listing the cache directory and file sizes,
#'   or a message that the cache is empty.
#' @export
cache_list <- function() {
  cache_dir <- .cache_dir()
  files <- list.files(cache_dir, pattern = "\\.rds$")
  if (length(files) == 0) {
    message("Cache is empty.")
  } else {
    sizes <- file.size(file.path(cache_dir, files))
    msg <- paste(sprintf("  %-40s  %.1f MB", files, sizes / 1e6), collapse = "\n")
    message("Cached datasets in ", cache_dir, ":\n", msg)
  }
  invisible(files)
}

# =============================================================================
# R/grouped.R — Three-Level Grouped Layout Engine (Section 12)
# =============================================================================
#
# Extends the core two-level displacement (Algorithm 1) with a third level
# for national or multi-region grouped layouts. Implements Algorithm 2 from:
#
#   "A Hierarchical Vector-Based Framework for Multi-Scale Exploded-View
#    Cartography" — George Arthur, Section 12
#
# THREE-LEVEL HIERARCHY
# ─────────────────────
# Level 1: Deterministic local explosion (Algorithm 1, unchanged)
#          Units within each parent region are displaced using the
#          centroid-driven field. Geometry guarantees (Props 1-3) apply.
#
# Level 2: Automatic anchor placement
#          Parent region centroids are displaced radially outward from
#          the national centroid to generate initial anchor positions.
#
# Level 3: Collision-aware anchor refinement
#          Overlapping anchors are iteratively repelled while a spring
#          term maintains proximity to original radial targets.
#
# ANCHOR MODES
# ─────────────
# "auto"            — Level 2 only (radial placement, no collision resolution)
# "auto_collision"  — Level 2 + Level 3 (radial + spring-repulsion solver)
# "manual"          — User-supplied anchor positions
#
# =============================================================================


# ─────────────────────────────────────────────────────────────────────────────
# BLOCK RADIUS ESTIMATION
# ─────────────────────────────────────────────────────────────────────────────

#' Estimate block radius for each region
#'
#' Computes the 85th percentile of distances from child-unit centroids to
#' their region centroid. The 85th percentile is preferred over the maximum
#' because the maximum is sensitive to outlier units and produces
#' over-conservative block radii.
#'
#' @param sf_obj Projected sf object with region column
#' @param region_col Grouping column name
#' @param quantile_p Quantile for radius estimation (default 0.85)
#' @param centroid_fun "centroid" or "point_on_surface"
#' @return data.frame with columns: region, block_radius, cx, cy, n_units
#' @export
estimate_block_radii <- function(sf_obj, region_col,
                                 quantile_p = 0.85,
                                 centroid_fun = c("centroid", "point_on_surface")) {
  centroid_fun <- match.arg(centroid_fun)

  reg_sf <- sf_obj |>
    dplyr::group_by(dplyr::across(dplyr::all_of(region_col))) |>
    dplyr::summarise(geometry = sf::st_union(.data$geometry), .groups = "drop")

  rc <- sf::st_coordinates(centroid_geoms(reg_sf, centroid_fun))

  reg_df <- data.frame(
    region = reg_sf[[region_col]],
    rx = rc[, 1],
    ry = rc[, 2],
    stringsAsFactors = FALSE
  )
  names(reg_df)[1] <- region_col

  unit_xy <- sf::st_coordinates(centroid_geoms(sf_obj, centroid_fun))

  tmp <- sf_obj |>
    sf::st_drop_geometry() |>
    dplyr::mutate(
      .cx = unit_xy[, 1],
      .cy = unit_xy[, 2]
    ) |>
    dplyr::left_join(reg_df, by = region_col) |>
    dplyr::mutate(
      .dist = sqrt((.data$.cx - .data$rx)^2 + (.data$.cy - .data$ry)^2)
    )

  out <- tmp |>
    dplyr::group_by(dplyr::across(dplyr::all_of(region_col))) |>
    dplyr::summarise(
      block_radius = as.numeric(stats::quantile(.data$.dist, probs = quantile_p, na.rm = TRUE)),
      n_units = dplyr::n(),
      .groups = "drop"
    ) |>
    dplyr::left_join(reg_df, by = region_col)

  names(out)[names(out) == "rx"] <- "cx"
  names(out)[names(out) == "ry"] <- "cy"

  out
}


# ─────────────────────────────────────────────────────────────────────────────
# ANCHOR GENERATION
# ─────────────────────────────────────────────────────────────────────────────

#' Generate initial radial anchors from the national centroid
#' @keywords internal
.generate_anchors <- function(block_df, nat_centroid,
                              kappa = 1.8,
                              padding = 50000,
                              delta = 15000) {
  stopifnot(all(c("cx", "cy", "block_radius", "n_units") %in% names(block_df)))

  dx <- block_df$cx - nat_centroid[1]
  dy <- block_df$cy - nat_centroid[2]
  r0 <- sqrt(dx^2 + dy^2)

  # Handle regions whose centroid is exactly at the national centroid
  zero_idx <- which(r0 == 0 | is.na(r0))
  if (length(zero_idx) > 0) {
    angles0 <- seq(0, 2 * pi, length.out = length(zero_idx) + 1)[seq_along(zero_idx)]
    dx[zero_idx] <- cos(angles0)
    dy[zero_idx] <- sin(angles0)
    r0[zero_idx] <- 1
  }

  ux <- dx / r0
  uy <- dy / r0

  target_r <- kappa * r0 + padding + delta * log1p(block_df$n_units)

  block_df$anchor_x <- nat_centroid[1] + ux * target_r
  block_df$anchor_y <- nat_centroid[2] + uy * target_r
  block_df$target_x <- block_df$anchor_x
  block_df$target_y <- block_df$anchor_y
  block_df
}


# ─────────────────────────────────────────────────────────────────────────────
# COLLISION-AWARE ANCHOR REFINEMENT
# ─────────────────────────────────────────────────────────────────────────────

#' Refine anchors with pairwise repulsion plus spring return
#' @keywords internal
.refine_anchors <- function(block_df,
                            lambda = 0.18,
                            eta = 0.18,
                            padding_sep = 20000,
                            max_iter = 60) {
  stopifnot(all(c("anchor_x", "anchor_y", "block_radius", "target_x", "target_y") %in% names(block_df)))

  n <- nrow(block_df)
  if (n <= 1) {
    block_df$.converged <- TRUE
    block_df$.iterations <- 0L
    return(block_df)
  }

  converged <- FALSE
  iter <- 0L

  for (iter in seq_len(max_iter)) {
    moved <- FALSE

    # spring-to-target term
    block_df$anchor_x <- block_df$anchor_x + lambda * (block_df$target_x - block_df$anchor_x)
    block_df$anchor_y <- block_df$anchor_y + lambda * (block_df$target_y - block_df$anchor_y)

    for (i in seq_len(n - 1)) {
      for (j in seq.int(i + 1, n)) {
        dx <- block_df$anchor_x[j] - block_df$anchor_x[i]
        dy <- block_df$anchor_y[j] - block_df$anchor_y[i]
        d <- sqrt(dx^2 + dy^2)

        min_sep <- block_df$block_radius[i] + block_df$block_radius[j] + padding_sep

        if (is.na(d) || d == 0) {
          theta <- ((i * 73856093 + j * 19349663) %% 360) * pi / 180
          dx <- cos(theta)
          dy <- sin(theta)
          d <- 1
        }

        if (d < min_sep) {
          overlap <- min_sep - d
          ux <- dx / d
          uy <- dy / d
          step <- 0.5 * eta * overlap

          block_df$anchor_x[i] <- block_df$anchor_x[i] - step * ux
          block_df$anchor_y[i] <- block_df$anchor_y[i] - step * uy
          block_df$anchor_x[j] <- block_df$anchor_x[j] + step * ux
          block_df$anchor_y[j] <- block_df$anchor_y[j] + step * uy

          moved <- TRUE
        }
      }
    }

    if (!moved) {
      converged <- TRUE
      break
    }
  }

  block_df$.converged <- converged
  block_df$.iterations <- iter
  block_df
}


# ─────────────────────────────────────────────────────────────────────────────
# ANCHOR LAYOUT ORCHESTRATOR
# ─────────────────────────────────────────────────────────────────────────────

#' Compute region anchor positions for grouped layouts
#'
#' Implements the anchor layout procedure from Section 12.
#' Supports three modes: automatic radial placement, automatic with
#' collision resolution, and manual user-specified anchors.
#'
#' @param sf_obj Projected sf object with region column
#' @param region_col Grouping column name
#' @param mode "auto" (radial only), "auto_collision" (radial + solver), or "manual"
#' @param anchors For mode = "manual": data.frame with columns (region_col, anchor_x, anchor_y)
#' @param kappa Radial expansion factor (default 1.8)
#' @param padding Base padding in map units (default 50000)
#' @param delta Log-density scaling factor (default 15000)
#' @param lambda Spring coefficient for collision solver (default 0.18)
#' @param eta Repulsion step size for collision solver (default 0.18)
#' @param padding_sep Minimum separation between blocks (default 20000)
#' @param max_iter Max iterations for collision solver (default 60)
#' @param quantile_p Quantile for block radius estimation (default 0.85)
#' @param centroid_fun "centroid" or "point_on_surface"
#' @param quiet If `TRUE`, suppress `message()` output. Default `FALSE`.
#' @return data.frame with region, anchor_x, anchor_y, block_radius, n_units
#' @export
layout_regions <- function(sf_obj, region_col,
                           mode = c("auto", "auto_collision", "manual"),
                           anchors = NULL,
                           kappa = 1.8,
                           padding = 50000,
                           delta = 15000,
                           lambda = 0.18,
                           eta = 0.18,
                           padding_sep = 20000,
                           max_iter = 60,
                           quantile_p = 0.85,
                           centroid_fun = c("centroid", "point_on_surface"),
                           quiet = FALSE) {
  mode <- match.arg(mode)
  centroid_fun <- match.arg(centroid_fun)

  if (mode == "manual") {
    if (is.null(anchors)) {
      stop("mode = 'manual' requires an `anchors` data.frame.", call. = FALSE)
    }
    if (!all(c(region_col, "anchor_x", "anchor_y") %in% names(anchors))) {
      stop("`anchors` must contain columns: ", region_col, ", anchor_x, anchor_y",
           call. = FALSE)
    }
    if (anyDuplicated(anchors[[region_col]]) > 0) {
      stop("`anchors` must contain at most one row per region.", call. = FALSE)
    }

    block_df <- estimate_block_radii(sf_obj, region_col, quantile_p, centroid_fun)

    result <- block_df |>
      dplyr::left_join(
        anchors |>
          dplyr::select(dplyr::all_of(c(region_col, "anchor_x", "anchor_y"))),
        by = region_col
      )

    missing_anchor <- is.na(result$anchor_x) | is.na(result$anchor_y)
    if (any(missing_anchor)) {
      stop(
        "Missing manual anchors for region(s): ",
        paste(result[[region_col]][missing_anchor], collapse = ", "),
        call. = FALSE
      )
    }

    return(result)
  }

  block_df <- estimate_block_radii(sf_obj, region_col, quantile_p, centroid_fun)

  nat_centroid <- sf::st_coordinates(
    centroid_geoms(sf::st_union(sf::st_geometry(sf_obj)), centroid_fun)
  )[1, ]

  block_df <- .generate_anchors(block_df, nat_centroid, kappa, padding, delta)

  if (mode == "auto_collision") {
    block_df <- .refine_anchors(block_df, lambda, eta, padding_sep, max_iter)

    if (!quiet && !is.null(block_df$.converged)) {
      if (isTRUE(block_df$.converged[1])) {
        message("Anchor solver converged in ", block_df$.iterations[1], " iterations.")
      } else {
        message(
          "Anchor solver reached max iterations (", max_iter,
          "). Layout may have residual overlaps."
        )
      }
    }
  }

  block_df
}


# ─────────────────────────────────────────────────────────────────────────────
# PUBLIC API: GROUPED EXPLODED MAP
# ─────────────────────────────────────────────────────────────────────────────

#' Create a three-level grouped exploded map
#'
#' Combines Level 1 (local explosion within regions) with Level 2/3
#' (anchor-based region block placement). This is the full three-level
#' extension from Section 12 of the paper.
#'
#' The guarantees of Propositions 1-3 apply strictly at Level 1.
#' Higher levels preserve structural grouping and directional correspondence
#' rather than topological coverage.
#'
#' @param sf_obj Projected sf object with region column
#' @param region_col Grouping column name
#' @param mode "auto", "auto_collision", or "manual"
#' @param anchors For mode = "manual": data.frame with anchor positions
#' @param alpha_l Local expansion parameter for Level 1 (metres)
#' @param p Distance scaling exponent (default 1.25)
#' @param gamma_l Local clearance coefficient (default 1.136); used if alpha_l is NULL
#' @param kappa Radial expansion factor (default 1.8)
#' @param padding Base padding (default 50000)
#' @param delta Log-density scaling (default 15000)
#' @param lambda Spring coefficient (default 0.18)
#' @param eta Repulsion step (default 0.18)
#' @param padding_sep Minimum block separation (default 20000)
#' @param anchor_expand,anchor_buffer,density_scale,block_sep Optional aliases
#'   for `kappa`, `padding`, `delta`, and `padding_sep`, respectively. These
#'   names are convenient in Shiny dashboards where the controls describe the
#'   visual effect rather than the solver term.
#' @param max_iter Max collision iterations (default 60)
#' @param fix_invalid Auto-repair invalid geometries (default TRUE)
#' @param centroid_fun "centroid" or "point_on_surface"
#' @param plot Print plots (default TRUE). Automatically suppressed inside a
#'   live Shiny session; use [plot.grouped_exploded_map()] inside `renderPlot()`.
#' @param export NULL, TRUE, or file path
#' @param label Title for plots
#' @param quiet If `TRUE`, suppress `message()` output. Default `FALSE`.
#' @return A `grouped_exploded_map` S3 object (inherits from `exploded_map`)
#' @export
explode_grouped <- function(sf_obj, region_col,
                            mode         = c("auto", "auto_collision", "manual"),
                            anchors      = NULL,
                            alpha_l      = NULL,
                            p            = 1.25,
                            gamma_l      = 1.136,
                            kappa        = 1.8,
                            padding      = 50000,
                            delta        = 15000,
                            lambda       = 0.18,
                            eta          = 0.18,
                            padding_sep  = 20000,
                            anchor_expand = NULL,
                            anchor_buffer = NULL,
                            density_scale = NULL,
                            block_sep     = NULL,
                            max_iter     = 60,
                            fix_invalid  = TRUE,
                            centroid_fun = c("centroid", "point_on_surface"),
                            plot         = TRUE,
                            export       = NULL,
                            label        = "Grouped Layout",
                            quiet        = FALSE) {
  mode <- match.arg(mode)
  centroid_fun <- match.arg(centroid_fun)

  if (!is.null(anchor_expand)) kappa <- anchor_expand
  if (!is.null(anchor_buffer)) padding <- anchor_buffer
  if (!is.null(density_scale)) delta <- density_scale
  if (!is.null(block_sep)) padding_sep <- block_sep

  sf_obj <- validate_input(
    sf_obj,
    region_col,
    allow_other = TRUE,
    fix_invalid = fix_invalid
  )

  sf_clean <- sf_obj[sf_obj[[region_col]] != "Other", ]
  stats <- compute_stats(sf_clean, region_col, centroid_fun = centroid_fun)

  if (is.null(alpha_l)) {
    alpha_l <- gamma_l * 2 * stats$R_local / sqrt(stats$n_bar)
  }

  if (!quiet)
    message("Level 1: Applying local explosion (alpha_l = ", round(alpha_l), " m)...")
  sf_local <- explode_sf_core(
    sf_obj,
    region_col,
    alpha_r = 0,
    alpha_l = alpha_l,
    p = p,
    centroid_fun = centroid_fun
  )

  if (!quiet)
    message(
      "Level 2", if (mode == "auto_collision") "/3" else "",
      ": Computing anchor positions (mode = ", mode, ")..."
    )

  anchor_df <- layout_regions(
    sf_local,
    region_col,
    mode = mode,
    anchors = anchors,
    kappa = kappa,
    padding = padding,
    delta = delta,
    lambda = lambda,
    eta = eta,
    padding_sep = padding_sep,
    max_iter = max_iter,
    centroid_fun = centroid_fun,
    quiet = quiet
  )

  if (!quiet) message("Applying anchor displacement...")

  reg_sf <- sf_local |>
    dplyr::group_by(dplyr::across(dplyr::all_of(region_col))) |>
    dplyr::summarise(geometry = sf::st_union(.data$geometry), .groups = "drop")

  rc_now <- sf::st_coordinates(centroid_geoms(reg_sf, centroid_fun))

  current_centroids <- data.frame(
    region = reg_sf[[region_col]],
    cx_now = rc_now[, 1],
    cy_now = rc_now[, 2],
    stringsAsFactors = FALSE
  )
  names(current_centroids)[1] <- region_col

  anchor_offsets <- anchor_df |>
    dplyr::select(dplyr::all_of(c(region_col, "anchor_x", "anchor_y"))) |>
    dplyr::left_join(current_centroids, by = region_col) |>
    dplyr::mutate(
      off_x = .data$anchor_x - .data$cx_now,
      off_y = .data$anchor_y - .data$cy_now
    )

  orig_crs <- sf::st_crs(sf_local)
  unit_regions <- sf_local[[region_col]]

  offset_lookup <- stats::setNames(
    split(anchor_offsets[, c("off_x", "off_y")], anchor_offsets[[region_col]]),
    anchor_offsets[[region_col]]
  )

  new_geoms <- purrr::map2(
    sf::st_geometry(sf_local),
    unit_regions,
    function(geom, reg) {
      offs <- offset_lookup[[reg]]
      if (is.null(offs)) return(geom)
      geom + c(offs$off_x[1], offs$off_y[1])
    }
  )

  sf_grouped <- sf_local
  sf_grouped$geometry <- sf::st_sfc(new_geoms, crs = orig_crs)
  sf_grouped <- sf::st_as_sf(sf_grouped)
  sf_grouped_wgs <- sf::st_transform(sf_grouped, 4326)

  params <- list(
    alpha_l = alpha_l,
    p = p,
    gamma_l = gamma_l,
    mode = mode,
    kappa = kappa,
    padding = padding,
    delta = delta,
    lambda = lambda,
    eta = eta,
    padding_sep = padding_sep
  )

  plots <- .make_grouped_plots(
    sf_orig = sf_obj,
    sf_local = sf_local,
    sf_grouped = sf_grouped,
    region_col = region_col,
    title = label,
    params = params
  )

  if (isTRUE(plot)) {
    if (.shiny_is_running()) {
      message(
        "explodemap: auto-plot suppressed inside Shiny. ",
        "Use plot(result) inside renderPlot() to display the maps."
      )
    } else {
      print(plots$grouped)
    }
  }

  .handle_export(export, sf_grouped_wgs, label)

  result <- list(
    sf_orig        = sf_obj,
    sf_local       = sf_local,
    sf_grouped     = sf_grouped,
    sf_grouped_wgs = sf_grouped_wgs,
    stats          = stats,
    params         = params,
    anchors        = anchor_df,
    plots          = plots,
    diagnostics    = list(
      label        = label,
      region_col   = region_col,
      centroid_fun = centroid_fun,
      mode         = mode
    )
  )

  structure(result, class = c("grouped_exploded_map", "exploded_map", "list"))
}


# ─────────────────────────────────────────────────────────────────────────────
# PLOTTING FOR GROUPED LAYOUTS
# ─────────────────────────────────────────────────────────────────────────────

#' Build plots for grouped layouts
#' @keywords internal
.make_grouped_plots <- function(sf_orig, sf_local, sf_grouped, region_col, title, params) {
  regions <- sort(unique(sf_orig[[region_col]]))

  pal <- stats::setNames(
    grDevices::colorRampPalette(
      c(
        "#2166ac", "#d73027", "#1a9850", "#984ea3",
        "#ff7f00", "#a65628", "#f781bf", "#999999",
        "#33a02c", "#b2df8a"
      )
    )(length(regions)),
    regions
  )

  subtitle <- paste0(
    "mode=", params$mode,
    "  alpha_l=", round(params$alpha_l), "m",
    "  p=", params$p
  )

  p_orig <- ggplot2::ggplot(sf::st_transform(sf_orig, 4326)) +
    ggplot2::geom_sf(
      ggplot2::aes(fill = .data[[region_col]]),
      color = "white",
      linewidth = 0.2
    ) +
    ggplot2::scale_fill_manual(values = pal, name = region_col) +
    ggplot2::theme_void(base_size = 11) +
    ggplot2::labs(title = paste(title, "- Original"))

  p_local <- ggplot2::ggplot(sf::st_transform(sf_local, 4326)) +
    ggplot2::geom_sf(
      ggplot2::aes(fill = .data[[region_col]]),
      color = "white",
      linewidth = 0.2
    ) +
    ggplot2::scale_fill_manual(values = pal, name = region_col) +
    ggplot2::theme_void(base_size = 11) +
    ggplot2::labs(title = paste(title, "- Local"))

  p_grouped <- ggplot2::ggplot(sf::st_transform(sf_grouped, 4326)) +
    ggplot2::geom_sf(
      ggplot2::aes(fill = .data[[region_col]]),
      color = "white",
      linewidth = 0.2
    ) +
    ggplot2::scale_fill_manual(values = pal, name = region_col) +
    ggplot2::theme_void(base_size = 11) +
    ggplot2::labs(title = paste(title, "- Grouped"), subtitle = subtitle)

  list(
    orig = p_orig,
    local = p_local,
    grouped = p_grouped
  )
}

test_that("layout_regions returns anchors for auto mode", {
  x <- make_grouped_sf()

  anchors <- layout_regions(x, region_col = "region", mode = "auto")

  expect_s3_class(anchors, "data.frame")
  expect_true(all(c("region", "anchor_x", "anchor_y", "block_radius", "n_units") %in% names(anchors)))
  expect_equal(nrow(anchors), 3)
})

test_that("layout_regions manual mode requires anchor columns", {
  x <- make_grouped_sf()

  bad_anchors <- data.frame(region = c("R1", "R2", "R3"))

  expect_error(
    layout_regions(x, region_col = "region", mode = "manual", anchors = bad_anchors),
    "anchor_x"
  )
})

test_that("layout_regions manual mode merges supplied anchors", {
  x <- make_grouped_sf()

  anchors_in <- data.frame(
    region = c("R1", "R2", "R3"),
    anchor_x = c(0, 100, 200),
    anchor_y = c(0, 50, 100)
  )

  anchors <- layout_regions(
    x,
    region_col = "region",
    mode = "manual",
    anchors = anchors_in
  )

  expect_equal(anchors$anchor_x, c(0, 100, 200))
  expect_equal(anchors$anchor_y, c(0, 50, 100))
})

test_that("layout_regions manual mode requires anchors for every region", {
  x <- make_grouped_sf()

  anchors_in <- data.frame(
    region = c("R1", "R2"),
    anchor_x = c(0, 100),
    anchor_y = c(0, 50)
  )

  expect_error(
    layout_regions(
      x,
      region_col = "region",
      mode = "manual",
      anchors = anchors_in
    ),
    "Missing manual anchors"
  )
})

test_that("layout_regions collision mode is deterministic for coincident anchors", {
  block_df <- data.frame(
    region = c("R1", "R2"),
    anchor_x = c(0, 0),
    anchor_y = c(0, 0),
    target_x = c(0, 0),
    target_y = c(0, 0),
    block_radius = c(10, 10)
  )

  a <- .refine_anchors(block_df, max_iter = 2)
  b <- .refine_anchors(block_df, max_iter = 2)

  expect_equal(a$anchor_x, b$anchor_x)
  expect_equal(a$anchor_y, b$anchor_y)
})

library(shiny)
library(sf)
library(dplyr)
library(tigris)
library(explodemap)

options(tigris_use_cache = TRUE)

states <- data.frame(
  name = c("New Jersey", "Texas", "Georgia", "California", "Pennsylvania"),
  abbr = c("NJ", "TX", "GA", "CA", "PA"),
  stringsAsFactors = FALSE
)

app_error <- function(action, error) {
  paste0(
    "We could not ", action, ". ",
    conditionMessage(error),
    " Check your internet connection, try a smaller boundary file, or choose a different state."
  )
}

load_counties <- function(state_abbr, cb = TRUE, year = 2024) {
  out <- tigris::counties(state = state_abbr, year = year, cb = cb, class = "sf") |>
    dplyr::filter(!sf::st_is_empty(.data$geometry)) |>
    dplyr::mutate(
      county_label = dplyr::coalesce(.data$NAMELSAD, .data$NAME),
      county_geoid = .data$GEOID,
      county_id = paste0(.data$STATEFP, .data$COUNTYFP)
    )

  if (!nrow(out)) {
    stop(
      "No counties were returned for ", state_abbr, ". ",
      "Try another state or turn off cartographic boundary files.",
      call. = FALSE
    )
  }

  out
}

assign_quadrant_regions <- function(sf_obj) {
  centroids <- suppressWarnings(sf::st_centroid(sf_obj))
  coords <- sf::st_coordinates(centroids)
  cx <- stats::median(coords[, 1])
  cy <- stats::median(coords[, 2])
  dx <- coords[, 1] - cx
  dy <- coords[, 2] - cy

  sf_obj$region <- dplyr::case_when(
    dy >= 0 & abs(dy) >= abs(dx) ~ "North",
    dy <  0 & abs(dy) >= abs(dx) ~ "South",
    dx >= 0 & abs(dx) >  abs(dy) ~ "East",
    TRUE ~ "West"
  )
  sf_obj
}

ui <- fluidPage(
  tags$head(
    tags$style(HTML("
      .container-fluid { padding: 0; }
      .focus-shell { display: grid; grid-template-columns: 300px minmax(0, 1fr); height: 100vh; }
      .focus-controls { padding: 18px; border-right: 1px solid #dbe6f1; overflow-y: auto; background: #fff; }
      .focus-controls h2 { margin-top: 0; font-size: 20px; }
      .focus-map { min-width: 0; min-height: 0; padding: 12px; background: #f5f8fc; }
      .focus-map .html-widget, .focus-map .shiny-html-output { width: 100% !important; height: 100% !important; }
      @media (max-width: 900px) {
        .focus-shell { grid-template-columns: 1fr; grid-template-rows: auto minmax(560px, 1fr); height: auto; min-height: 100vh; }
        .focus-controls { border-right: 0; border-bottom: 1px solid #dbe6f1; }
        .focus-map { height: 75vh; }
      }
      .shiny-output-error-validation {
        color: #713f12;
        background: #fff7ed;
        border: 1px solid #fed7aa;
        border-radius: 6px;
        padding: 14px 16px;
        margin: 12px;
        font-size: 15px;
        line-height: 1.4;
      }
    "))
  ),
  div(
    class = "focus-shell",
    div(
      class = "focus-controls",
      h2("focus_map: counties"),
      selectInput("state", "State", choices = stats::setNames(states$abbr, states$name), selected = "TX"),
      checkboxInput("cb", "Use cartographic boundary file", value = TRUE),
      radioButtons("mode", "Mode", choices = c("Raw" = "raw", "Exploded" = "exploded"), inline = TRUE),
      sliderInput("font_size", "Focused label size", min = 4, max = 16, value = 8, step = 1),
      sliderInput("lift_scale", "Lift scale", min = 1.00, max = 1.35, value = 1.12, step = 0.01),
      sliderInput("focus_size", "Zoomed area size", min = 0.35, max = 1.30, value = 1.17, step = 0.01),
      sliderInput("focus_padding", "Focus padding", min = 12, max = 120, value = 48, step = 4),
      sliderInput("info_card_scale", "Info card size", min = 0.75, max = 1.60, value = 1.60, step = 0.05),
      checkboxInput("show_labels", "Show focused labels", value = TRUE),
      helpText("Zoomed area size controls how much of the viewport the selected area may occupy. Focus padding keeps the boundary away from the edge; lower it if a high zoomed-area size stops getting bigger.")
    ),
    div(
      class = "focus-map",
      focusmapOutput("map", height = "100%")
    )
  )
)

server <- function(input, output, session) {
  counties <- reactive({
    tryCatch(
      load_counties(input$state, cb = isTRUE(input$cb)),
      error = function(e) {
        validate(need(FALSE, app_error("load county boundaries", e)))
      }
    )
  }) |>
    bindCache(input$state, input$cb)

  map_data <- reactive({
    x <- counties()

    if (identical(input$mode, "exploded")) {
      validate(need(
        nrow(x) >= 3,
        "Exploded mode needs at least three mapped areas. Choose a state with more counties or switch to Raw mode."
      ))

      result <- tryCatch(
        {
          projected <- x |>
            sf::st_transform(5070) |>
            assign_quadrant_regions()

          explode_sf(
            projected,
            region_col = "region",
            plot = FALSE,
            quiet = TRUE,
            label = paste(input$state, "counties")
          )
        },
        error = function(e) {
          validate(need(FALSE, app_error("create the exploded map", e)))
        }
      )

      list(x = result, label_col = NULL, group_col = "region")
    } else {
      list(x = x, label_col = "county_label", group_col = NULL)
    }
  })

  output$map <- renderFocusmap({
    md <- map_data()
    tryCatch(
      focus_map(
        md$x,
        label_col = md$label_col,
        id_col = "county_geoid",
        group_col = md$group_col,
        info_cols = c("county_label", "county_geoid"),
        info_labels = c(county_label = "County", county_geoid = "GEOID"),
        info_title = "county_label",
        font_size = input$font_size,
        lift_scale = input$lift_scale,
        focus_size = input$focus_size,
        focus_padding = input$focus_padding,
        info_card_scale = input$info_card_scale,
        show_labels = isTRUE(input$show_labels),
        performance_mode = NULL,
        width = "100%",
        height = "100%"
      ),
      error = function(e) {
        validate(need(FALSE, app_error("render the interactive map", e)))
      }
    )
  })
}

shinyApp(ui, server)

# =============================================================================
# R/core.R — Core algorithm for exploded-view cartography
# =============================================================================
# Single source of truth for:
#   - Shared centroid helper (warning-free)
#   - Input validation
#   - Geometry statistics (compute_stats)
#   - Parameter derivation (Analytical Results 1 & 2)
#   - Displacement engine (explode_sf_core)
#   - Print formatting helper
# =============================================================================


# ─────────────────────────────────────────────────────────────────────────────
# SHARED CENTROID HELPER — use everywhere instead of raw st_centroid()
# ─────────────────────────────────────────────────────────────────────────────

#' Compute centroids without attribute warnings
#'
#' Wraps sf::st_centroid / sf::st_point_on_surface, operating on
#' st_geometry() rather than the full sf object. Returns an sfc column.
#'
#' @param x An sf or sfc object
#' @param centroid_fun "centroid" (default) or "point_on_surface"
#' @return sfc geometry column of point centroids
#' @keywords internal
centroid_geoms <- function(x, centroid_fun = c("centroid", "point_on_surface")) {
  centroid_fun <- match.arg(centroid_fun)
  geom <- sf::st_geometry(x)
  switch(
    centroid_fun,
    centroid         = sf::st_centroid(geom),
    point_on_surface = sf::st_point_on_surface(geom)
  )
}


# ─────────────────────────────────────────────────────────────────────────────
# INPUT VALIDATION
# ─────────────────────────────────────────────────────────────────────────────

#' Validate inputs before explosion
#'
#' Checks CRS, empty geometries, invalid geometries, region count, and
#' unmatched "Other" units. Optionally repairs invalid geometries.
#'
#' @param sf_obj sf object to validate
#' @param region_col Name of the grouping column
#' @param allow_other If TRUE, permits units labelled "Other"
#' @param fix_invalid If TRUE, attempts st_make_valid() on invalid geometries
#' @return The (possibly repaired) sf object, invisibly
#' @keywords internal
validate_input <- function(sf_obj, region_col,
                           allow_other = FALSE,
                           fix_invalid = TRUE) {
  if (!inherits(sf_obj, "sf"))
    stop("`sf_obj` must be an sf object.", call. = FALSE)
  if (!(region_col %in% names(sf_obj)))
    stop("Column '", region_col, "' not found. Available: ",
         paste(names(sf_obj), collapse = ", "), call. = FALSE)
  if (is.na(sf::st_crs(sf_obj)))
    stop("`sf_obj` has no CRS. Set one with st_set_crs() or st_transform().",
         call. = FALSE)
  if (sf::st_is_longlat(sf_obj))
    stop("`sf_obj` is in geographic (lon/lat) coordinates. ",
         "Project first with st_transform().", call. = FALSE)
  if (any(sf::st_is_empty(sf_obj)))
    stop(sum(sf::st_is_empty(sf_obj)), " empty geometries found. ",
         "Remove with sf_obj[!st_is_empty(sf_obj), ]", call. = FALSE)

  # Fix invalid geometries
  invalid <- !sf::st_is_valid(sf_obj)
  if (any(invalid, na.rm = TRUE)) {
    n_bad <- sum(invalid, na.rm = TRUE)
    if (fix_invalid) {
      message("Repairing ", n_bad, " invalid geometries with st_make_valid().")
      sf_obj <- sf::st_make_valid(sf_obj)
    } else {
      stop(n_bad, " invalid geometries found. ",
           "Pass fix_invalid = TRUE to auto-repair, or fix manually.",
           call. = FALSE)
    }
  }

  n_regions <- dplyr::n_distinct(sf_obj[[region_col]])
  if (n_regions < 2)
    stop("Only ", n_regions, " region found. Need at least 2.", call. = FALSE)

  n_other <- sum(sf_obj[[region_col]] == "Other", na.rm = TRUE)
  if (n_other > 0 && !allow_other)
    stop(n_other, " units mapped to 'Other' (unmatched). ",
         "Fix your grouping or pass allow_other = TRUE.", call. = FALSE)

  sf_obj
}


# ─────────────────────────────────────────────────────────────────────────────
# GEOMETRY STATISTICS
# ─────────────────────────────────────────────────────────────────────────────

#' Compute geometry statistics for parameter derivation
#'
#' Extracts w_bar, R_local, n_bar, n_regions, and the tightness ratio
#' from a projected sf object with a grouping column.
#'
#' @param sf_obj Projected sf object
#' @param region_col Name of the grouping column
#' @param centroid_fun "centroid" (default) or "point_on_surface"
#' @return A named list of geometry statistics
#' @export
compute_stats <- function(sf_obj, region_col,
                          centroid_fun = c("centroid", "point_on_surface")) {
  centroid_fun <- match.arg(centroid_fun)

  areas <- as.numeric(sf::st_area(sf_obj))
  w_bar <- stats::median(sqrt(4 * areas / pi), na.rm = TRUE)

  # Region centroids (warning-free)
  reg_sf <- sf_obj |>
    dplyr::group_by(dplyr::across(dplyr::all_of(region_col))) |>
    dplyr::summarise(geometry = sf::st_union(.data$geometry), .groups = "drop")
  rc <- sf::st_coordinates(centroid_geoms(reg_sf, centroid_fun))
  D_region <- if (nrow(rc) > 1) stats::median(as.numeric(stats::dist(rc))) else NA_real_

  # Unit centroids (warning-free)
  muni_c <- sf::st_coordinates(centroid_geoms(sf_obj, centroid_fun))

  # Build region centroid lookup — preserve original column name
  reg_df <- reg_sf |>
    sf::st_drop_geometry() |>
    dplyr::transmute(
      !!region_col := .data[[region_col]],
      rx = rc[, 1],
      ry = rc[, 2]
    )

  # Compute per-unit distance to region centroid
  tmp <- sf_obj |>
    sf::st_drop_geometry() |>
    dplyr::mutate(cx = muni_c[, 1], cy = muni_c[, 2]) |>
    dplyr::left_join(reg_df, by = region_col) |>
    dplyr::mutate(d = sqrt((.data$cx - .data$rx)^2 + (.data$cy - .data$ry)^2))

  d_max_tbl <- tmp |>
    dplyr::group_by(dplyr::across(dplyr::all_of(region_col))) |>
    dplyr::summarise(d_max = max(.data$d, na.rm = TRUE), .groups = "drop")
  R_local <- stats::median(d_max_tbl$d_max)

  n_regions <- dplyr::n_distinct(sf_obj[[region_col]])
  n_bar     <- stats::median(as.numeric(table(sf_obj[[region_col]])))

  # Per-region unit counts — preserve original column name
  region_summary <- sf_obj |>
    sf::st_drop_geometry() |>
    dplyr::group_by(dplyr::across(dplyr::all_of(region_col))) |>
    dplyr::summarise(n_units = dplyr::n(), .groups = "drop")

  list(
    n_units        = nrow(sf_obj),
    w_bar          = w_bar,
    R_local        = R_local,
    D_region       = D_region,
    n_regions      = n_regions,
    n_bar          = n_bar,
    ratio          = R_local / w_bar,
    per_region     = d_max_tbl,
    region_summary = region_summary
  )
}


# ─────────────────────────────────────────────────────────────────────────────
# PARAMETER DERIVATION
# ─────────────────────────────────────────────────────────────────────────────

#' Derive displacement parameters from geometry statistics
#'
#' Implements Analytical Results 1 and 2 from the paper:
#'   alpha_r = gamma_r * w_bar / (2 * sin(pi / n_regions))
#'   alpha_l = gamma_l * 2 * R_local / sqrt(n_bar)
#'
#' @param stats Output of [compute_stats()]
#' @param gamma_r Regional clearance coefficient (default 3.0)
#' @param gamma_l Local clearance coefficient (default 1.136)
#' @param p Distance scaling exponent (default 1.25)
#' @return Named list with alpha_r, alpha_l, p, gamma_r, gamma_l
#' @export
derive_params <- function(stats, gamma_r = 3.0, gamma_l = 1.136, p = 1.25) {
  if (stats$n_regions < 2)
    stop("Need at least 2 regions.", call. = FALSE)
  if (!is.numeric(gamma_r) || length(gamma_r) != 1 || is.na(gamma_r) ||
      gamma_r < 0) {
    stop("`gamma_r` must be a single non-negative number.", call. = FALSE)
  }
  if (!is.numeric(gamma_l) || length(gamma_l) != 1 || is.na(gamma_l) ||
      gamma_l < 0) {
    stop("`gamma_l` must be a single non-negative number.", call. = FALSE)
  }
  if (!is.numeric(p) || length(p) != 1 || is.na(p) || p <= 0) {
    stop("`p` must be a single positive number.", call. = FALSE)
  }
  list(
    alpha_r = gamma_r * stats$w_bar / (2 * sin(pi / stats$n_regions)),
    alpha_l = gamma_l * 2 * stats$R_local / sqrt(stats$n_bar),
    p       = p,
    gamma_r = gamma_r,
    gamma_l = gamma_l
  )
}


# ─────────────────────────────────────────────────────────────────────────────
# DISPLACEMENT ENGINE
# ─────────────────────────────────────────────────────────────────────────────

#' Core displacement algorithm (Algorithm 1 from the paper)
#'
#' Applies the hierarchical centroid-driven displacement field to every polygon.
#' Each unit receives rigid-body translation t_i = alpha_r * dhat_state + alpha_l * s_i * dhat_local.
#'
#' Propositions 1-3 guarantee: exact geometry preservation, radial ordering, and
#' bounded displacement (||t_i|| <= alpha_r + alpha_l).
#'
#' @param sf_obj Projected sf object with region column
#' @param region_col Grouping column name
#' @param alpha_r Regional separation magnitude (metres)
#' @param alpha_l Local expansion magnitude (metres)
#' @param p Distance scaling exponent (default 1.25)
#' @param centroid_fun "centroid" (default) or "point_on_surface"
#' @return Exploded sf object (same CRS as input)
#' @keywords internal
explode_sf_core <- function(sf_obj, region_col,
                            alpha_r, alpha_l, p = 1.25,
                            centroid_fun = c("centroid", "point_on_surface")) {
  centroid_fun <- match.arg(centroid_fun)
  orig_crs <- sf::st_crs(sf_obj)

  # State centroid (global frame) — warning-free
  Cs <- sf::st_coordinates(
    centroid_geoms(sf::st_union(sf_obj), centroid_fun)
  )[1, ]

  # Region centroids — warning-free
  reg_sf <- sf_obj |>
    dplyr::group_by(dplyr::across(dplyr::all_of(region_col))) |>
    dplyr::summarise(geometry = sf::st_union(.data$geometry), .groups = "drop")
  rc <- sf::st_coordinates(centroid_geoms(reg_sf, centroid_fun))

  reg_df <- reg_sf |>
    sf::st_drop_geometry() |>
    dplyr::transmute(
      !!region_col := .data[[region_col]],
      rx = rc[, 1],
      ry = rc[, 2]
    )

  # Unit centroids — warning-free
  muni_c <- sf::st_coordinates(centroid_geoms(sf_obj, centroid_fun))

  # Build displacement dataframe
  df <- sf_obj |>
    sf::st_drop_geometry() |>
    dplyr::mutate(cx = muni_c[, 1], cy = muni_c[, 2]) |>
    dplyr::left_join(reg_df, by = region_col) |>
    dplyr::mutate(
      # Regional direction: Cs -> Cr (vector from state centroid to region centroid)
      dr_x = .data$rx - Cs[1], dr_y = .data$ry - Cs[2],
      dr_len   = sqrt(.data$dr_x^2 + .data$dr_y^2),
      dhat_r_x = dplyr::if_else(.data$dr_len > 0, .data$dr_x / .data$dr_len, 0),
      dhat_r_y = dplyr::if_else(.data$dr_len > 0, .data$dr_y / .data$dr_len, 0),
      # Local direction: Cr -> Ci (vector from region centroid to unit centroid)
      dl_x = .data$cx - .data$rx, dl_y = .data$cy - .data$ry,
      dl_len   = sqrt(.data$dl_x^2 + .data$dl_y^2),
      dhat_l_x = dplyr::if_else(.data$dl_len > 0, .data$dl_x / .data$dl_len, 0),
      dhat_l_y = dplyr::if_else(.data$dl_len > 0, .data$dl_y / .data$dl_len, 0)
    )

  # d_max per region (for s_i normalization)
  d_max_tbl <- df |>
    dplyr::group_by(dplyr::across(dplyr::all_of(region_col))) |>
    dplyr::summarise(d_max = max(.data$dl_len, na.rm = TRUE), .groups = "drop")

  # Compute s_i and displacement offsets
  df <- df |>
    dplyr::left_join(d_max_tbl, by = region_col) |>
    dplyr::mutate(
      s_i   = dplyr::if_else(.data$d_max > 0, (.data$dl_len / .data$d_max)^p, 0),
      x_off = alpha_r * .data$dhat_r_x + alpha_l * .data$s_i * .data$dhat_l_x,
      y_off = alpha_r * .data$dhat_r_y + alpha_l * .data$s_i * .data$dhat_l_y
    )

  # Apply rigid-body translation (Proposition 1)
  sf_exp <- sf_obj
  sf_exp$geometry <- sf::st_sfc(
    purrr::pmap(list(sf_obj$geometry, df$x_off, df$y_off),
                function(g, dx, dy) g + c(dx, dy)),
    crs = orig_crs
  )
  sf::st_as_sf(sf_exp)
}

.translate_by_offsets <- function(sf_obj, x_off, y_off) {
  out <- sf_obj
  out$geometry <- sf::st_sfc(
    purrr::pmap(
      list(sf::st_geometry(sf_obj), x_off, y_off),
      function(g, dx, dy) g + c(dx, dy)
    ),
    crs = sf::st_crs(sf_obj)
  )
  sf::st_as_sf(out)
}

refine_collisions <- function(sf_obj, region_col,
                              min_gap,
                              max_shift,
                              max_iter = 20,
                              step = 0.5,
                              within = c("region", "all"),
                              centroid_fun = c("centroid", "point_on_surface")) {
  within <- match.arg(within)
  centroid_fun <- match.arg(centroid_fun)

  if (!is.numeric(min_gap) || length(min_gap) != 1 || is.na(min_gap) ||
      min_gap <= 0) {
    stop("`refine_min_gap` must be a single positive number.", call. = FALSE)
  }
  if (!is.numeric(max_shift) || length(max_shift) != 1 || is.na(max_shift) ||
      max_shift < 0) {
    stop("`refine_max_shift` must be a single non-negative number.",
         call. = FALSE)
  }
  if (!is.numeric(max_iter) || length(max_iter) != 1 || is.na(max_iter) ||
      max_iter < 1) {
    stop("`refine_max_iter` must be a positive integer.", call. = FALSE)
  }
  if (!is.numeric(step) || length(step) != 1 || is.na(step) ||
      step <= 0 || step > 1) {
    stop("`refine_step` must be a single number in (0, 1].", call. = FALSE)
  }

  n <- nrow(sf_obj)
  diagnostics <- list(
    enabled = TRUE,
    min_gap = min_gap,
    max_shift = max_shift,
    max_iter = as.integer(max_iter),
    step = step,
    within = within,
    iterations = 0L,
    corrected_pairs = 0L,
    active_pairs_last = 0L,
    max_shift_observed = 0
  )
  if (n < 2 || max_shift == 0) {
    return(list(sf = sf_obj, diagnostics = diagnostics))
  }

  current <- sf_obj
  total_offsets <- matrix(0, nrow = n, ncol = 2)

  for (iter in seq_len(as.integer(max_iter))) {
    diagnostics$iterations <- iter
    geom <- sf::st_geometry(current)
    close <- sf::st_is_within_distance(geom, geom, dist = min_gap)
    cent <- sf::st_coordinates(centroid_geoms(current, centroid_fun))
    delta <- matrix(0, nrow = n, ncol = 2)
    active_pairs <- 0L

    for (i in seq_len(n - 1L)) {
      js <- close[[i]]
      js <- js[js > i]
      if (!length(js)) next

      for (j in js) {
        if (within == "region" &&
            !isTRUE(current[[region_col]][i] == current[[region_col]][j])) {
          next
        }

        d <- as.numeric(sf::st_distance(geom[i], geom[j], by_element = TRUE))
        if (!is.finite(d) || d >= min_gap) next

        dir <- cent[j, ] - cent[i, ]
        dir_len <- sqrt(sum(dir^2))
        if (!is.finite(dir_len) || dir_len == 0) {
          theta <- ((i * 73856093 + j * 19349663) %% 360) * pi / 180
          dir <- c(cos(theta), sin(theta))
        } else {
          dir <- dir / dir_len
        }

        move <- 0.5 * step * (min_gap - d)
        delta[i, ] <- delta[i, ] - dir * move
        delta[j, ] <- delta[j, ] + dir * move
        active_pairs <- active_pairs + 1L
      }
    }

    diagnostics$active_pairs_last <- active_pairs
    if (active_pairs == 0L) break

    for (i in seq_len(n)) {
      if (all(delta[i, ] == 0)) next
      proposed <- total_offsets[i, ] + delta[i, ]
      proposed_norm <- sqrt(sum(proposed^2))
      if (proposed_norm > max_shift) {
        capped_total <- proposed / proposed_norm * max_shift
        delta[i, ] <- capped_total - total_offsets[i, ]
      }
    }

    if (max(abs(delta)) < sqrt(.Machine$double.eps)) break

    current <- .translate_by_offsets(current, delta[, 1], delta[, 2])
    total_offsets <- total_offsets + delta
    diagnostics$corrected_pairs <- diagnostics$corrected_pairs + active_pairs
  }

  shift_norms <- sqrt(rowSums(total_offsets^2))
  diagnostics$max_shift_observed <- max(shift_norms, na.rm = TRUE)

  list(sf = current, diagnostics = diagnostics)
}


# ─────────────────────────────────────────────────────────────────────────────
# PRINT FORMATTING HELPER
# ─────────────────────────────────────────────────────────────────────────────

#' Format a distance in metres as a human-readable km string
#'
#' Adapts precision to magnitude: "23.6 km" for large values,
#' "0.067 km" for small ones, so tiny test geometries don't show "0 km".
#'
#' @param x_m Distance in metres
#' @return Character string
#' @keywords internal
fmt_dist <- function(x_m) {
  if (is.na(x_m)) return("NA")
  x_km <- x_m / 1000
  if (x_km >= 10)    return(sprintf("%.0f km", x_km))
  if (x_km >= 1)     return(sprintf("%.1f km", x_km))
  if (x_km >= 0.01)  return(sprintf("%.3f km", x_km))
  return(sprintf("%.1f m", x_m))
}


# ─────────────────────────────────────────────────────────────────────────────
# SHINY DETECTION
# ─────────────────────────────────────────────────────────────────────────────

#' Detect whether code is executing inside a live Shiny session
#'
#' Uses `shiny::isRunning()` when the shiny package is available.
#' Returns FALSE in all other contexts (scripts, knitr, testthat, etc.).
#'
#' @return Logical scalar
#' @keywords internal
#' @noRd
.shiny_is_running <- function() {
  requireNamespace("shiny", quietly = TRUE) &&
    isTRUE(tryCatch(shiny::isRunning(), error = function(e) FALSE))
}

test_that("explode_sf returns exploded_map object with expected structure", {
  x <- make_test_sf()

  out <- explode_sf(x, region_col = "region", plot = FALSE)

  expect_s3_class(out, "exploded_map")
  expect_true(all(c(
    "sf_orig", "sf_exp", "sf_exp_wgs", "stats", "params",
    "gamma_r_implied", "gamma_l_implied", "plots", "refinement", "diagnostics"
  ) %in% names(out)))

  expect_s3_class(out$sf_orig, "sf")
  expect_s3_class(out$sf_exp, "sf")
  expect_equal(nrow(out$sf_orig), nrow(out$sf_exp))
  expect_equal(as.integer(sf::st_crs(out$sf_exp_wgs)$epsg), 4326)
  expect_false(out$refinement$enabled)
})

test_that("explode_sf respects manual alpha overrides", {
  x <- make_test_sf()

  out <- explode_sf(
    x,
    region_col = "region",
    alpha_r = 100,
    alpha_l = 200,
    plot = FALSE
  )

  expect_equal(out$params$alpha_r, 100)
  expect_equal(out$params$alpha_l, 200)
  expect_equal(out$params$p, 1.25)
})

test_that("explode_sf respects partial manual alpha overrides", {
  x <- make_test_sf()

  derived <- derive_params(compute_stats(x, "region"))

  out_r <- explode_sf(
    x,
    region_col = "region",
    alpha_r = 100,
    plot = FALSE
  )
  expect_equal(out_r$params$alpha_r, 100)
  expect_equal(out_r$params$alpha_l, derived$alpha_l)

  out_l <- explode_sf(
    x,
    region_col = "region",
    alpha_l = 200,
    plot = FALSE
  )
  expect_equal(out_l$params$alpha_r, derived$alpha_r)
  expect_equal(out_l$params$alpha_l, 200)
})

test_that("explode_sf errors on missing grouping column", {
  x <- make_test_sf()

  expect_error(
    explode_sf(x, region_col = "does_not_exist", plot = FALSE),
    "not found"
  )
})

test_that("explode_sf errors on geographic CRS", {
  x <- make_test_sf() |> sf::st_transform(4326)

  expect_error(
    explode_sf(x, region_col = "region", plot = FALSE),
    "Project first"
  )
})

test_that("explode_sf errors when Other is present and not allowed", {
  x <- make_test_sf()
  x$region[1] <- "Other"

  expect_error(
    explode_sf(x, region_col = "region", allow_other = FALSE, plot = FALSE),
    "Other"
  )
})

test_that("explode_sf allows Other when allow_other is TRUE", {
  x <- make_test_sf()
  x$region[1] <- "Other"

  out <- explode_sf(x, region_col = "region", allow_other = TRUE, plot = FALSE)

  expect_s3_class(out, "exploded_map")
})

test_that("explode_sf collision refinement applies bounded corrections", {
  sq <- function(xmin, ymin, size = 1) {
    sf::st_polygon(list(matrix(
      c(
        xmin, ymin,
        xmin + size, ymin,
        xmin + size, ymin + size,
        xmin, ymin + size,
        xmin, ymin
      ),
      ncol = 2,
      byrow = TRUE
    )))
  }

  x <- sf::st_sf(
    id = c("a1", "a2", "b1", "b2"),
    region = c("A", "A", "B", "B"),
    geometry = sf::st_sfc(
      sq(0, 0),
      sq(1, 0),
      sq(10, 0),
      sq(11, 0),
      crs = 3857
    )
  )

  out <- explode_sf(
    x,
    region_col = "region",
    alpha_r = 0,
    alpha_l = 0,
    refine = TRUE,
    refine_min_gap = 0.5,
    refine_max_shift = 0.2,
    refine_max_iter = 2,
    plot = FALSE
  )

  c0 <- sf::st_coordinates(centroid_geoms(x))
  c1 <- sf::st_coordinates(centroid_geoms(out$sf_exp))
  shift <- sqrt(rowSums((c1 - c0)^2))

  expect_true(out$refinement$enabled)
  expect_gt(out$refinement$corrected_pairs, 0)
  expect_lte(max(shift), 0.2 + 1e-8)
  expect_equal(out$params$refine_within, "region")
})

test_that("compute_stats returns expected fields and positive values", {
  x <- make_test_sf()

  s <- compute_stats(x, "region")

  expect_type(s, "list")
  expect_true(all(c("w_bar", "R_local", "n_bar", "n_regions", "ratio") %in% names(s)))
  expect_equal(s$n_regions, 2)
  expect_equal(s$n_bar, 2)
  expect_true(s$w_bar > 0)
  expect_true(s$R_local >= 0)
  expect_true(s$ratio >= 0)
})

test_that("derive_params validates user-controlled parameters", {
  x <- make_test_sf()
  s <- compute_stats(x, "region")

  expect_error(derive_params(s, gamma_r = -1), "gamma_r")
  expect_error(derive_params(s, gamma_l = -1), "gamma_l")
  expect_error(derive_params(s, p = 0), "`p`")
})

# =============================================================================
# R/plot.R — Plotting helpers
# =============================================================================

#' Build standard original/exploded ggplot pair
#' @keywords internal
.make_plots <- function(sf_orig, sf_exp, region_col, title, params) {
  regions <- sort(unique(sf_orig[[region_col]]))
  pal <- stats::setNames(
    grDevices::colorRampPalette(
      c("#2166ac", "#d73027", "#1a9850", "#984ea3",
        "#ff7f00", "#a65628", "#f781bf", "#999999",
        "#33a02c", "#b2df8a")
    )(length(regions)),
    regions
  )

  subtitle <- if (!is.null(params))
    paste0("alpha_r=", round(params$alpha_r), "m  alpha_l=",
           round(params$alpha_l), "m  p=", params$p)
  else ""

  p_orig <- ggplot2::ggplot(sf_orig |> sf::st_transform(4326)) +
    ggplot2::geom_sf(ggplot2::aes(fill = .data[[region_col]]),
                     color = "white", linewidth = 0.2) +
    ggplot2::scale_fill_manual(values = pal, name = region_col) +
    ggplot2::theme_void(base_size = 11) +
    ggplot2::labs(title = paste(title, "- Original"))

  p_exp <- ggplot2::ggplot(sf_exp |> sf::st_transform(4326)) +
    ggplot2::geom_sf(ggplot2::aes(fill = .data[[region_col]]),
                     color = "white", linewidth = 0.2) +
    ggplot2::scale_fill_manual(values = pal, name = region_col) +
    ggplot2::theme_void(base_size = 11) +
    ggplot2::labs(title = paste(title, "- Exploded"), subtitle = subtitle)

  list(orig = p_orig, exp = p_exp)
}

#' explodemap: Hierarchical Exploded-View Cartography
#'
#' Internal package declarations for NSE checks.
#'
#' @keywords internal
#' @importFrom rlang .data :=
"_PACKAGE"

## usethis namespace: start
#' @importFrom rlang .data
## usethis namespace: end
NULL

---
title: "Reproducing the paper examples"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Reproducing the paper examples}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
editor_options:
  markdown:
    wrap: 72
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Overview

This vignette reproduces the key examples from:

> Arthur, G. "A Hierarchical Vector-Based Framework for Multi-Scale
> Exploded-View Cartography."

Each section corresponds to a result in the paper and uses the package
API directly, including `explode_state()`, `explode_sf()`,
`explode_grouped()`, `layout_regions()`, and `calibration_row()`.

This vignette is designed for reproduction rather than routine package
checks. Several sections download external boundary files and may take
substantial time and disk space. Heavy chunks are therefore marked
`eval = FALSE`.

Reported values should match the paper within rounding tolerance.
Externally downloaded datasets may introduce small differences if source
files change over time.

```{r setup}
library(explodemap)
library(sf)
library(dplyr)
```

------------------------------------------------------------------------

## 1. New Jersey — Ground-truth calibration (Section 5)

New Jersey is the calibration dataset. The known-good parameters
($\alpha_r = 6{,}000$ m, $\alpha_l = 10{,}000$ m) were established by
visual validation and are used to derive the legibility coefficients
$\gamma_r$ and $\gamma_l$.

```{r nj-calibration, eval = FALSE}
nj <- explode_state(
  state_fips = "34", crs = 32118,
  region_map = list(
    North   = c("Bergen", "Essex", "Hudson", "Morris",
                "Passaic", "Sussex", "Union", "Warren"),
    Central = c("Hunterdon", "Mercer", "Middlesex",
                "Monmouth", "Somerset"),
    South   = c("Atlantic", "Burlington", "Camden", "Cape May",
                "Cumberland", "Gloucester", "Ocean", "Salem")
  ),
  label = "New Jersey"
)

summary(nj)
```

**Expected output (key values):**

| Quantity                                      | Value    |
|-----------------------------------------------|----------|
| n units                                       | 564      |
| n regions                                     | 3        |
| w_bar                                         | 3.94 km  |
| R_local                                       | 62.4 km  |
| n_bar                                         | 177      |
| R_local/w_bar                                 | 15.83    |
| alpha_r (derived)                             | 6,844 m  |
| alpha_l (derived)                             | 10,641 m |
| gamma_r implied (from known alpha_r = 6,000)  | 2.64     |
| gamma_l implied (from known alpha_l = 10,000) | 1.136    |

The implied $\gamma_l = 1.136$ from the New Jersey ground truth becomes
the recommended default for transfer to other datasets.

------------------------------------------------------------------------

## 2. Pennsylvania — Transfer test (Section 6)

Pennsylvania tests whether New Jersey-calibrated coefficients transfer
to a larger, denser dataset without retuning. The paper reports
formula-derived parameters $\alpha_r = 20{,}174$ m and
$\alpha_l = 12{,}447$ m.

The region map is defined as a reusable object so that both the transfer
run and the sensitivity analysis reference the same grouping:

```{r pa-region-map}
pa_region_map <- list(
  Southeast    = c("Philadelphia", "Delaware", "Chester",
                   "Montgomery", "Bucks"),
  Northeast    = c("Pike", "Monroe", "Carbon", "Northampton", "Lehigh",
                   "Luzerne", "Lackawanna", "Wayne", "Susquehanna",
                   "Wyoming", "Sullivan", "Columbia", "Montour",
                   "Schuylkill", "Berks", "Bradford"),
  Central      = c("Centre", "Clinton", "Lycoming", "Tioga", "Potter",
                   "Cameron", "Elk", "Clearfield", "Jefferson", "Indiana",
                   "Blair", "Huntingdon", "Mifflin", "Snyder", "Union",
                   "Northumberland", "Juniata", "Perry", "Dauphin",
                   "Lebanon"),
  SouthCentral = c("York", "Adams", "Lancaster", "Cumberland", "Franklin",
                   "Fulton", "Bedford", "Somerset", "Cambria"),
  Southwest    = c("Allegheny", "Westmoreland", "Fayette", "Greene",
                   "Washington", "Beaver", "Butler", "Armstrong",
                   "Lawrence"),
  Northwest    = c("Erie", "Crawford", "Mercer", "Venango", "Clarion",
                   "Forest", "Warren", "McKean")
)
```

```{r pa-transfer, eval = FALSE}
pa <- explode_state(
  state_fips = "42", crs = 26918,
  region_map = pa_region_map,
  label = "Pennsylvania"
)

summary(pa)
```

**Expected output (key values):**

| Quantity          | Value     |
|-------------------|-----------|
| n units           | 2,572     |
| n regions         | 6         |
| w_bar             | 6,725 m   |
| R_local           | 116,086 m |
| n_bar             | 449       |
| R_local/w_bar     | 17.26     |
| alpha_r (derived) | 20,174 m  |
| alpha_l (derived) | 12,447 m  |

### Sensitivity analysis

The paper reports that $\alpha_l$ is stable under $\pm 15\%$
perturbation:

```{r pa-sensitivity, eval = FALSE}
alpha_l_canonical <- 12447
alpha_r_canonical <- 20174

factors <- c(0.85, 0.90, 0.95, 1.00, 1.05, 1.10, 1.15)
labels  <- c("-15%", "-10%", "-5%", "canonical", "+5%", "+10%", "+15%")

rows <- list()
for (i in seq_along(factors)) {
  run <- explode_state(
    state_fips = "42", crs = 26918,
    region_map = pa_region_map,
    alpha_r = alpha_r_canonical,
    alpha_l = round(alpha_l_canonical * factors[i]),
    plot = FALSE, export = FALSE,
    label = paste0("PA ", labels[i])
  )
  rows[[i]] <- calibration_row(run)
  rows[[i]]$label <- labels[i]
  rows[[i]]$factor <- factors[i]
}

sensitivity_df <- bind_rows(rows)
print(sensitivity_df)
```

**Expected output:**

| Label     | alpha_l | Mean displacement |
|-----------|---------|-------------------|
| -15%      | 10,580  | \~20,476 m        |
| -10%      | 11,202  | \~20,524 m        |
| -5%       | 11,825  | \~20,575 m        |
| canonical | 12,447  | \~20,630 m        |
| +5%       | 13,069  | \~20,688 m        |
| +10%      | 13,692  | \~20,749 m        |
| +15%      | 14,314  | \~20,813 m        |

Mean displacement CV across the $\pm 15\%$ range is $< 0.02$, confirming
stability.

------------------------------------------------------------------------

## 3. Cross-state calibration (Section 7)

The state registry in `inst/registries/state_registry.R` contains New
Jersey, Pennsylvania, Ohio, and New York. The calibration runner
processes all registered states and reports gamma stability.

```{r calibration, eval = FALSE}
source(system.file("registries/state_registry.R", package = "explodemap"))

calib_rows <- list()
for (key in names(state_registry)) {
  reg <- state_registry[[key]]

  result <- tryCatch(
    explode_state(
      state_fips = reg$fips, crs = reg$crs,
      region_map = reg$region_map,
      allow_other = TRUE, plot = FALSE,
      label = reg$name
    ),
    error = function(e) {
      message("ERROR: ", e$message)
      NULL
    }
  )

  if (is.null(result)) next
  calib_rows[[key]] <- calibration_row(result)
}

calib_df <- bind_rows(calib_rows)
print(calib_df)
```

**Expected output (approximate):**

| State        | n     | Regions | w_bar (km) | R_local (km) | Ratio | gamma_r | gamma_l |
|--------------|-------|---------|------------|--------------|-------|---------|---------|
| New Jersey   | 564   | 3       | 3.94       | 62.4         | 15.83 | 2.64\*  | 1.136\* |
| Pennsylvania | 2,572 | 6       | 6.73       | 116.1        | 17.26 | 3.72\*  | 1.136   |
| Ohio         | 1,602 | 5       | 7.31       | 93.1         | 12.75 | 3.00    | 1.136   |
| New York     | 1,794 | 5       | 10.04      | 97.2         | 9.68  | 3.00    | 1.136   |

\* Implied from known ground-truth parameters.

The key finding is that $\gamma_l = 1.136$ is stable across states,
while $\gamma_r$ varies more, indicating that regional clearance still
benefits from dataset-specific visual validation.

------------------------------------------------------------------------

## 4. Ohio — Extended validation (Section 7)

Ohio provides a five-region test with three competing urban cores
(Cleveland, Columbus, Cincinnati):

```{r ohio, eval = FALSE}
oh <- explode_state(
  state_fips = "39", crs = 32617,
  region_map = list(
    Northeast = c("Cuyahoga", "Summit", "Lorain", "Lake", "Medina",
                  "Portage", "Geauga", "Ashtabula", "Trumbull", "Mahoning",
                  "Columbiana", "Carroll", "Stark", "Wayne", "Holmes",
                  "Harrison", "Jefferson"),
    Northwest = c("Lucas", "Wood", "Fulton", "Williams", "Defiance",
                  "Paulding", "Henry", "Putnam", "Hancock", "Sandusky",
                  "Erie", "Ottawa", "Seneca", "Wyandot", "Crawford",
                  "Huron", "Ashland", "Richland", "Morrow", "Knox",
                  "Marion", "Hardin", "Logan", "Union", "Delaware",
                  "Allen", "Van Wert", "Auglaize", "Shelby", "Mercer"),
    Central   = c("Franklin", "Licking", "Fairfield", "Pickaway",
                  "Madison", "Fayette", "Ross", "Clark", "Greene",
                  "Montgomery", "Preble", "Darke", "Miami", "Champaign"),
    Southwest = c("Hamilton", "Butler", "Warren", "Clermont", "Clinton",
                  "Highland", "Brown", "Adams", "Scioto", "Lawrence",
                  "Gallia", "Jackson", "Pike"),
    Southeast = c("Belmont", "Monroe", "Washington", "Meigs", "Morgan",
                  "Noble", "Guernsey", "Muskingum", "Perry", "Hocking",
                  "Athens", "Tuscarawas", "Coshocton", "Vinton")
  ),
  label = "Ohio"
)

summary(oh)
plot(oh, "both")
```

**Expected output:** $R_{\text{local}}/\bar{w} = 12.75$, placing Ohio in
the dense-municipal cluster. All three urban cores are correctly
suppressed by the $s_i$ term.

------------------------------------------------------------------------

## 5. Canada — Non-US validation (Section 7)

The Canada validation tests whether the framework transfers outside the
US administrative system entirely. Data comes from Statistics Canada
2021 Census Subdivisions.

```{r canada-regions}
province_regions <- data.frame(
  PRUID  = c("10", "11", "12", "13", "24", "35",
             "46", "47", "48", "59", "60", "61", "62"),
  region = c(rep("Atlantic", 4), "Quebec", "Ontario",
             rep("Prairies", 3), "Pacific",
             rep("Territories", 3)),
  stringsAsFactors = FALSE
)
```

```{r canada-download, eval = FALSE}
cache_file <- file.path(path.expand("~"), "explode_map_cache",
                        "canada_csds_2021.rds")

if (file.exists(cache_file)) {
  sf_raw <- readRDS(cache_file)
} else {
  url <- paste0(
    "https://www12.statcan.gc.ca/census-recensement/2021/geo/sip-pis/",
    "boundary-limites/files-fichiers/lcsd000b21a_e.zip"
  )
  tmp <- tempfile(fileext = ".zip")
  download.file(url, tmp, mode = "wb")
  dir <- file.path(tempdir(), "canada_csds")
  dir.create(dir, showWarnings = FALSE)
  unzip(tmp, exdir = dir)
  shp <- list.files(dir, "\\.shp$", recursive = TRUE, full.names = TRUE)
  sf_raw <- st_read(shp[1], quiet = TRUE)
  dir.create(dirname(cache_file), showWarnings = FALSE, recursive = TRUE)
  saveRDS(sf_raw, cache_file)
}
```

```{r canada-explode, eval = FALSE}
sf_proj <- sf_raw |>
  st_transform(3347) |>
  left_join(province_regions, by = "PRUID")

sf_proj$region[is.na(sf_proj$region)] <- "Other"

sf_prov <- sf_proj |>
  filter(region != "Territories")

canada <- explode_sf(
  sf_prov,
  region_col = "region",
  allow_other = TRUE,
  label = "Canada (provinces)"
)

summary(canada)
plot(canada, "both")
```

**Expected output:**

| Quantity      | Value                           |
|---------------|---------------------------------|
| n units       | \~4,800 (excluding territories) |
| n regions     | 5                               |
| R_local/w_bar | \~113                           |

The tightness ratio is an order of magnitude larger than in the US state
examples because Canadian CSDs include vast northern municipalities. The
formula-derived parameters still produce a coherent layout, illustrating
that the method can remain usable even in extreme tightness-ratio
regimes.

------------------------------------------------------------------------

## 6. HHS national grouped layout (Section 12)

The three-level extension places US states into 10 HHS region blocks
using anchor-based placement with collision refinement.

```{r hhs-lookup}
hhs_lookup <- data.frame(
  STUSPS = c(
    "CT", "ME", "MA", "NH", "RI", "VT",
    "NJ", "NY", "PR", "VI",
    "DE", "DC", "MD", "PA", "VA", "WV",
    "AL", "FL", "GA", "KY", "MS", "NC", "SC", "TN",
    "IL", "IN", "MI", "MN", "OH", "WI",
    "AR", "LA", "NM", "OK", "TX",
    "IA", "KS", "MO", "NE",
    "CO", "MT", "ND", "SD", "UT", "WY",
    "AZ", "CA", "HI", "NV", "GU", "AS", "MP",
    "AK", "ID", "OR", "WA"
  ),
  hhs_region = c(
    rep("1", 6), rep("2", 4), rep("3", 6), rep("4", 8),
    rep("5", 6), rep("6", 5), rep("7", 4), rep("8", 6),
    rep("9", 7), rep("10", 4)
  ),
  stringsAsFactors = FALSE
)
```

```{r hhs-download, eval = FALSE}
cache_file <- file.path(path.expand("~"), "explode_map_cache",
                        "us_states.rds")

if (file.exists(cache_file)) {
  states_sf <- readRDS(cache_file)
} else {
  url <- "https://www2.census.gov/geo/tiger/TIGER2024/STATE/tl_2024_us_state.zip"
  tmp <- tempfile(fileext = ".zip")
  download.file(url, tmp, mode = "wb", quiet = TRUE)
  dir <- file.path(tempdir(), "us_states")
  dir.create(dir, showWarnings = FALSE)
  unzip(tmp, exdir = dir)
  shp <- list.files(dir, "\\.shp$", recursive = TRUE, full.names = TRUE)
  states_sf <- st_read(shp[1], quiet = TRUE)
  dir.create(dirname(cache_file), showWarnings = FALSE, recursive = TRUE)
  saveRDS(states_sf, cache_file)
}
```

```{r hhs-explode, eval = FALSE}
states_proj <- states_sf |>
  st_transform(5070) |>
  left_join(hhs_lookup, by = "STUSPS")

states_proj$hhs_region[is.na(states_proj$hhs_region)] <- "Other"

hhs <- explode_grouped(
  states_proj,
  region_col   = "hhs_region",
  mode         = "auto_collision",
  alpha_l      = 120000,
  p            = 1.25,
  kappa        = 1.8,
  padding      = 80000,
  delta        = 20000,
  lambda       = 0.18,
  eta          = 0.18,
  padding_sep  = 30000,
  max_iter     = 60,
  label        = "US by HHS Region"
)

print(hhs)
plot(hhs, "all")
```

**Expected output:** The anchor solver converges within 60 iterations.
All 10 HHS regions are separated with a recognisable continental
arrangement. The `auto_collision` mode produces substantially more
legible output than `auto` alone because the spring-repulsion solver
reduces block overlaps in the densely packed Northeast corridor.

------------------------------------------------------------------------

## Replication checklist

After running all sections above, verify:

-   [ ] New Jersey implied $\gamma_l \approx 1.136$ from known
    $\alpha_l = 10{,}000$ m
-   [ ] Pennsylvania formula-derived $\alpha_l = 12{,}447$ m from
    $\gamma_l = 1.136$
-   [ ] Pennsylvania sensitivity CV $< 0.02$ for mean displacement
    across $\pm 15\%$
-   [ ] Cross-state $R_{\text{local}}/\bar{w}$ clusters around
    dense-unit and large-unit regimes
-   [ ] Canada layout remains coherent at
    $R_{\text{local}}/\bar{w} \approx 113$
-   [ ] HHS anchor solver converges with all 10 regions separated

All values should match those reported in the paper within rounding
tolerance.

Together, these examples cover the paper's calibration, transfer,
cross-state, international, and grouped-layout results using the package
API.

------------------------------------------------------------------------

## Session info

```{r session-info}
sessionInfo()
```

---
title: "Getting started with explodemap"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting started with explodemap}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
```

## Overview

`explodemap` generates hierarchical exploded-view maps for dense
administrative boundary data. It applies rigid-body translations to
polygon geometries so that features are visually separated while each
feature's internal geometry is preserved exactly.

The basic two-level workflow is:

1. Group units into regions using a column in your data.
2. Displace units within and across those regions using a centroid-driven
   vector field with analytically derived parameters.

For the two-level core described in the paper, the package is designed
to satisfy three key properties:

- **Proposition 1:** each polygon's shape, area, perimeter, and topology
  are preserved exactly.
- **Proposition 2:** the radial ordering of units within each region is
  preserved.
- **Proposition 3:** no unit is displaced by more than
  `alpha_r + alpha_l` metres.

## Input requirements

`explodemap` expects an `sf` object with:

- Polygon or multipolygon geometries
- A grouping column identifying the parent region for each unit
- A **projected** CRS with linear metre units

Geographic coordinates such as EPSG:4326 (longitude/latitude) must be
transformed before use. For U.S. work, a state plane, UTM, or Albers-type
projected CRS is usually appropriate.

```{r setup}
library(sf)
library(explodemap)
```

## A minimal example

We create a small synthetic dataset with four square polygons in two
regions.

```{r synthetic-data}
sq <- function(xmin, ymin, size = 1000) {
  st_polygon(list(matrix(
    c(xmin, ymin,
      xmin + size, ymin,
      xmin + size, ymin + size,
      xmin, ymin + size,
      xmin, ymin),
    ncol = 2,
    byrow = TRUE
  )))
}

geom <- st_sfc(
  sq(0, 0), sq(3000, 0),       # Region A
  sq(12000, 0), sq(15000, 0),  # Region B
  crs = 3857
)

x <- st_sf(
  id     = c("a1", "a2", "b1", "b2"),
  region = c("A", "A", "B", "B"),
  geometry = geom
)

x
```

## Running the explosion

The simplest entry point is `explode_sf()`. Pass your sf object and the
name of the grouping column:

```{r explode-basic}
result <- explode_sf(x, region_col = "region", plot = FALSE)
```

In Shiny or other non-interactive pipelines, add `quiet = TRUE` to
suppress progress messages:

```{r explode-quiet}
quiet_result <- explode_sf(x, region_col = "region", plot = FALSE, quiet = TRUE)
class(quiet_result)
```

The returned object is an S3 object of class `exploded_map`:

```{r class-check}
class(result)
names(result)
```

It contains the original and exploded geometries, a WGS84 export-ready
version, derived statistics and parameters, plots, and diagnostics.

## Diagnostics

`print()` shows geometry statistics and derived parameters:

```{r print}
print(result)
```

`summary()` adds implied gamma coefficients that are useful for
calibration work:

```{r summary}
summary(result)
```

## Plotting

```{r plot-exploded}
plot(result)
```

You can also view both original and exploded layouts side by side:

```{r plot-both, fig.width = 10}
plot(result, "both")
```

## Calibration output

`calibration_row()` returns a one-row data frame suitable for combining
across datasets when building a cross-dataset calibration table:

```{r calibration}
calibration_row(result)
```

## Manual parameter overrides

By default, `explodemap` derives displacement parameters analytically
from dataset geometry using the paper's two closed-form results. You can
also supply parameters directly. Overrides may be supplied
independently, so you can tune regional separation without changing
local expansion, or vice versa:

```{r manual}
manual <- explode_sf(
  x,
  region_col = "region",
  alpha_r = 100,
  alpha_l = 200,
  plot = FALSE
)

manual$params
```

```{r partial-manual}
more_region_gap <- explode_sf(
  x,
  region_col = "region",
  alpha_r = result$params$alpha_r * 1.5,
  plot = FALSE
)

more_region_gap$params
```

## Optional collision refinement

The two-level algorithm is the clean paper model. For dense municipal
cores, you can add a bounded refinement pass that nudges close
same-region neighbors apart after the analytical displacement:

```{r refine-example, eval=FALSE}
refined <- explode_sf(
  x,
  region_col = "region",
  refine = TRUE,
  refine_min_gap = 0.15,
  refine_max_shift = 0.10,
  plot = FALSE
)

refined$refinement
```

```{r refine, echo=FALSE}
has_refinement <- "refine" %in% names(formals(explode_sf))

if (has_refinement) {
  refined <- explode_sf(
    x,
    region_col = "region",
    refine = TRUE,
    refine_min_gap = 0.15,
    refine_max_shift = 0.10,
    plot = FALSE
  )

  refined$refinement
} else {
  message("This installed explodemap build does not include refinement yet.")
}
```

`refine_max_shift` caps the extra correction per feature, so the
refinement remains a small display adjustment rather than a replacement
for the displacement model. Use `refine_within = "all"` when the
remaining crowding crosses region boundaries.

## Centroid options

For irregular or multipart polygons, `"point_on_surface"` may be
preferable to the default geometric centroid:

```{r centroid-mode}
pos <- explode_sf(
  x,
  region_col = "region",
  centroid_fun = "point_on_surface",
  plot = FALSE
)
```

## Using TIGER/Line data for U.S. states

`explode_state()` downloads U.S. Census TIGER/Line boundaries
automatically and groups municipalities by a county-to-region mapping:

```{r tiger-example, eval = FALSE}
nj <- explode_state(
  state_fips = "34", crs = 32118,
  region_map = list(
    North   = c("Bergen", "Essex", "Hudson", "Morris",
                "Passaic", "Sussex", "Union", "Warren"),
    Central = c("Hunterdon", "Mercer", "Middlesex",
                "Monmouth", "Somerset"),
    South   = c("Atlantic", "Burlington", "Camden", "Cape May",
                "Cumberland", "Gloucester", "Ocean", "Salem")
  ),
  label = "New Jersey"
)
```

Downloaded data is cached locally so subsequent runs are faster.

Use `quiet = TRUE` in app code if downloads and region assignment happen
inside a reactive expression.

## Using a lookup table

`explode_sf_with_lookup()` joins an external lookup table to your sf
object before exploding:

```{r lookup-example, eval = FALSE}
groups <- read.csv("region_assignments.csv")

result <- explode_sf_with_lookup(
  my_sf,
  join_col   = "GEOID",
  lookup     = groups,
  lookup_key = "geoid",
  region_col = "region"
)
```

Unmatched units are labelled `"Other"`. This is useful when a lookup
table is incomplete. You can include or exclude those units using
`allow_other`.

## Export

The `export` parameter supports three modes:

- `NULL` (default): no export
- `TRUE`: auto-named GeoJSON file
- A file path string: explicit output location

```{r export-example, eval = FALSE}
result <- explode_sf(
  my_sf,
  region_col = "region",
  export = "output.geojson"
)
```

## Interactive focus maps and Shiny

`focus_map()` renders raw `sf`, `exploded_map`, or
`grouped_exploded_map` objects as an interactive htmlwidget. Click a
polygon to zoom and lift it into focus; right-click or press Escape to
reset. Information cards can show selected fields without blocking the
map.

```{r focus-map, eval=FALSE}
focus_map(
  result,
  label_col = "id",
  id_col = "id",
  group_col = "region",
  group_palette = c(A = "#4C78A8", B = "#F58518", C = "#54A24B"),
  info_cols = c("id", "region"),
  info_card_scale = 0.95
)
```

In Shiny, use `focusmapOutput()` and `renderFocusmap()`. The widget
emits `input$<outputId>_selected`, which includes the selected feature
ID, label, group, and properties.

```{r shiny-focus-map, eval=FALSE}
ui <- shiny::fluidPage(
  focusmapOutput("map", height = "650px"),
  shiny::verbatimTextOutput("selected")
)

server <- function(input, output, session) {
  exploded <- explode_sf(x, "region", plot = FALSE, quiet = TRUE)

  output$map <- renderFocusmap({
    focus_map(
      exploded,
      label_col = "id",
      id_col = "id",
      group_col = "region",
      group_palette = c(A = "#4C78A8", B = "#F58518", C = "#54A24B")
    )
  })

  output$selected <- shiny::renderPrint(input$map_selected)
}
```

For section drill-downs, use `explode_section()` before rendering. It
explodes the selected section and marks the rest of the layer as context:

```{r section-focus-map, eval=FALSE}
focused <- explode_section(
  x,
  section_col = "region",
  section = "A",
  region_col = "county",
  alpha_r = 900,
  alpha_l = 600,
  plot = FALSE,
  quiet = TRUE
)

focus_map(
  focused,
  label_col = "id",
  context_col = ".explodemap_role",
  context_mode = "fade"
)
```

Small municipal features can use adaptive focus sizing so the selected
polygon is not left visually distant after the camera moves:

```{r tiny-feature-focus, eval=FALSE}
focus_map(
  focused,
  label_col = "id",
  context_col = ".explodemap_role",
  context_mode = "fade",
  min_focus_width = 115,
  min_focus_height = 88,
  tiny_feature_threshold = 52,
  tiny_feature_boost = 1.45,
  origin_context = "inset",
  origin_context_position = "bottom-left",
  focus_context_opacity = 0.14,
  show_drag_zoom = TRUE
)
```

## Notes

- Always use a projected CRS before running the algorithm.
- The two-level core preserves each feature's internal geometry exactly
  through rigid-body translation.
- Parameter derivation is deterministic and reproducible: the same
  dataset and gamma coefficients always produce the same output.

## Next steps

See `vignette("grouped-layouts")` for the three-level extension using
`explode_grouped()`, which adds region-block anchor placement for larger
multi-region or national layouts.

---
title: "Choosing an explodemap workflow"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Choosing an explodemap workflow}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
```

## The short version

`explodemap` has four main user-facing workflows:

| Goal | Use |
|---|---|
| Explode a projected `sf` object that already has a grouping column | `explode_sf()` |
| Join a lookup table before exploding | `explode_sf_with_lookup()` |
| Download and explode U.S. county subdivisions by state | `explode_state()` |
| Separate larger region blocks for national or multi-region layouts | `explode_grouped()` |
| Add interactive selected-area zoom and information cards | `focus_map()` |

The best default is usually `explode_sf()`: prepare a projected `sf`
object, add a region column, and let the package derive parameters from
the data geometry.

## Static exploded maps

Use `explode_sf()` when you already have polygon data and a grouping
column:

```{r eval=FALSE}
result <- explode_sf(
  my_sf,
  region_col = "region",
  plot = FALSE,
  quiet = TRUE
)
```

Use `explode_sf_with_lookup()` when group membership lives in a separate
table:

```{r eval=FALSE}
result <- explode_sf_with_lookup(
  my_sf,
  join_col = "GEOID",
  lookup = region_lookup,
  lookup_key = "geoid",
  region_col = "region",
  plot = FALSE,
  quiet = TRUE
)
```

Use `explode_state()` when you want the package to download U.S. Census
TIGER/Line county subdivision data for a state:

```{r eval=FALSE}
result <- explode_state(
  state_fips = "34",
  crs = 32118,
  region_map = nj_regions,
  plot = FALSE,
  quiet = TRUE
)
```

Downloaded data is cached by `tigris`, so repeated runs are faster.

## Dense municipal cores

The core two-level model is closed-form and deterministic. For very
dense municipal cores, you may still see residual crowding after the
analytical displacement. In that case, add the optional bounded
refinement pass:

```{r eval=FALSE}
result <- explode_sf(
  my_sf,
  region_col = "region",
  refine = TRUE,
  refine_min_gap = 250,
  refine_max_shift = 150,
  plot = FALSE
)
```

Use `refine = TRUE` as a finishing pass, not as the first tuning knob.
The most important parameter is `refine_max_shift`, which caps the extra
correction per feature. Start small and increase only when dense cores
still touch or visually crowd.

The default `refine_within = "region"` only refines pairs that share the
same parent region. Use `refine_within = "all"` when the remaining
crowding crosses region boundaries.

## Grouped layouts

Use `explode_grouped()` when the map needs an additional layout level,
such as a national map with multiple region blocks:

```{r eval=FALSE}
grouped <- explode_grouped(
  my_sf,
  region_col = "hhs_region",
  mode = "auto_collision",
  plot = FALSE,
  quiet = TRUE
)
```

The grouped workflow has three modes:

| Mode | Use when |
|---|---|
| `"auto"` | Radial anchor placement is enough |
| `"auto_collision"` | Region blocks need collision-aware anchor refinement |
| `"manual"` | You want to provide anchor coordinates yourself |

## Interactive focus maps

Use `focus_map()` when the goal is exploration in Shiny or htmlwidgets,
not a new static geometry layer. It lets users click or select a feature,
zoom it into a readable viewport target, and show contextual fields in a
non-blocking information card:

```{r eval=FALSE}
focus_map(
  result,
  group_col = "region",
  group_palette = c(North = "#4C78A8", Central = "#F58518", South = "#54A24B"),
  info_cols = c("NAME", "GEOID"),
  focus_size = 0.76,
  focus_padding = 40,
  lift_scale = 1.16,
  info_card_scale = 1.2
)
```

For Shiny:

```{r eval=FALSE}
ui <- fluidPage(
  focusmapOutput("map", height = "700px")
)

server <- function(input, output, session) {
  result <- explode_sf(munis, "region", plot = FALSE, quiet = TRUE)

  output$map <- renderFocusmap({
    focus_map(
      result,
      label_col = "NAME",
      id_col = "GEOID",
      group_col = "region",
      group_palette = c(North = "#4C78A8", Central = "#F58518", South = "#54A24B"),
      info_cols = c("GEOID", "population")
    )
  })
}
```

The Shiny widget publishes selections at `input$map_selected`. Use that
reactive value to drive side panels, tables, or linked charts.

For region or category filters, `explode_section()` can explode only the
selected section while keeping the rest of the layer as geographic context:

```{r eval=FALSE}
focused <- explode_section(
  munis,
  section_col = "nj_region",
  section = input$region,
  region_col = "county_name",
  alpha_r = 900,
  alpha_l = 600,
  plot = FALSE,
  quiet = TRUE
)

focus_map(
  focused,
  label_col = "NAME",
  id_col = "GEOID",
  context_col = ".explodemap_role",
  context_mode = "fade",
  context_opacity = 0.16,
  performance_mode = TRUE
)
```

This keeps focused sections compact while preserving the full map outline
as context. Use `context_mode = "hide"` when the background should be
available to the layout but not visible in the widget.

## Performance guidance

Before exploding or focusing:

- Use a projected CRS for displacement workflows.
- Use `plot = FALSE` inside apps or scripts when you only need returned
  objects.
- Use `quiet = TRUE` inside app server code to suppress progress messages.
- Use `focus_map(performance_mode = TRUE)` for dense municipal layers.
- Simplify for web display when the browser becomes the bottleneck.
- Keep downloaded TIGER/Line examples in scripts, vignettes with
  `eval = FALSE`, or `inst/examples/`, not in CRAN-running examples.
- Use TopoJSON or GeoJSON export for deployment workflows rather than
  recomputing geometry in the app server.

## Installed examples

Small local examples:

```{r eval=FALSE}
source(system.file("examples/basic_explode_sf.R", package = "explodemap"))
source(system.file("examples/collision_refinement.R", package = "explodemap"))
source(system.file("examples/lookup_workflow.R", package = "explodemap"))
source(system.file("examples/manual_parameter_tuning.R", package = "explodemap"))
```

Interactive examples that download public data:

```{r eval=FALSE}
if (interactive()) {
  shiny::runApp(system.file("examples/focusmap_munis_app.R", package = "explodemap"))
  shiny::runApp(system.file("examples/focusmap_counties_app.R", package = "explodemap"))
}
```

---
title: "Grouped layouts and anchor placement"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Grouped layouts and anchor placement}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
```

## Overview

`explode_grouped()` extends the core two-level exploded-map workflow
with a three-level hierarchy for multi-region or national-scale layouts.

This grouped extension is most useful when a standard two-level
explosion still leaves region blocks visually crowded or difficult to
compare across a larger spatial extent.

The three levels are:

1. **Level 1 — Local explosion.** Units within each parent region are
   displaced using the centroid-driven field from the core algorithm.
   The geometric guarantees of Propositions 1--3 apply at this level.

2. **Level 2 — Radial anchor placement.** Parent region centroids are
   displaced radially outward from the national centroid to generate
   initial anchor positions for each region block.

3. **Level 3 — Collision-aware refinement** (optional). Overlapping
   anchors are iteratively repelled while a spring term maintains
   proximity to the original radial targets.

Levels 2 and 3 preserve structural grouping and directional
correspondence rather than topological coverage. The formal geometric
guarantees of Propositions 1--3 apply strictly at Level 1.

```{r setup}
library(sf)
library(explodemap)
```

## A synthetic grouped example

We create a small dataset with six units in three regions, spread across
a wider spatial extent than the two-region example in
`vignette("getting-started")`.

```{r synthetic-grouped}
sq <- function(xmin, ymin, size = 1000) {
  st_polygon(list(matrix(
    c(xmin, ymin,
      xmin + size, ymin,
      xmin + size, ymin + size,
      xmin, ymin + size,
      xmin, ymin),
    ncol = 2,
    byrow = TRUE
  )))
}

geom <- st_sfc(
  sq(0, 0), sq(3000, 0),       # R1
  sq(9000, 0), sq(12000, 0),   # R2
  sq(24000, 0), sq(27000, 0),  # R3
  crs = 3857
)

x <- st_sf(
  id     = paste0("u", 1:6),
  region = c("R1", "R1", "R2", "R2", "R3", "R3"),
  geometry = geom
)
```

## Anchor modes

`explode_grouped()` supports three anchor placement modes:

| Mode | Description |
|------|-------------|
| `"auto"` | Level 2 only — radial placement without collision resolution |
| `"auto_collision"` | Level 2 + Level 3 — radial placement with iterative refinement |
| `"manual"` | User-supplied anchor coordinates |

## Automatic grouped layout

The simplest call uses `mode = "auto"`:

```{r auto-mode}
g_auto <- explode_grouped(
  x,
  region_col = "region",
  mode       = "auto",
  plot       = FALSE,
  quiet      = TRUE
)

class(g_auto)
print(g_auto)
```

The result is a `grouped_exploded_map` S3 object, which also inherits
from `exploded_map`:

```{r auto-names}
names(g_auto)
```

Plot the grouped layout:

```{r auto-plot}
plot(g_auto)
```

## Collision-aware grouped layout

`mode = "auto_collision"` adds Level 3 refinement. Overlapping region
blocks are iteratively repelled while a spring term pulls them back
toward their radial targets:

```{r collision-mode}
g_collide <- explode_grouped(
  x,
  region_col = "region",
  mode       = "auto_collision",
  plot       = FALSE,
  quiet      = TRUE
)

plot(g_collide)
```

The anchor table reports the resulting block radii and coordinates:

```{r convergence}
g_collide$anchors[, c("region", "block_radius", "anchor_x", "anchor_y")]
```

## Viewing all stages

`plot(g, "all")` shows the original, locally exploded, and grouped
layouts side by side:

```{r all-views, fig.width = 10, fig.height = 4}
plot(g_collide, "all")
```

## Inspecting anchor positions

`layout_regions()` computes anchor positions as a standalone step, which
is useful for custom workflows or manual adjustment:

```{r layout-regions}
anchors <- layout_regions(
  x,
  region_col = "region",
  mode       = "auto",
  quiet      = TRUE
)

anchors[, c("region", "block_radius", "n_units", "anchor_x", "anchor_y")]
```

## Manual anchors

You can edit anchor positions and pass them back into
`explode_grouped()`:

```{r manual-anchors}
manual_anchors <- anchors
manual_anchors$anchor_x <- manual_anchors$anchor_x + c(0, 500, 1000)
manual_anchors$anchor_y <- manual_anchors$anchor_y + c(0, 250, 500)

g_manual <- explode_grouped(
  x,
  region_col = "region",
  mode       = "manual",
  anchors    = manual_anchors,
  plot       = FALSE,
  quiet      = TRUE
)

plot(g_manual)
```

## Key parameters

Level 1 parameters control local displacement within regions:

| Parameter | Default | Purpose |
|-----------|---------|---------|
| `alpha_l` | derived | Local expansion magnitude (metres) |
| `p` | 1.25 | Distance-scaling exponent |
| `gamma_l` | 1.136 | Local clearance coefficient (used if `alpha_l` is NULL) |

Level 2 and Level 3 parameters control anchor placement and refinement:

| Parameter | Default | Purpose |
|-----------|---------|---------|
| `kappa` | 1.8 | Radial expansion factor |
| `padding` | 50000 | Base padding (map units) |
| `delta` | 15000 | Log-density scaling factor |
| `lambda` | 0.18 | Spring coefficient for refinement |
| `eta` | 0.18 | Repulsion step size for refinement |
| `padding_sep` | 20000 | Minimum separation between blocks |

For real-world data, these defaults are tuned for national-scale U.S.
layouts such as states grouped into larger reporting regions. For
smaller or larger extents, adjust `padding`, `delta`, and
`padding_sep` to match your coordinate system and visual scale.

## Summary

```{r summary}
summary(g_collide)
```

## Interactive grouped focus maps

Grouped results can be passed directly to `focus_map()`. The widget
automatically uses `sf_grouped_wgs`, so the displayed geometry matches
the grouped layout:

```{r grouped-focus-map, eval=FALSE}
focus_map(
  g_collide,
  label_col = "id",
  id_col = "id",
  group_col = "region",
  info_cols = c("region"),
  performance_mode = TRUE
)
```

Inside Shiny, pair `explode_grouped(..., quiet = TRUE, plot = FALSE)`
with `renderFocusmap()` to avoid console noise and accidental plot
printing.

## What the grouped object stores

| Field | Contents |
|-------|----------|
| `sf_orig` | Original input geometries |
| `sf_local` | Geometries after Level 1 local explosion |
| `sf_grouped` | Final geometries after anchor displacement |
| `sf_grouped_wgs` | Final geometries in WGS84 (EPSG:4326) |
| `stats` | Geometry statistics from `compute_stats()` |
| `params` | All parameters used |
| `anchors` | Anchor table with block radii and positions |
| `plots` | ggplot objects for original, local, and grouped layouts |
| `diagnostics` | Label, region column, centroid mode, and anchor mode |

## Scope of guarantees

The local explosion stage (Level 1) preserves the package's core
geometric guarantees: exact feature-level geometry preservation
(Proposition 1), radial ordering within regions (Proposition 2), and
bounded displacement (Proposition 3).

The grouped anchor stage (Levels 2--3) is a higher-level layout
extension. It preserves grouping structure and directional
correspondence rather than topological coverage.

## Next steps

For the core two-level workflow and parameter derivation, see
`vignette("getting-started")`.

For full paper-scale examples including cross-state calibration, Canada,
and HHS grouped layouts, see
`vignette("reproducing-paper-examples")`.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tiger.R
\name{cache_list}
\alias{cache_list}
\title{List cached datasets}
\usage{
cache_list()
}
\value{
Invisibly returns a character vector of cached \code{.rds} file names.
The visible output is a message listing the cache directory and file sizes,
or a message that the cache is empty.
}
\description{
List cached datasets
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core.R
\name{validate_input}
\alias{validate_input}
\title{Validate inputs before explosion}
\usage{
validate_input(sf_obj, region_col, allow_other = FALSE, fix_invalid = TRUE)
}
\arguments{
\item{sf_obj}{sf object to validate}

\item{region_col}{Name of the grouping column}

\item{allow_other}{If TRUE, permits units labelled "Other"}

\item{fix_invalid}{If TRUE, attempts st_make_valid() on invalid geometries}
}
\value{
The (possibly repaired) sf object, invisibly
}
\description{
Checks CRS, empty geometries, invalid geometries, region count, and
unmatched "Other" units. Optionally repairs invalid geometries.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tiger.R
\name{.attach_regions_tiger}
\alias{.attach_regions_tiger}
\title{Attach region labels to TIGER/Line data via county names}
\usage{
.attach_regions_tiger(sf_obj, fips, region_map, quiet = FALSE)
}
\description{
Attach region labels to TIGER/Line data via county names
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/export.R
\name{export_topojson}
\alias{export_topojson}
\title{Export an exploded map as TopoJSON}
\usage{
export_topojson(x, file, simplify = NULL, overwrite = FALSE)
}
\arguments{
\item{x}{An \code{sf}, \code{exploded_map}, or \code{grouped_exploded_map} object.
For \code{exploded_map} objects, the WGS84 exploded geometry (\code{sf_exp_wgs})
is exported. For \code{grouped_exploded_map} objects, the WGS84 grouped
geometry (\code{sf_grouped_wgs}) is exported.}

\item{file}{Output file path. Should end in \code{.topojson} or \code{.json}.}

\item{simplify}{Optional simplification proportion passed to
\code{mapshaper -simplify}. Must be a single number between 0 and 1
(exclusive). For example, \code{simplify = 0.5} retains 50\% of vertices.
Default is \code{NULL} (no simplification). Note that simplification
modifies polygon geometry and therefore breaks the exact geometry
preservation guarantee of Proposition 1.}

\item{overwrite}{Logical; if \code{TRUE}, overwrite \code{file} if it already
exists. Default is \code{FALSE}.}
}
\value{
Invisibly returns the output file path.
}
\description{
Converts an \code{sf}, \code{exploded_map}, or \code{grouped_exploded_map} object to
TopoJSON using the external \code{mapshaper} command-line tool. The input is
first written as a temporary GeoJSON file, then converted by mapshaper.
}
\details{
This is intended as a convenience helper for downstream tools such as
Power BI, D3.js, and Observable that prefer or require TopoJSON input.

This function requires the external \code{mapshaper} command-line tool to be
installed and available on the system path. It can be installed with:

\preformatted{npm install -g mapshaper}

The \code{mapshaper} tool is not an R package dependency; it is invoked via
\code{system2()}. If mapshaper is not found, the function errors with an
informative message.
}
\examples{
\dontrun{
# Two-level export
result <- explode_sf(my_sf, region_col = "region", plot = FALSE)
export_topojson(result, "exploded.topojson")

# Three-level grouped export
grp <- explode_grouped(my_sf, region_col = "region", plot = FALSE)
export_topojson(grp, "grouped.topojson")

# With simplification (breaks Proposition 1 guarantee)
export_topojson(result, "simplified.topojson", simplify = 0.5)

# Raw sf object
export_topojson(my_sf, "raw.topojson", overwrite = TRUE)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/focusmap.R
\name{focus_map}
\alias{focus_map}
\alias{focusmapOutput}
\alias{renderFocusmap}
\title{Interactive focus-map viewer}
\usage{
focus_map(
  x,
  label_col = NULL,
  id_col = NULL,
  group_col = NULL,
  group_palette = NULL,
  context_col = NULL,
  context_values = "context",
  context_mode = c("fade", "hide", "show"),
  context_fill = "#cfd9df",
  context_opacity = 0.18,
  context_clickable = FALSE,
  focus_preset = c("none", "municipal", "drilldown", "municipal_drilldown"),
  simplify = TRUE,
  fill = "#2d6ea3",
  fill_opacity = 0.58,
  stroke = "#ffffff",
  lift_scale = 1.16,
  focus_padding = 40,
  focus_size = 0.76,
  min_focus_width = 0,
  min_focus_height = 0,
  tiny_feature_threshold = 48,
  tiny_feature_boost = 1,
  max_zoom = NULL,
  origin_context = c("none", "socket", "inset", "both"),
  origin_context_position = c("bottom-left", "bottom-right", "top-left", "top-right"),
  focus_context_opacity = 0.3,
  show_drag_zoom = FALSE,
  font_size = 14,
  show_labels = TRUE,
  show_sidebar = TRUE,
  performance_mode = NULL,
  info_cols = NULL,
  info_labels = NULL,
  info_title = NULL,
  info_position = c("top-right", "top-left", "bottom-right", "bottom-left"),
  info_card_scale = 1,
  area_min = 5000,
  width_min = 95,
  height_min = 28,
  width = "100\%",
  height = "600px",
  elementId = NULL
)

focusmapOutput(outputId, width = "100\%", height = "600px")

renderFocusmap(expr, env = parent.frame(), quoted = FALSE)
}
\arguments{
\item{x}{An \code{sf}, \code{exploded_map}, or
\code{grouped_exploded_map} object.}

\item{label_col}{Character. Column name for polygon labels.
Auto-detected if \code{NULL}.}

\item{id_col}{Optional stable feature ID column for Shiny selection events.
Defaults to row order if \code{NULL}.}

\item{group_col}{Character. Optional column for region/group
colouring. Polygons sharing a group value share a hue.}

\item{group_palette}{Optional named character vector of colours for
\code{group_col} values. Names should match group values; unmatched groups
fall back to the widget palette.}

\item{context_col}{Optional column identifying features that should remain
as geographic context rather than active focus features.}

\item{context_values}{Character vector of values in \code{context_col} that mark
context features. Default \code{"context"}.}

\item{context_mode}{How context features are drawn: \code{"fade"} keeps them
visible but muted, \code{"hide"} makes them invisible, and \code{"show"} draws them
normally.}

\item{context_fill}{Fill colour for context features when
\code{context_mode = "fade"}.}

\item{context_opacity}{Fill opacity for faded context features.}

\item{context_clickable}{Should context features remain clickable? Default
\code{FALSE}.}

\item{focus_preset}{Optional named preset for common interactive workflows.
\code{"municipal"} tunes small-area focus, source cues, drag zoom, and dense
layer performance. \code{"drilldown"} tunes context fading and source cues for
selected-section maps. \code{"municipal_drilldown"} combines both. Explicit
arguments supplied by the user override preset defaults.}

\item{simplify}{Controls geometry simplification for rendering
performance. \code{TRUE} (default) applies a sensible tolerance
(\code{dTolerance = 0.001} in WGS 84 degrees, \eqn{\approx}
100 m). A positive number sets a custom tolerance. \code{FALSE}
disables simplification. Only affects the widget copy — the
original data is never modified.}

\item{fill}{Fill colour (used when \code{group_col} is \code{NULL}).
Default \code{"#2d6ea3"}.}

\item{fill_opacity}{Fill opacity. Default \code{0.58}.}

\item{stroke}{Stroke colour. Default \code{"#ffffff"}.}

\item{lift_scale}{Initial toast lift scale. Default \code{1.16}.
Increase this to make the lifted feature larger.}

\item{focus_padding}{Extra screen-space padding in pixels around the lifted
feature during focus. Increase this if large lifted features feel too
close to the map edge.}

\item{focus_size}{Target fraction of the map viewport the lifted feature
may occupy. Increase this to make selected areas appear larger while
preserving \code{focus_padding}.}

\item{min_focus_width, min_focus_height}{Minimum focused feature width and
height in screen pixels. When a selected feature is very small, the widget
may zoom past the usual density-aware default until the lifted feature
reaches these dimensions. Set to \code{0} to disable either constraint.}

\item{tiny_feature_threshold}{Screen-pixel size below which a selected
feature receives an adaptive lift-scale boost. Set to \code{0} to disable.}

\item{tiny_feature_boost}{Maximum multiplier applied to \code{lift_scale}
for the smallest features. Values below \code{1} are not allowed.}

\item{max_zoom}{Optional maximum camera zoom. If \code{NULL}, a density-aware
default is used.}

\item{origin_context}{How the selected feature's source location should be
shown while focused. The default \code{"none"} keeps focus maps visually
unchanged unless this feature is explicitly enabled. \code{"socket"} keeps
the source outline in the main
map, \code{"inset"} shows a small overview map, \code{"both"} uses both,
and \code{"none"} disables the cue.}

\item{origin_context_position}{Position for the overview inset:
\code{"bottom-left"}, \code{"bottom-right"}, \code{"top-left"}, or
\code{"top-right"}.}

\item{focus_context_opacity}{Fill opacity for non-selected features while a
feature is focused. Lower values make tiny selected areas easier to read.}

\item{show_drag_zoom}{Show a widget-level drag-zoom toggle. When enabled,
users can draw a marquee rectangle to zoom into dense clusters while
ordinary feature clicks continue to focus the map. Shift-drag works as a
shortcut even when the button is hidden.}

\item{font_size}{Label font size in px. Default \code{14}.}

\item{show_labels}{Show labels on lifted shapes? Default \code{TRUE}.}

\item{show_sidebar}{Deprecated and has no effect. Will be removed in a
future version.}

\item{performance_mode}{Logical or \code{NULL}. If \code{NULL}, dense
layers automatically use shorter camera transitions and lighter
in-flight rendering. Set \code{TRUE} to force it or \code{FALSE} to
disable it.}

\item{info_cols}{Optional character vector of columns to show in a
non-blocking focus card when a feature is selected.}

\item{info_labels}{Optional named character vector or list for display
labels in the focus card. Names should match \code{info_cols}.}

\item{info_title}{Optional column to use as the focus card title. Defaults
to \code{label_col}.}

\item{info_position}{Position for the focus card: \code{"top-right"},
\code{"top-left"}, \code{"bottom-right"}, or \code{"bottom-left"}.}

\item{info_card_scale}{Relative size for the focus card. Values above
\code{1} make the card larger; values below \code{1} make it more compact.}

\item{area_min}{Min screen area (px^2) for label visibility.}

\item{width_min}{Min screen width (px) for label visibility.}

\item{height_min}{Min screen height (px) for label visibility.}

\item{width}{Widget width. Default \code{"100\%"}.}

\item{height}{Widget height. Default \code{"600px"}.}

\item{elementId}{Optional element ID.}

\item{outputId}{Shiny output ID.}

\item{expr}{Expression that returns a \code{focus_map()} widget.}

\item{env}{Environment in which to evaluate \code{expr}.}

\item{quoted}{Logical. Is \code{expr} already quoted?}
}
\value{
An \code{htmlwidgets} object.
}
\description{
Renders spatial features as a smooth, interactive SVG map. Click any
polygon to zoom in and lift it from the map with a "toast" effect;
right-click or press Escape to reset. Camera transitions use D3's
optimal zoom interpolation for fluid 60 fps motion with zero server
round-trips.
}
\details{
Accepts raw \code{sf} objects, \code{exploded_map} results from
\code{\link{explode_sf}}, or \code{grouped_exploded_map} results from
\code{\link{explode_grouped}}. For exploded objects, the displaced
(WGS 84) geometry is used automatically.
}
\examples{
\donttest{
poly <- function(xmin, ymin, xmax, ymax) {
  sf::st_polygon(list(rbind(
    c(xmin, ymin), c(xmax, ymin), c(xmax, ymax),
    c(xmin, ymax), c(xmin, ymin)
  )))
}

counties <- sf::st_sf(
  NAME = c("A", "B"),
  region = c("North", "South"),
  geometry = sf::st_sfc(
    poly(-74.2, 40.0, -74.0, 40.2),
    poly(-73.9, 40.0, -73.7, 40.2),
    crs = 4326
  )
)

focus_map(counties, label_col = "NAME", group_col = "region")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{.make_plots}
\alias{.make_plots}
\title{Build standard original/exploded ggplot pair}
\usage{
.make_plots(sf_orig, sf_exp, region_col, title, params)
}
\description{
Build standard original/exploded ggplot pair
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{print.grouped_exploded_map}
\alias{print.grouped_exploded_map}
\title{Print a grouped_exploded_map object}
\usage{
\method{print}{grouped_exploded_map}(x, ...)
}
\arguments{
\item{x}{A \code{grouped_exploded_map} object}

\item{...}{Ignored}
}
\value{
Invisibly returns \code{x}, the original \code{grouped_exploded_map} object.
The method is called for its side effect of printing a compact diagnostic
overview of units, regions, grouping mode, and grouped-layout parameters.
}
\description{
Print a grouped_exploded_map object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/section.R
\name{explode_section}
\alias{explode_section}
\title{Explode one selected section and keep the rest as context}
\usage{
explode_section(
  sf_obj,
  section_col,
  section,
  region_col = section_col,
  layout = c("explode", "grouped"),
  context = c("fade", "hide", "none"),
  role_col = ".explodemap_role",
  all_values = "all",
  ...
)
}
\arguments{
\item{sf_obj}{Projected \code{sf} object.}

\item{section_col}{Column containing the high-level sections users choose
from, such as \code{"North"}, \code{"Central"}, and \code{"South"}.}

\item{section}{Selected section value. Values in \code{all_values} explode the
full layer.}

\item{region_col}{Column used for the explosion inside the selected section.
Defaults to \code{section_col}. For municipality drill-downs, this is often a
county column.}

\item{layout}{\code{"explode"} for \code{\link[=explode_sf]{explode_sf()}} or \code{"grouped"} for
\code{\link[=explode_grouped]{explode_grouped()}}.}

\item{context}{\code{"fade"} or \code{"hide"} keeps non-selected features in the
returned object as context; \code{"none"} drops them.}

\item{role_col}{Name of the role column added to the output. Focus features
are marked \code{"focus"} and context features are marked \code{"context"}.}

\item{all_values}{Values that mean "all sections". Default \code{"all"}.}

\item{...}{Passed to \code{\link[=explode_sf]{explode_sf()}} or \code{\link[=explode_grouped]{explode_grouped()}}.}
}
\value{
An \code{exploded_map} or \code{grouped_exploded_map} object with recombined
focus/context geometry and extra diagnostics.
}
\description{
\code{explode_section()} is a dashboard-oriented helper for exploratory maps.
It applies an exploded layout only to the requested section, then recombines
the untouched remainder of the layer as geographic context. The result can
be passed directly to \code{\link[=focus_map]{focus_map()}} with \code{context_col} to fade or hide the
non-selected features.
}
\examples{
\donttest{
poly <- function(xmin, ymin, xmax, ymax) {
  sf::st_polygon(list(rbind(
    c(xmin, ymin), c(xmax, ymin), c(xmax, ymax),
    c(xmin, ymax), c(xmin, ymin)
  )))
}

municipalities <- sf::st_sf(
  NAME = c("A", "B", "C", "D"),
  nj_region = c("South", "South", "North", "North"),
  county_name = c("Atlantic", "Cape May", "Bergen", "Hudson"),
  geometry = sf::st_sfc(
    poly(0, 0, 1000, 1000),
    poly(2000, 0, 3000, 1000),
    poly(0, 2000, 1000, 3000),
    poly(2000, 2000, 3000, 3000),
    crs = 3857
  )
)

focused <- explode_section(
  municipalities,
  section_col = "nj_region",
  section = "South",
  region_col = "county_name",
  alpha_r = 1800,
  alpha_l = 1200,
  plot = FALSE,
  quiet = TRUE
)

focus_map(
  focused,
  label_col = "NAME",
  context_col = ".explodemap_role",
  context_mode = "fade"
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{calibration_row}
\alias{calibration_row}
\title{Extract calibration row from an exploded_map}
\usage{
calibration_row(x)
}
\arguments{
\item{x}{An \code{exploded_map} object}
}
\value{
A one-row data.frame
}
\description{
Returns a one-row data.frame suitable for binding across datasets
to build a calibration table.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grouped.R
\name{.make_grouped_plots}
\alias{.make_grouped_plots}
\title{Build plots for grouped layouts}
\usage{
.make_grouped_plots(sf_orig, sf_local, sf_grouped, region_col, title, params)
}
\description{
Build plots for grouped layouts
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/export.R
\name{.as_export_sf}
\alias{.as_export_sf}
\title{Normalize input to an export-ready sf object in WGS84}
\usage{
.as_export_sf(x)
}
\arguments{
\item{x}{An sf, exploded_map, or grouped_exploded_map object}
}
\value{
sf object
}
\description{
Normalize input to an export-ready sf object in WGS84
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core.R
\name{explode_sf_core}
\alias{explode_sf_core}
\title{Core displacement algorithm (Algorithm 1 from the paper)}
\usage{
explode_sf_core(
  sf_obj,
  region_col,
  alpha_r,
  alpha_l,
  p = 1.25,
  centroid_fun = c("centroid", "point_on_surface")
)
}
\arguments{
\item{sf_obj}{Projected sf object with region column}

\item{region_col}{Grouping column name}

\item{alpha_r}{Regional separation magnitude (metres)}

\item{alpha_l}{Local expansion magnitude (metres)}

\item{p}{Distance scaling exponent (default 1.25)}

\item{centroid_fun}{"centroid" (default) or "point_on_surface"}
}
\value{
Exploded sf object (same CRS as input)
}
\description{
Applies the hierarchical centroid-driven displacement field to every polygon.
Each unit receives rigid-body translation t_i = alpha_r * dhat_state + alpha_l * s_i * dhat_local.
}
\details{
Propositions 1-3 guarantee: exact geometry preservation, radial ordering, and
bounded displacement (||t_i|| <= alpha_r + alpha_l).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{summary.exploded_map}
\alias{summary.exploded_map}
\title{Summary of an exploded_map object}
\usage{
\method{summary}{exploded_map}(object, ...)
}
\arguments{
\item{object}{An \code{exploded_map} object}

\item{...}{Ignored}
}
\value{
Invisibly returns \code{object}, the original \code{exploded_map} object. The
method is called for its side effect of printing a human-readable summary
of dataset size, geometry statistics, displacement parameters, refinement
diagnostics, and implied calibration coefficients.
}
\description{
Summary of an exploded_map object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grouped.R
\name{estimate_block_radii}
\alias{estimate_block_radii}
\title{Estimate block radius for each region}
\usage{
estimate_block_radii(
  sf_obj,
  region_col,
  quantile_p = 0.85,
  centroid_fun = c("centroid", "point_on_surface")
)
}
\arguments{
\item{sf_obj}{Projected sf object with region column}

\item{region_col}{Grouping column name}

\item{quantile_p}{Quantile for radius estimation (default 0.85)}

\item{centroid_fun}{"centroid" or "point_on_surface"}
}
\value{
data.frame with columns: region, block_radius, cx, cy, n_units
}
\description{
Computes the 85th percentile of distances from child-unit centroids to
their region centroid. The 85th percentile is preferred over the maximum
because the maximum is sensitive to outlier units and produces
over-conservative block radii.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tiger.R
\name{cache_clear}
\alias{cache_clear}
\title{Clear explodemap download cache}
\usage{
cache_clear(key = NULL)
}
\arguments{
\item{key}{Specific cache key to clear, or NULL to clear all}
}
\value{
Invisibly returns \code{NULL}. The function is called for its side effect
of deleting cached \code{.rds} files and reporting what was removed.
}
\description{
Clear explodemap download cache
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/presets.R
\name{focus_map_preset}
\alias{focus_map_preset}
\title{Focus-map option presets}
\usage{
focus_map_preset(
  name = c("municipal", "drilldown", "municipal_drilldown", "none")
)
}
\arguments{
\item{name}{Preset name. \code{"municipal"} tunes small-area focus maps with
adaptive sizing, source cues, drag zoom, and dense-layer performance.
\code{"drilldown"} tunes selected-section maps with faded context and source
cues. \code{"municipal_drilldown"} combines both.}
}
\value{
A named list of \code{\link[=focus_map]{focus_map()}} arguments.
}
\description{
Returns a named list of \code{\link[=focus_map]{focus_map()}} arguments for common interactive
mapping workflows. Presets are intentionally plain lists so they can be
inspected, modified, or passed through \code{do.call()}.
}
\examples{
focus_map_preset("municipal")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api.R
\name{explode_sf_with_lookup}
\alias{explode_sf_with_lookup}
\title{Explode any sf object using an external lookup table}
\usage{
explode_sf_with_lookup(
  sf_obj,
  join_col,
  lookup,
  lookup_key = join_col,
  region_col = "region",
  quiet = FALSE,
  ...
)
}
\arguments{
\item{sf_obj}{Projected sf object}

\item{join_col}{Column in sf_obj to join on}

\item{lookup}{data.frame with join key and region column}

\item{lookup_key}{Column name in lookup matching join_col}

\item{region_col}{Column name in lookup containing region labels}

\item{quiet}{If \code{TRUE}, suppress \code{message()} output. Default \code{FALSE}.
Passed through to \code{\link[=explode_sf]{explode_sf()}}.}

\item{...}{Passed to \code{\link[=explode_sf]{explode_sf()}}}
}
\value{
An \code{exploded_map} S3 object
}
\description{
Joins the lookup to \code{sf_obj} before exploding. Unmatched units are
labelled "Other".
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/focusmap.R
\name{.sf_to_geojson_fast}
\alias{.sf_to_geojson_fast}
\title{Fast bulk GeoJSON via a single sf::st_write call}
\usage{
.sf_to_geojson_fast(
  sf_obj,
  label_col,
  id_col = NULL,
  group_col = NULL,
  context_col = NULL,
  info_cols = NULL,
  info_title = label_col
)
}
\description{
Writes the whole sf to a temp GeoJSON file and passes the raw JSON
string to the widget. The JS side does \code{JSON.parse()}, which
avoids the \code{fromJSON → R list → toJSON} round-trip that can
mangle deeply nested coordinate arrays.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/export.R
\name{.build_mapshaper_args}
\alias{.build_mapshaper_args}
\title{Build mapshaper CLI arguments}
\usage{
.build_mapshaper_args(input, output, simplify = NULL)
}
\arguments{
\item{input}{Path to input GeoJSON file}

\item{output}{Path to output TopoJSON file}

\item{simplify}{Optional simplification proportion in \verb{(0, 1)}.}
}
\value{
Character vector of arguments for system2()
}
\description{
Build mapshaper CLI arguments
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grouped.R
\name{layout_regions}
\alias{layout_regions}
\title{Compute region anchor positions for grouped layouts}
\usage{
layout_regions(
  sf_obj,
  region_col,
  mode = c("auto", "auto_collision", "manual"),
  anchors = NULL,
  kappa = 1.8,
  padding = 50000,
  delta = 15000,
  lambda = 0.18,
  eta = 0.18,
  padding_sep = 20000,
  max_iter = 60,
  quantile_p = 0.85,
  centroid_fun = c("centroid", "point_on_surface"),
  quiet = FALSE
)
}
\arguments{
\item{sf_obj}{Projected sf object with region column}

\item{region_col}{Grouping column name}

\item{mode}{"auto" (radial only), "auto_collision" (radial + solver), or "manual"}

\item{anchors}{For mode = "manual": data.frame with columns (region_col, anchor_x, anchor_y)}

\item{kappa}{Radial expansion factor (default 1.8)}

\item{padding}{Base padding in map units (default 50000)}

\item{delta}{Log-density scaling factor (default 15000)}

\item{lambda}{Spring coefficient for collision solver (default 0.18)}

\item{eta}{Repulsion step size for collision solver (default 0.18)}

\item{padding_sep}{Minimum separation between blocks (default 20000)}

\item{max_iter}{Max iterations for collision solver (default 60)}

\item{quantile_p}{Quantile for block radius estimation (default 0.85)}

\item{centroid_fun}{"centroid" or "point_on_surface"}

\item{quiet}{If \code{TRUE}, suppress \code{message()} output. Default \code{FALSE}.}
}
\value{
data.frame with region, anchor_x, anchor_y, block_radius, n_units
}
\description{
Implements the anchor layout procedure from Section 12.
Supports three modes: automatic radial placement, automatic with
collision resolution, and manual user-specified anchors.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core.R
\name{centroid_geoms}
\alias{centroid_geoms}
\title{Compute centroids without attribute warnings}
\usage{
centroid_geoms(x, centroid_fun = c("centroid", "point_on_surface"))
}
\arguments{
\item{x}{An sf or sfc object}

\item{centroid_fun}{"centroid" (default) or "point_on_surface"}
}
\value{
sfc geometry column of point centroids
}
\description{
Wraps sf::st_centroid / sf::st_point_on_surface, operating on
st_geometry() rather than the full sf object. Returns an sfc column.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api.R
\name{.handle_export}
\alias{.handle_export}
\title{Handle export logic}
\usage{
.handle_export(export, sf_exp_wgs, label)
}
\description{
Handle export logic
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{print.exploded_map}
\alias{print.exploded_map}
\title{Print an exploded_map object}
\usage{
\method{print}{exploded_map}(x, ...)
}
\arguments{
\item{x}{An \code{exploded_map} object}

\item{...}{Ignored}
}
\value{
Invisibly returns \code{x}, the original \code{exploded_map} object. The
method is called for its side effect of printing a compact diagnostic
overview of units, regions, derived parameters, and optional refinement
information.
}
\description{
Print an exploded_map object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tiger.R
\name{.download_cousub}
\alias{.download_cousub}
\title{Download TIGER/Line county subdivision boundaries}
\usage{
.download_cousub(fips, crs)
}
\arguments{
\item{fips}{2-digit state FIPS code}

\item{crs}{Target projected CRS}
}
\value{
sf object
}
\description{
Download TIGER/Line county subdivision boundaries
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/explodemap-package.R
\docType{package}
\name{explodemap-package}
\alias{explodemap}
\alias{explodemap-package}
\title{explodemap: Hierarchical Exploded-View Cartography}
\description{
Internal package declarations for NSE checks.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://prigasg.github.io/explodemap/}
  \item \url{https://github.com/PrigasG/explodemap}
  \item Report bugs at \url{https://github.com/PrigasG/explodemap/issues}
}

}
\author{
\strong{Maintainer}: George Arthur \email{prigasgenthian48@gmail.com}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{summary.grouped_exploded_map}
\alias{summary.grouped_exploded_map}
\title{Summary of a grouped_exploded_map object}
\usage{
\method{summary}{grouped_exploded_map}(object, ...)
}
\arguments{
\item{object}{A \code{grouped_exploded_map} object}

\item{...}{Ignored}
}
\value{
Invisibly returns \code{object}, the original \code{grouped_exploded_map}
object. The method is called for its side effect of printing dataset size,
grouping diagnostics, anchor parameters, and anchor radii.
}
\description{
Summary of a grouped_exploded_map object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grouped.R
\name{.generate_anchors}
\alias{.generate_anchors}
\title{Generate initial radial anchors from the national centroid}
\usage{
.generate_anchors(
  block_df,
  nat_centroid,
  kappa = 1.8,
  padding = 50000,
  delta = 15000
)
}
\description{
Generate initial radial anchors from the national centroid
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grouped.R
\name{explode_grouped}
\alias{explode_grouped}
\title{Create a three-level grouped exploded map}
\usage{
explode_grouped(
  sf_obj,
  region_col,
  mode = c("auto", "auto_collision", "manual"),
  anchors = NULL,
  alpha_l = NULL,
  p = 1.25,
  gamma_l = 1.136,
  kappa = 1.8,
  padding = 50000,
  delta = 15000,
  lambda = 0.18,
  eta = 0.18,
  padding_sep = 20000,
  anchor_expand = NULL,
  anchor_buffer = NULL,
  density_scale = NULL,
  block_sep = NULL,
  max_iter = 60,
  fix_invalid = TRUE,
  centroid_fun = c("centroid", "point_on_surface"),
  plot = TRUE,
  export = NULL,
  label = "Grouped Layout",
  quiet = FALSE
)
}
\arguments{
\item{sf_obj}{Projected sf object with region column}

\item{region_col}{Grouping column name}

\item{mode}{"auto", "auto_collision", or "manual"}

\item{anchors}{For mode = "manual": data.frame with anchor positions}

\item{alpha_l}{Local expansion parameter for Level 1 (metres)}

\item{p}{Distance scaling exponent (default 1.25)}

\item{gamma_l}{Local clearance coefficient (default 1.136); used if alpha_l is NULL}

\item{kappa}{Radial expansion factor (default 1.8)}

\item{padding}{Base padding (default 50000)}

\item{delta}{Log-density scaling (default 15000)}

\item{lambda}{Spring coefficient (default 0.18)}

\item{eta}{Repulsion step (default 0.18)}

\item{padding_sep}{Minimum block separation (default 20000)}

\item{anchor_expand, anchor_buffer, density_scale, block_sep}{Optional aliases
for \code{kappa}, \code{padding}, \code{delta}, and \code{padding_sep}, respectively. These
names are convenient in Shiny dashboards where the controls describe the
visual effect rather than the solver term.}

\item{max_iter}{Max collision iterations (default 60)}

\item{fix_invalid}{Auto-repair invalid geometries (default TRUE)}

\item{centroid_fun}{"centroid" or "point_on_surface"}

\item{plot}{Print plots (default TRUE). Automatically suppressed inside a
live Shiny session; use \code{\link[=plot.grouped_exploded_map]{plot.grouped_exploded_map()}} inside \code{renderPlot()}.}

\item{export}{NULL, TRUE, or file path}

\item{label}{Title for plots}

\item{quiet}{If \code{TRUE}, suppress \code{message()} output. Default \code{FALSE}.}
}
\value{
A \code{grouped_exploded_map} S3 object (inherits from \code{exploded_map})
}
\description{
Combines Level 1 (local explosion within regions) with Level 2/3
(anchor-based region block placement). This is the full three-level
extension from Section 12 of the paper.
}
\details{
The guarantees of Propositions 1-3 apply strictly at Level 1.
Higher levels preserve structural grouping and directional correspondence
rather than topological coverage.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core.R
\name{fmt_dist}
\alias{fmt_dist}
\title{Format a distance in metres as a human-readable km string}
\usage{
fmt_dist(x_m)
}
\arguments{
\item{x_m}{Distance in metres}
}
\value{
Character string
}
\description{
Adapts precision to magnitude: "23.6 km" for large values,
"0.067 km" for small ones, so tiny test geometries don't show "0 km".
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tiger.R
\name{.get_national_counties}
\alias{.get_national_counties}
\title{Download national county name-to-FIPS lookup}
\usage{
.get_national_counties()
}
\description{
Download national county name-to-FIPS lookup
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api.R
\name{explode_sf}
\alias{explode_sf}
\title{Explode any sf object with an existing grouping column}
\usage{
explode_sf(
  sf_obj,
  region_col = "region",
  gamma_r = 3,
  gamma_l = 1.136,
  p = 1.25,
  alpha_r = NULL,
  alpha_l = NULL,
  refine = FALSE,
  refine_min_gap = NULL,
  refine_max_shift = NULL,
  refine_max_iter = 20,
  refine_step = 0.5,
  refine_within = c("region", "all"),
  allow_other = FALSE,
  fix_invalid = TRUE,
  centroid_fun = c("centroid", "point_on_surface"),
  plot = TRUE,
  export = NULL,
  label = "Custom Dataset",
  quiet = FALSE
)
}
\arguments{
\item{sf_obj}{Projected sf object (metric CRS)}

\item{region_col}{Name of the column defining groups}

\item{gamma_r}{Regional clearance coefficient (default 3.0)}

\item{gamma_l}{Local clearance coefficient (default 1.136)}

\item{p}{Distance scaling exponent (default 1.25)}

\item{alpha_r}{Optional manual override for regional separation (metres).
May be supplied independently of \code{alpha_l}.}

\item{alpha_l}{Optional manual override for local expansion (metres).
May be supplied independently of \code{alpha_r}.}

\item{refine}{If TRUE, apply a bounded collision-refinement pass after the
analytical displacement. Default is FALSE.}

\item{refine_min_gap}{Optional minimum boundary-to-boundary gap in map units.
If NULL and \code{refine = TRUE}, uses 2\% of the characteristic diameter.}

\item{refine_max_shift}{Optional maximum correction per feature in map units.
If NULL and \code{refine = TRUE}, uses 10\% of \code{alpha_r + alpha_l}.}

\item{refine_max_iter}{Maximum refinement iterations.}

\item{refine_step}{Fraction of each gap deficit corrected per iteration.}

\item{refine_within}{Refine pairs within each \code{"region"} (default) or across
\code{"all"} features.}

\item{allow_other}{If TRUE, permits "Other" units}

\item{fix_invalid}{If TRUE, auto-repairs invalid geometries}

\item{centroid_fun}{"centroid" (default) or "point_on_surface"}

\item{plot}{Print plots on return. Automatically suppressed when called
inside a live Shiny session; use \code{\link[=plot.exploded_map]{plot.exploded_map()}} inside
\code{renderPlot()} instead.}

\item{export}{NULL, TRUE, or file path}

\item{label}{Title for plots}

\item{quiet}{If \code{TRUE}, suppress all \code{message()} output. Default \code{FALSE}.}
}
\value{
An \code{exploded_map} S3 object
}
\description{
Explode any sf object with an existing grouping column
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{plot.exploded_map}
\alias{plot.exploded_map}
\title{Plot an exploded_map object}
\usage{
\method{plot}{exploded_map}(x, which = c("exploded", "original", "both"), ...)
}
\arguments{
\item{x}{An \code{exploded_map} object}

\item{which}{"exploded" (default), "original", or "both"}

\item{...}{Ignored}
}
\value{
Invisibly returns \code{x}, the original \code{exploded_map} object. The
method is called for its side effect of drawing the stored \code{ggplot2}
original and/or exploded map objects.
}
\description{
Plot an exploded_map object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grouped.R
\name{.refine_anchors}
\alias{.refine_anchors}
\title{Refine anchors with pairwise repulsion plus spring return}
\usage{
.refine_anchors(
  block_df,
  lambda = 0.18,
  eta = 0.18,
  padding_sep = 20000,
  max_iter = 60
)
}
\description{
Refine anchors with pairwise repulsion plus spring return
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/export.R
\name{.find_mapshaper}
\alias{.find_mapshaper}
\title{Locate the mapshaper executable on the system path}
\usage{
.find_mapshaper()
}
\value{
Path to mapshaper executable
}
\description{
Locate the mapshaper executable on the system path
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{plot.grouped_exploded_map}
\alias{plot.grouped_exploded_map}
\title{Plot a grouped_exploded_map object}
\usage{
\method{plot}{grouped_exploded_map}(x, which = c("grouped", "original", "local", "all"), ...)
}
\arguments{
\item{x}{A \code{grouped_exploded_map} object}

\item{which}{"grouped" (default), "original", "local", or "all"}

\item{...}{Ignored}
}
\value{
Invisibly returns \code{x}, the original \code{grouped_exploded_map} object.
The method is called for its side effect of drawing the stored \code{ggplot2}
grouped, original, and/or local map objects.
}
\description{
Plot a grouped_exploded_map object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core.R
\name{derive_params}
\alias{derive_params}
\title{Derive displacement parameters from geometry statistics}
\usage{
derive_params(stats, gamma_r = 3, gamma_l = 1.136, p = 1.25)
}
\arguments{
\item{stats}{Output of \code{\link[=compute_stats]{compute_stats()}}}

\item{gamma_r}{Regional clearance coefficient (default 3.0)}

\item{gamma_l}{Local clearance coefficient (default 1.136)}

\item{p}{Distance scaling exponent (default 1.25)}
}
\value{
Named list with alpha_r, alpha_l, p, gamma_r, gamma_l
}
\description{
Implements Analytical Results 1 and 2 from the paper:
alpha_r = gamma_r * w_bar / (2 * sin(pi / n_regions))
alpha_l = gamma_l * 2 * R_local / sqrt(n_bar)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core.R
\name{compute_stats}
\alias{compute_stats}
\title{Compute geometry statistics for parameter derivation}
\usage{
compute_stats(
  sf_obj,
  region_col,
  centroid_fun = c("centroid", "point_on_surface")
)
}
\arguments{
\item{sf_obj}{Projected sf object}

\item{region_col}{Name of the grouping column}

\item{centroid_fun}{"centroid" (default) or "point_on_surface"}
}
\value{
A named list of geometry statistics
}
\description{
Extracts w_bar, R_local, n_bar, n_regions, and the tightness ratio
from a projected sf object with a grouping column.
}
